;;; CAD to Blender OBJ generator.
;;; Copyright (C) 2012-2023 Paul Kotelevets aka 1D_Inc
;;; forum: http://blenderartists.org/forum/showthread.php?267990-Direct-Autocad-export-to-Blender-Beta
;;; forum: http://blenderartists.org/forum/showthread.php?267990-Direct-Autocad-Intellicad-to-Blender-exporter-LISP-to-Download
;;;
;;; License = GPL v2
;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;;; GNU General Public License for more details. 
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see http://www.gnu.org/licenses/





(setq 1DSETUP 1);; nil



;|
About using: 
0) This file contains an OBJ file generator for cads (tested on AutoCAD 2009, ZWCAD 2012 and Bricscad v12). Creates OBJ file for Blender import.
1) Drag&Drop this file to opened CAD, or use _APPLOAD command to bind it.
2) Type "1D2B" command in commandline, select objects to export.
3) Set LM NT RS DIV values. If default value is suitable, accept it by pressing space.
4) Wait for some time.
5) Generated OBJ file is opened in notepad, and it's path is shown into commandline.
6) Open it in Blender, using default Blender's Obj import.

	Stats for export:
	POINT LWPOLYLINE POLYLINE LINE SPLINE ARC CIRCLE ELLIPSE types are currently supported.
	30 000 geometry objects ~ 370 sec ~ 4Mb obj		-may not to open
	18 000 geometry objects ~ 150 sec ~ 2.5Mb obj		-opens quite well
	1 500 000 point dots selection with NT=15 gives 100 000	-export's too long, but imports excellent

7) This file contains an OBJ file importer for cads (tested on AutoCAD 2009, AutoCAD 2013). Imports OBJ file from Blender export into CAD.
8) WARNING! OBJ file should be exported with following settings:
	Forward =  Y Forward
	Up 	=  Z Up
9) Type "1D2OBJ" command in commandline, select OBJ file to import.
10) Select precision (digits after delimeter to read), scale, Geomerty mode, and UV mode.
	= If enter "13" at geometry mode request , 3D Faces and Points will be created.
	= In Faces mode (enter "1"), wire Blender lines will be also builded as lines, quads and tris - as 3DFaces.
	= For Faces and Lines will be created random - colored Layers from Blender Materials.
	= For UV import all exported objects should have a UV.
	
	Stats for import:
	Bypass Importing is rather expensive. Try import Suzanne at first.
	0.5Mb OBJ tooks 10sec, 2Mb OBJ ~ 50sec at Faces import mode (AutoCAD 2009)
 
Special thanks to Lee Mac ( http://www.lee-mac.com ), a great LISP coder, for his help and brilliant code.


Current version: v5.17
===	Versions History:	===
=	0.97	(30.9.2012)	Created.
=	1.00	(02.10.2012)	CIRCLES, ARCS support providen, public release.
=	1.01    (03.10.2012)	Added output precision control and  bulged polylines support
=	1.01a	(03.10.2012)	Bulges at end of polylines support
=	1.02	(03.10.2012)	Ellipse support, String in obj file's header, Layer num, select filtered
				objects by C:33, object, that caused problems by C:44.
=	1.03	(04.10.2012)	Characters count in commanline, MODEMACRO and file's header.
=	1.04_icad	(05.10.2012)	Intellicads fork (written for ZWCAD 2012).
=	1.05a_icad	(05.10.2012)	Intellicads fork (adopted for Briscad v9).
=	1.05d_icad	(7.10.2012)	Intellicads fork (adopted for Briscad v9)
=	1.07	(08.10.2012)	Collapsing forks. Adopted to AUTOCAD, BRICSCADv12, ZWCAD 2012. 3DFACEs support.
=	2.00	(08.10.2012)	Refactoring, multiple files creation.
=	2.00a	(09.10.2012)	Splitting polyline's arcs on bulgency fix.
=	2.01	(11.10.2012)	Some bad objecttypes support, multiple files creation fix, speed process optimization attempt, 2DSOLID support, closed 2DPOLYLINES.
=	2.02	(16.10.2013) Layer writing ability control.
=	3.00	(25.10.2013) commands 1D2OBJ, OBJJ and 1DZ0 added  (Blender OBJ to CAD import)
=	3.01	(28.10.2013) 1D2OJB added MESH convert and Vertex by layers ability
=	3.02	(29.10.2013) 1D2OBJ YZ -ZY transformations added (Blender's output default)
=	3.03	(30.10.2013) UV import for files, containing not unwrapped objects.
=	3.04	(05.12.2013) X and Y Shifts for 1D2B
=	3.05	(18.02.2014) freely positioned in 3D space (with not '(0.0.1) Normal) LWPOLYLINES support, including bulgency. $$$NOR
=	3.06	(04.12.2014) 2010 MESH support attempt, storing previous settings.
=	3.07	(26.05.2015) MESH output (new 1DM command) and scale, naming output OBJ files (1D2B).
=	3.08	(16.06.2015) Split volumes value influeces speed and RAM overload.
=	3.09	(23.06.2015) RV added + ifile + renumer 44444
= 	4.00	(07.07.2015) 1DFEX, 1DBB, 1DWG, 1DWF, 1DBQ, 1DRQ commands added
=	4.01	(15.07.2015) 1DFEX multiple object types support added, including text. 1DV_EXTRACT, 1DBB implementation
=	4.02	(17.07.2015) 1DFEX and 1DBB objects types support: SPLINE, MESH, TEXT, MTEXT, liear/aligned and angular dimensions
=	4.03	(05.08.2015) 1DFEX and 1DBB recycling closed wireframes, 1DFWIN -1
=	4.04	(15:06 25.08.2015) ARCFIX 1dunf off
=	4.05	(9:53 02.09.2015) 1D_Latin Translit module
=	4.06	(10:04 03.12.2015) 1DBB 1D_Latin Blocknames output 
=	4.07	(9:52 18.05.2016) 1DFEX fixed ARC vertices distribution
=	4.08	(10:32 22.09.2016) 1DWG added autodist closest blocks from plines, changed window sizes, FITZERO arc fix on 1DV_EXTRACT
=	5.00	(18:13 10.07.2017) Unified NO text/dims option (TEXX) for 1DFEX, 1DBB througn 1DV_EXTRACT.
=	5.01	(16:06 19.09.2017) fixed $$1DB export setup/preset system, TEXX inverted.
=	5.02	(17:32 15.03.2018) C:233 C:244
=	5.03	(17:53 28.05.2018) 1DFEX ELLIPSE support
=	5.04    (12:26 23.08.2018) 1DPT $$1DPOINTBASE support
=	5.05	(13:29 23.08.2018) 1DFEX 1DBB POINT object type support
= 	(if (< q 0.2) (setq q 0.9))	;; 20:24 19.03.2018 resolve flat arc
=	5.06	(17:01 11.09.2018) 1DV_EXTRACT_ zeropoint verts lines. 1DL - check for last proceed for 1DFEX 1DBB
= 	5.07	(18:23 04.10.2018) D2 path placer as text added, 1DPT pointer function added 
=	5.08	(3:53 12.10.2018) IUSCALE block scale context
=	5.09	(18:30 27.12.2018) 1D2OBJ import meshes as flat unclosed PLINES support, preset system $$1DOBJ
=	5.10	(5:18 09.01.2019) 1D2OBJ 1d_MkPLINE mode 6 added - elevated plines support, 1DZ0 - flatten selected lines.
=       5.11    (13:44 13.01.2020) 1D2OBJ DBFLIST
=	5.13	(15:33 17.02.2021) DIMTEX size setup
=	5.14	(17:00 29.07.2021) RTD added for AcDb2LineAngularDimension but didnt helped
=	5.15	(21:00 27.12.2021) 1D_LM:GetDynProps added (if (not DS) (setq DS $$DBS))
=	5.16	(18:42 10.01.2022) 1D_VOID3DBW added
=	5.17	(18:34 25.06.2022) ELLIPSE in blocks fix ELLS, INSUNITS = 4, 2dpolyline 1dbb support
=	5.17+	(18:44 16.05.2023) (if (> q 130) (setq q (- q 848))) 1DTXB
	
|;

(vl-load-com)


;;; ======== string format ==========
(defun 1D:Tros (tm prce / )

(if (= (type tm) 'REAL) (setq tm (rtos tm 2 prce)))
(if (= (type tm) 'INT) (setq tm (itoa tm)))

(setq tm (vl-string-translate "," "." tm))

(if (wcmatch tm "*.*")
(progn
(repeat (- prce (strlen (substr tm (+ 2 (vl-string-search "." tm)) 10)))
(setq tm (strcat tm "0"))
)
)
(setq tm (strcat tm ".00"))
)

(setq tm tm); output
)



; converts radians to degrees
 (defun RtD (r) (* 180.0 (/ r pi)))
 
; converts degrees to radians
 (defun DtR (d) (* pi (/ d 180.0)))




;;; http://www.theswamp.org/index.php?topic=55.0
(defun STIMER (/) (setq $$time (getvar "DATE")))
  
(defun GTIMER	( func / mins)
  (setq	secs	(- (getvar "DATE") $$time)
	secs	(* 86400.0 (- secs (fix secs)))
	)
  (gc)

(setq mins (itoa (1+ (fix (/ secs 60.0)) )))
(setq secs (1D:Tros secs 2))

(setq secs (strcat secs "s <" mins "min"));; output data
)








;|[[[[[[[[[[[[[[[[[[
[[[[ ][
by 1D_Inc - DXF parser test
|;
(defun C:][ (/
ent entl ct ent2 ent3 ent4
)
(setq ent (car (entsel "\nSelect object for DXF v5.17:")))
  (setq entl (entget ent)) ; Set entl to association list of  last entity.
(setq ent2 (entget ent '("ACAD")));  DXF  
(setq ent2 (cdadr (assoc -3 ent2)));  Xdata

(setq ent3 (entget ent '("ACADannotative")));  DXF  
(setq ent3 (cdadr (assoc -3 ent3)));  Xdata

(if (assoc 347 entl) (progn (princ "\n\n 347 GROUP found:") (princ (entget (cdr (assoc 347 entl)))) ) )
(if (assoc 330 entl) (progn (princ "\n\n 330 GROUP found:") (princ (entget (cdr (assoc 330 entl)))) ) )
(princ "\n")

  (setq ct 0)              ; Set ct (a counter) to 0.
  (textpage)               ; Switch to the text screen.

(princ "\n \n-- ENTGET DXF xDATA --")

  (repeat (length entl)    ; Repeat for number of members in list:
    (print (nth ct entl))  ; Print a newline, then each list member.
    (setq ct (1+ ct))      ; Increments the counter by one.
  )

(if ent2 (progn
(princ "\n \n-- ACADxDATA --")

  (setq ct 0)              ; Set ct (a counter) to 0.
  (repeat (length ent2)    ; Repeat for number of members in list:
    (print (nth ct ent2))  ; Print a newline, then each list member.
    (setq ct (1+ ct))      ; Increments the counter by one.
  )
)
(princ "\n \n-- ACADxDATA is not defined --")
)

(princ "\n \n  -- ANNOxDATA --")

(if ent3 (progn
  (setq ct 0)              ; Set ct (a counter) to 0.
  (repeat (length ent3)    ; Repeat for number of members in list:
    (print (nth ct ent3))  ; Print a newline, then each list member.
    (setq ct (1+ ct))      ; Increments the counter by one.
  )
)
(princ "\n \n-- ANNOxDATA is not defined --\n")
)

  (princ)                  ; Exit quietly.
)



;|[[[[[[[[[[[[[[[[[[
[[[[ ][P
by 1D_Inc  - ActiveX Dump test
|;
(defun C:][P  (/
MT lw PLIST i MLIST
)
(vl-load-com)
(setq MT (car (entsel)))
(setq lw (vlax-ename->vla-object MT))
(princ "\n Obj type = ")
(princ ( vlax-get lw 'Objectname ))
(princ "\n ======= Base Props v5.17 ====== \n")
(setq PLIST '("Objectname" "Color" "Area" "Length" "ArcLength" "Center" "Elevation" "Normal" "Coordinates") )
(setq MLIST '("vlax-curve-getStartParam" "vlax-curve-getdistatpoint" "vlax-curve-getEndParam" "vlax-curve-getStartParam" "vlax-curve-getpointatdist" "vlax-curve-getPointAtParam" "vlax-curve-getPointAtDist") )

(setq i 1)
(foreach n PLIST
(if (vlax-property-available-p lw (read n))
(progn (if (= (GCD i 4) 4)  (princ "\n[") (princ " | [") )
(princ (strcat (vl-princ-to-string n) "] = " (vl-princ-to-string (vlax-get lw (read n))))) )
(princ (strcat "\n[No " n "]"))
)

(setq i (1+ i))
)

;|
(setq i 1)
(foreach n MLIST
(if (vlax-property-available-p lw (read n))
(progn (if (= (GCD i 4) 4)  (princ "\n[") (princ " | [") )
(princ (strcat (vl-princ-to-string n) "] = " (vl-princ-to-string (vlax-get lw (read n))))) )
(progn (if (= (GCD i 4) 4)  (princ "\n[No") (princ " | [No") ) (princ (strcat n "]")))
)

(setq i (1+ i))
)

(vlax-curve-getPointAtDist lw 200)
|;

(princ "\n\n ======= Dump ====== \n\n")

(vlax-dump-object lw)
(princ "\n ====")
(princ (vlax-write-enabled-p lw))
(textscr)
(princ "\n ====")
(princ (vlax-method-applicable-p
lw 'GetBoundingBox ))
(setq yy (vlax-invoke-method lw 'GetBoundingBox pt1 pt2))
(princ yy)
(princ pt2)
(princ "\n ====")
(princ (vlax-get-property lw 'AREA))
(princ "\n ====")
(princ "ret =")(princ (1Dlib:id-get-verts lw 3 0.0))

(princ "\n=== END ===\n")

(princ)
)



;;======================================================;;
;;  written by Fatty The Old Horse 10/13/05    ;;
;;      (framework)      ;;
;;======================================================;;
;;      helper functions  ;;
;; group list by number 1D_mod


(defun pl:group-by-num (lst num / ls retf)
(if (= (rem (length lst) num ) 0)
 (progn (setq ls nil)
  (repeat (/ (length lst) num)
    (repeat num (setq ls (cons (car lst) ls) lst (cdr lst)))
  (setq retf (append retf (list (reverse ls))) ls nil)))) retf)



;| disabled test function

(defun C:AD (/)
(setq AXBase '(
;0=type      1=LINE    2=SPLINE     3=ARC	4=CIRCLE    5=ELLIPSE	  6=POLYLINE	7=3DPLINE	 8=2DPLINE	  9=3DFACE	10=PLG_MESH	11=POLYFACE	  12=2DSOLID 
("ACAD"     "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBPOLYLINE" "ACDB3DPOLYLINE" "ACDB2DPOLYLINE" "ACDBFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
("BRICSCAD" "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBPOLYLINE" "ACDB3DPOLYLINE" "ACDB2DPOLYLINE" "ACDBFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
("ZWCAD"    "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBLIGHTWEIGHTPOLYLINE" "ACDB3DPOLY" "ACDBPOLYLINE" "ACDB3DFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
))
(setq MTo (car (entsel)))
(setq MT MTo)
(setq id_obj (vlax-ename->vla-object MTo))
(setq lw id_obj)
(SETQ LWO (setq lwO (strcase (vlax-get id_obj 'ObjectName))))
(setq DIV 12)
(setq RESL 0.0)


	(setq lvl (1D:ZWAL (vlax-curve-getEndParam id_obj)))
	(setq DIVx DIV)
		(setq SEG (/ lvl RESL))
		(if (< SEG 50) (setq DIVx (/ SEG 6.)))
		(if (< SEG 5) (setq DIVx 2.))
		(if (>= SEG 50) (setq DIVx (/ SEG 9.)))
		(setq DIVx (fix DIVx))
(setq SEG (/ lvl RESL))


(setq ret (1Dlib:id-get-verts id_obj DIV RESL))
;(princ (vlax-curve-getdistatpoint id_obj (getpoint)))
;(setq q1 (vlax-curve-getEndParam id_obj))
;(setq q2 (vlax-curve-getStartParam id_obj))
;(command "_POINT" (vlax-curve-getpointatdist id_obj (getreal)))
;		(setq ret (pl:group-by-num (vlax-get id_obj 'Coordinates) 2))
;		(setq retx (1D:getbulge_list id_obj))
)
|;


;| disabled test function
(defun C:FF (/)
(setq MTo (car (entsel)))
(setq MT MTo)
(setq id_obj (vlax-ename->vla-object MTo))
(setq DIV 12)
(setq ret (pl:group-by-num (vlax-get id_obj 'Coordinates) 2))
(setq retx (1D:getbulge_list id_obj))
;(if (= AXE (nth 6 AXBaseD))
;(progn
(setq ret (1D:ZWAL (vlax-get (vlax-ename->vla-object MTo) 'Coordinates)))
(setq ret (pl:group-by-num ret 2))
(setq retx (1D:getbulge_list id_obj))
;))
)
(setq lvl (vla-get-Elevation id_obj))
(setq ret (mapcar '(lambda (x) (append x (list lvl))) ret) )  ;;;adding Z to verts
(setq i 0 lvl '())
	(foreach n retx
		(if (= n 0.0)
			(setq lvl (cons (nth i ret) lvl))
			(progn
				(setq SEG (1D:Bulge_split id_obj retx i n NOR))
				(setq lvl (cons (nth i ret) lvl))
				(setq lvl (append SEG lvl ))
			)
		)
	(setq i (1+ i))
	)

	(setq ret lvl)
		))



(setq i 0 lvl '())

(foreach n retx
(if (= n 0.0)
(setq lvl (cons (nth i ret) lvl))
(progn
(setq SEG (1D:Bulge_split id_obj retx i n NOR))
(setq lvl (cons (nth i ret) lvl))
(setq lvl (append lvl SEG))
)
)
(setq i (1+ i))
)

(setq ret lvl)


		(setq lvl (vla-get-Elevation id_obj))
		(mapcar '(lambda (x) (append x (list lvl))) ret)  ;;;adding Z to verts
)
|;



;;; ========= Parse Bulges in DXF ===========
(defun 1D:getbulge_list ( MTob / retn)
(if (/= (type MTob) 'Ename) (setq MTob (vlax-vla-object->ename MTob)))
(setq retn (apply 'append 		;42 DXF groups
                             (mapcar '(lambda (el)
                                       (if (member (car el) '(42));;10 11 bulge support
                                         (list 
						(cdr el)
					); only coords
                                       ) ;_  if
                                     ) ;_  lambda
                                   (entget MTob)			;current obj
                             ) ;_  mapcar
                     ) ;_  apply
)
)


;;; ========= Verticles of splitted bulged polyline's segment ===========
(defun 1D:Bulge_split ( id_obj lst p1 BLG NOR /		;;; $$$NOR 14:22 18.02.2014
p1 p2 ref DIVx px n
)

;; for testing input (1D:Bulge_split id_obj retx i n NOR)
;; (setq lst retx p1 i BLG n)

(setq $$x p1);; test
(setq $$y lst);; test
(setq $$z BLG)
(setq $$yy ret)


;|
(progn
(setq p1 $$x) ;; test
(setq lst $$y) ;; test
(setq BLG $$z)
(setq ret yy)
;;; FUS-RO-DAH!
(setq lvl (vla-get-Elevation id_obj))
;(setq  ret (mapcar '(lambda (x) (append x (list lvl))) (pl:group-by-num (1D:ZWAL (vlax-get (vlax-ename->vla-object MT) 'Coordinates)) 2) ) )
)
|;


(setq p2 (1+ p1))
(if (or
(> (1+ p2) (length lst))
(= (vlax-curve-getdistatpoint id_obj (trans (nth p2 ret) NOR 0) ) 0.0)	;;;$$$$LASTS
)

(progn
(setq p2 (vlax-get id_obj 'Length) )	;;;last segment
(setq p1 (vlax-curve-getdistatpoint id_obj (trans (nth p1 ret) NOR 0)))
)
(progn
(setq p2 (vlax-curve-getdistatpoint id_obj (trans (nth p2 ret) NOR 0) ) ) ;;not last segment
(setq p1 (vlax-curve-getdistatpoint id_obj (trans (nth p1 ret) NOR 0) ))		;;; $$$NOR 14:22 18.02.2014
)
)

;(setq p2 (vlax-curve-getdistatpoint id_obj (vlax-curve-getclosestpointto  id_obj (nth p2 ret))) )
;(setq p1 (vlax-curve-getdistatpoint id_obj (vlax-curve-getclosestpointto  id_obj (nth p1 ret))) )

(if (and p2 p1 (not (equal p2 p1 0.0001)))
	(progn
	
	;; (vlax-curve-getclosestpointto id_obj (nth p2 ret))
	
	(if (< (abs BLG) 1.5)
	(setq DIVx DIV)
	(setq DIVx (* 2 DIV))
	)
	
	(if (< (abs BLG) 0.1)
	(setq DIVx 3)
	)
	(setq p2 (- p2 p1))
	(setq p2 (/ p2 DIVx)
	ref nil
	)
	
	(repeat (- DIVx 1)
	(setq ref (cons (trans (vlax-curve-getPointAtDist id_obj (setq p1 (+ p1 p2)) ) 0 NOR) ref))	;;; $$$NOR
	)
	)
		(setq ref nil)
)

;; (foreach n retx (command "_POINT" n));; for test purposes
ref
)





(defun 1D:ZWAL (val / )
(if (= (TYPE val) 'VARIANT)
(progn
;(princ "\nVARIANT\n")
(setq val (vlax-safearray->list (vlax-variant-value val)) )
)
(setq val val)
))


;;; ==================================================
;;;              Getting Vertex Data
;;;		    by 1D_Inc
;;;	    for Elevation as Z coordinates
;;;	and other different objects type support
;;;
;;; ==================================================


(defun 1Dlib:id-get-verts (id_obj DIV RESL /
lvl ret SEG i EP SP AXE sps ses NCP DIVx param NOR
)
  (if (= (type id_obj) 'Ename)
    (setq id_obj (vlax-ename->vla-object id_obj))
  ) ;_ end of if

(setq AXE (strcase ( vlax-get id_obj 'Objectname )) )


;;; ----- PLINE section -----
	(if (= AXE (nth 6 AXBaseD))
;; (setq id_Obj (vlax-ename->vla-object (car (entsel))))
          (progn
		(setq lvl (vla-get-Elevation id_obj))
		(setq NOR (1D:ZWAL (vla-get-Normal id_obj)))
		(setq ret (1D:ZWAL (vlax-get id_obj 'Coordinates)))	;;;$$$13:40 18.02.2014
		(setq ret (pl:group-by-num ret 2))
		(setq ret (mapcar '(lambda (x) (append x (list lvl)) ) ret) )  ;;;adding Z to verts
		(setq retx (1D:getbulge_list id_obj))
(setq i 0 lvl '())

;|
	;;;thanks, ZWCAD :-\
	(foreach n retx
		(if (< (abs n) 0.03)
			(setq lvl (cons (nth i ret) lvl))
			(progn
				(setq SEG (1D:Bulge_split id_obj retx i n NOR))
				(setq lvl (cons (nth i ret) lvl))
				(setq lvl (append SEG lvl ))
			)
		)
	(setq i (1+ i))
	)
|;


	(repeat (length retx)
	(setq n (nth i retx))
	(if (< (abs n) 0.003)
			(setq lvl (cons (nth i ret) lvl))
			(progn
				(setq SEG (1D:Bulge_split id_obj retx i n NOR))
				(setq lvl (cons (nth i ret) lvl))
				(setq lvl (append SEG lvl ))
			)
		)
	(setq i (1+ i))
	)

(setq ret (mapcar '(lambda (x) (trans x NOR 0) ) lvl) )	;;; $$$NOR 12:03 18.02.2014
ret
		) )


;;; ----- 2DPOLYLINE section -----
        (if (= AXE (nth 8 AXBaseD))

	  (progn
		(setq ret (pl:group-by-num (1D:ZWAL (vlax-get id_obj 'Coordinates)) 3))
		(setq lvl (1D:ZWAL (vla-get-Elevation id_obj)))
		(setq ret (mapcar '(lambda (x) (list (nth 0 x) (nth 1 x) lvl)) ret))   ;;;replacing Z to verts
	  )
	)


;;; ----- 3DPOLYLINE section -----        
	(if (= AXE (nth 7 AXBaseD) )

         (setq ret (pl:group-by-num (1D:ZWAL (vlax-get id_obj 'Coordinates)) 3))
        )

;;; ------- 3DFACE
	(if	(or
		(= AXE (nth 9 AXBaseD) )
		(= AXE (nth 10 AXBaseD) )
		(= AXE (nth 11 AXBaseD) )
		(= AXE (nth 12 AXBaseD) )
		)

          (progn
		(setq ret (pl:group-by-num (1D:ZWAL (vlax-get id_obj 'Coordinates)) 3))
		(setq retc ret)

		))


;;; ----- LINE section -----
        (if (= AXE (nth 1 AXBaseD))
	(setq ret (list (vlax-curve-getstartpoint id_obj)(vlax-curve-getendpoint id_obj)))
	)

;;; ----- SPLINE section -----

	(if (= AXE (nth 2 AXBaseD))
	;;;(pl:group-by-num (vlax-get id_obj 'FitPoints) 3) ;;; not the case
	(progn


	(if (and (assoc 44 (entget (vlax-vla-object->ename id_obj) ))  (assoc 11 (entget (vlax-vla-object->ename id_obj) )))
	(progn
	(setq lvl (vlax-curve-getEndParam id_obj))
	(setq DIVx DIV)
;| maybe later

		(if (/= RESL 0.0)
		(progn
		(setq SEG (/ lvl RESL))
		(if (< SEG 50) (setq DIVx (/ SEG 6.)))
		(if (< SEG 5) (setq DIVx 2.))
		(if (>= SEG 50) (setq DIVx (/ SEG 9.)))
		(setq DIVx (fix DIVx))
		)
		)
|;
	
	(setq	SEG (/ lvl (* 3 DIVx))
		ret (cons (vlax-curve-getPointAtParam id_obj 0.0) nil)
		lvl 0.0)	;;;Segment length 
			(repeat (* 3 DIVx)
			(setq ret (cons (vlax-curve-getPointAtParam id_obj (setq lvl (+ lvl SEG))) ret)) ;;;feeding ret
			;;(command "_POINT" (car ret)) ;;;test purposes string
			)
		(setq ret (cons (vlax-curve-getPointAtParam id_obj (vlax-curve-getEndParam id_obj)) ret)) ;;; LastOne
	)

	(progn
	    (setq param (fix (/ DIV 4)))
	(if (< param 1) (setq param 1))
	    (setq sps (vlax-curve-getStartParam id_obj)
	          ses (vlax-curve-getEndParam id_obj)
	          NCP (vlax-get id_obj 'NumberOfControlPoints)
	          DIVx	(/ (- ses sps) (* NCP param) )
	          param (+ DIVx sps)
	          ret (cons (vlax-curve-getStartPoint id_obj) ret)
	    )
	 
	    (while (< param ses)
	      (setq pt1 (vlax-curve-getPointAtParam id_obj param)
		    pt2 (vlax-curve-getPointAtParam id_obj (+ param DIVx))
		   ; pt0 (vlax-curve-getPointAtParam id_obj (- param DIVx))
	            param (+ param DIVx)
	
	      )
		; (princ "\n")(princ pt0)
	        (setq ret (cons pt1 ret))
	   )
	;add last point and check for duplicates
	    (if 
	      (not (equal (setq pt1 (vlax-curve-getEndPoint id_obj) ) (car ret) 1e-8) )
	      (setq ret (cons pt1 ret))
	    )
	    (setq ret (reverse ret))
	
	);progn2


	);if
	));prognif



;;; ----- ARC section -----
	(if (= AXE (nth 3 AXBaseD))

	(progn
	(if (< (vlax-get id_obj 'TotalAngle) 3.1) (setq DIVx DIV) (setq DIVx (* 2 DIV)))
	
	(setq lvl (vlax-get id_obj 'ArcLength))
	(setq	SEG (/ lvl DIVx)
		ret (cons (vlax-curve-getPointAtDist id_obj 0.0) nil)
		lvl 0.0)
			(repeat (- DIVx 1)
			(setq ret (cons (vlax-curve-getPointAtDist id_obj (setq lvl (+ lvl SEG))) ret)) ;;;feeding ret
			)
		(setq ret (cons (vlax-curve-getPointAtDist id_obj  (- (vlax-get id_obj 'ArcLength) 0.0000001) ) ret) )  ;;; LastOne
	)
	)


;;; ----- ELLIPSE section -----
	(if (= AXE (nth 5 AXBaseD))
	(progn

(setq	EP (1D:ZWAL (vlax-get id_obj 'EndPoint))
	SP (1D:ZWAL (vlax-get id_obj 'StartPoint)))
	(if  (< (distance EP SP) (/ (vlax-get id_obj 'Area) 1e8) ); ifsemiclosed

	;; ====  ELLIPSE CIRCLE
	(progn (setq lvl (vlax-curve-getDistAtParam id_obj (* 2 pi) ))
	(setq ELC 1)
	(setq	SEG (/ lvl (* 2 DIV))
		ret nil
		lvl 0.0)
			(repeat (- (* 2 DIV) 1)
			(setq ret (cons (vlax-curve-getPointAtDist id_obj (setq lvl (+ lvl SEG))) ret)) ;;;feeding ret
			)
		(setq ret (cons EP ret))  ;;; LastOne


	)
	;; ====  ELLIPSE ARC 
	(progn
(if (not (setq lvl (vlax-curve-getDistAtPoint id_obj EP )))
	(setq lvl (vlax-curve-getDistAtParam id_obj (vlax-get id_obj 'EndParameter) ))  ;; sad but true, to fix ActiveX possible bug.
)
	(setq	SEG (/ lvl DIV)
		ret (cons SP nil)
		lvl 0.0)
			(repeat (- DIV 1)
			(setq ret (cons (vlax-curve-getPointAtDist id_obj (setq lvl (+ lvl SEG))) ret)) ;;;feeding ret
			)
		(setq ret (cons EP ret))  ;;; LastOne
	)
	)
	))

;;; ----- CIRCLE section -----
	(if (= AXE (nth 4 AXBaseD))

	(progn 
	;(setq ret (vlax-get id_obj 'Center))
	;(setq ret (cons ret (cons (list (nth 0 ret)(nth 1 ret)(+ 10.0 (nth 2 ret)) ) nil))) adding vertex to a center
		(setq ret (cons (vlax-curve-getPointAtParam id_obj 0.0) ret))
		(setq SEG (/ (* 2 pi) (* 2 DIV)) lvl 0.0)
		(repeat (- (* 2 DIV) 1)
		(setq ret (cons (vlax-curve-getPointAtParam id_obj (setq lvl (+ lvl SEG)))  ret))
		)
	
	)
	)


;;; ----- OTHERWISE section -----


(setq ret ret);; last call to sendout
);defun






;;;; =============================================
;;;; ============= MEMSLOTs SECTION ================
;;;; =======saving and restoring selections =========


;|[[[[[[[[[[[[[[[[[[
[[[[ 23
developed by 1D_Inc
|;

(DEFUN C:23 (/ AS VRS)
(princ "\n Select object to zoom <Current>:")
(setq AS (ssget)
);setq
(if(/= AS nil)(setq VS AS))
;[
(setq VRS (strcase (getvar "PROGRAM")))
(if (not VS) (princ "\n No Current Selection Set Found")
(progn
(command "_SELECT" "_U" ""); (command "_ZOOM" "_O" VS "")
(command "_ZOOM" "_S"  "0.95x")
;(if (= VRS "ZWCAD") (command "_ZOOM" "_OB" VS "" ""))
;(if (= VRS "ACAD") (command "_ZOOM" "_OB" VS ""))
))
;]

;	(command "_ZOOM" "_S"  "1.01x")

	(sssetfirst vs vs)
(princ "...done")
(princ)

);defun




;|[[[[[[[[[[[[[[[[[[
[[[[ 24
developed by 1D_Inc
|;

(DEFUN C:24 (/ AS VRS)
(princ "\n Select object to zoom <Current>:")
(setq AS (ssget)
);setq
(if(/= AS nil)(setq VSS AS))
;[
(setq VRS (strcase (getvar "PROGRAM")))
(if (not VSS) (princ "\n No Current Selection Set Found")
(progn
(if (= VRS "ZWCAD") (command "_ZOOM" "_OB" VSS "" ""))
(if (= VRS "ACAD") (command "_ZOOM" "_OB" VSS ""))
))
;]



;	(command "_ZOOM" "_S"  "1.01x")

	(sssetfirst vss vss)
(princ "...done")
(princ)

);defun





(DEFUN C:233 (/ AS i)
(princ "\n Select object to add to memslot <Current>:")
(setq AS (ssget) i 0);setq
(if (and AS vs)

(repeat (sslength AS)
(setq vs (ssadd (ssname AS i) vs))
(setq i (1+ i))
)
(setq VS AS)
)
;[
;(if VS
;(command "_ZOOM" "_O" VS "")
;(ACET-SS-ZOOM-EXTENTS VS)
;(princ "\n No Current Selection Set Found"));cond
;]
	(command "_ZOOM" "_S"  "0.99x")
	(sssetfirst vs vs)
(princ "...done")
(princ)

)



(DEFUN C:244 (/ AS i)
(princ "\n Select object to add to memslot <Current>:")
(setq AS (ssget) i 0);setq
(if (and AS vss)

(repeat (sslength AS)
(setq vss (ssadd (ssname AS i) vss))
(setq i (1+ i))
)
(setq VSS AS)
)

	(command "_ZOOM" "_S"  "0.99x")
	(sssetfirst vss vss)
(princ "...done")
(princ)

)




;|[[[[[[[[[[[[[[[[[[
[[[[ 33
developed by 1D_Inc
|;
(DEFUN C:33 (/ VRS)	;;;	Memslot vs selection
(if (and (/= VS  nil) (> (sslength VS) 0))
(progn
(princ "\n")
;(command "_SELECT" "_U" "")
(setq VRS (strcase (getvar "PROGRAM")))
(if (= VRS "ZWCAD") (command "_ZOOM" "_OB" VS "" ""))
(if (= VRS "ACAD") (command "_ZOOM" "_OB" VS ""))
(princ (sslength vs))
;(command "_PSELECT" VS "")
;(ACET-SS-ZOOM-EXTENTS vs)
(command "_ZOOM" "_S"  "0.8x")
(sssetfirst vs vs)

);progn
);if
(princ "...done")
(princ)
);defun


;|[[[[[[[[[[[[[[[[[[
[[[[ 44
developed by 1D_Inc
|;
(DEFUN C:44 (/ VRS)	;;;	Memslot vss selection

(if (and (/= VSS  nil) (> (sslength VSS) 0))
(progn
(princ "\n")
;(command "_SELECT" "_U" "")
(setq VRS (strcase (getvar "PROGRAM")))
(if (= VRS "ZWCAD") (command "_ZOOM" "_OB" VSS "" ""))
(if (= VRS "ACAD") (command "_ZOOM" "_OB" VSS ""))
(princ (sslength vss))
;(command "_PSELECT" VSS "")
;(ACET-SS-ZOOM-EXTENTS vSs)
(command "_ZOOM" "_S"  "0.8x")
(sssetfirst vss vss)

);progn
);if
(princ "...done")
(princ)
);defun


;|[[[[[[[[[[[[[[[[[[
[[[[99
developed by 1D_Inc
|;
(DEFUN C:99 (/ VRS)	;;;	Memslot vsss selection

(if (and (/= VSSS  nil) (> (sslength VSSS) 0))
(progn
(princ "\n")
;(command "_SELECT" "_U" "")
(setq VRS (strcase (getvar "PROGRAM")))
(if (= VRS "ZWCAD") (command "_ZOOM" "_OB" VSSS "" ""))
(if (= VRS "ACAD") (command "_ZOOM" "_OB" VSSS ""))
(princ (sslength vsss))
;(command "_PSELECT" VSSS "")
;(ACET-SS-ZOOM-EXTENTS vSSs)
(command "_ZOOM" "_S"  "0.8x")
(sssetfirst vsss vsss)

);progn
);if
(princ "...done")
(princ)
);defun












(defun 1D:Write_OBJ (MTX OpenF iFILE / )
;;; ===== Writing file header =====
(if (= OpenF 1) (setq secs (GTIMER "end")) (setq secs (GTIMER "process")))

(setq MTX (strcat 

"# File: '" MTFX "'
# DWG2OBJ4Blender_Converter v5.17 by Paul Kotelevets (1D_Inc) 09.10.2012-2022
# http://blenderartists.org/forum/showthread.php?267990-Direct-Autocad-Intellicad-to-Blender-exporter-LISP-to-Download \n# [" 
 (rtos MTLen 2 0) "_e3] characters for "
OUTSTRING
"\n# LM-layer mode, NT-PointCloud Nth, RS-Reduce Scale, DIV-Curve segmenting, PC-Precision
# " VRS " FILE:" (itoa iFILE) " at " secs
"\n\n\no 1D_cloud
\n v 0.0 0.0 0.0"
MTX
))


;;; ===== Writing OBJ file =====
;(if (= VRS "ACAD") (setvar "MODEMACRO" " Writing file")
(princ "\nWriting file")
;)
(setq FL (vl-filename-mktemp MTF (strcat (getvar "dwgprefix") ) ".obj")) ;;; naming obj
   (setq f (open FL "w"))
   (write-line MTX f)
   (close f)


(if (= OpenF 1)
(progn
;;; ==== Opening created OBJ file in notepad on export end =====
;(if (= VRS "ACAD") (setvar "MODEMACRO" "Opening file")
(princ "\nOpening file")
;)
   (vlax-invoke-method (setq f (vlax-get-or-create-object "wscript.shell"))
    "run" (strcat "notepad.exe " "\"" FL "\"")
    1
    :vlax-false) ;_ end of vlax-invoke-method

  (vlax-release-object f); do not search
))


;(if (= VRS "ACAD") (setvar "MODEMACRO" " "))
;;; ==== Spent Time End ====
(princ (strcat "\n=============  \n[" (rtos MTLen 2 0) "_e3] characters for " OUTSTRING))
(princ (strcat "\n=============  " (itoa id) " Objects proceed in " ))


;|
(if (= VRS "ACAD")(princ (strcat 
                (rtos (/ (- (car (_VL-TIMES)) tim) 1000.) 2 4)
                " secs."
        ) ;_  strcat
)
|;
(setq secs (GTIMER "process"))
(princ secs )
;)




(princ "\nFile output in:\n") (princ FL) (princ "     FILE:")(princ (itoa iFILE))


(if (> (sslength vs) 0)
(progn (princ "\n") (princ (sslength vs)) (princ " non-planar polylines found")
;(C:33)
))


(princ)
)



(defun C:1DPT ()
(setq $$1DPOINTBASE (getpoint) )
(setq $$1DPOINTBASE (cons (- (nth 0 $$1DPOINTBASE)) (- (nth 1 $$1DPOINTBASE)) ) )
)


;;;========================================================================
;;;========================================================================
;;;========================================================================

;;;========================================================================
;;;========================================================================
;;;========================================================================

;;;========================================================================
;;;========================================================================
;;;========================================================================

;;; =========================================
;;;
;;;		Main Function
;;;
;;; =========================================

;|
(setq idx 0 qq '(
$$time AXBase CNT CNTL CNTP CNTPnum DIV ELC Etag f fl ie id iFILE iflag ii iifirst LAYBASE lew Lmode lw lwO
MT MTE MTF MTFX MTG MTL MTLAY MTLen MTLenon MTNor MTO MTP MTPL MTPP MTPX MTX OpenF
OUTSTRING PRC PREC PT1 PT2 qq RESL ret RETC retlen retx RF RS secs tim tstamp VRS vs VSL VSP vssq vsx 
;;
tm $$time $$x $$y AS AXE ct DIV DIVx ELC ent ent2 ent3 entl f fl i id_obj ls lst lvl lw mins MLIST MT
MTo MTob MTX p1 p2 param PLIST pt1 q1 q2 ref RESL ret retc retf retn retx secs SEG sps val VRS VS VSS yy 
))
(foreach n qq (princ "\n") (princ n) (princ " = ") (princ (eval n)) (setq idx (1+ idx)))

|;

(defun C:1D2B (/	;18:13 26.05.2015 DWG2OBJ4BLENDER CONVERTER
;;;  Paul Kotelevets aka 1D_Inc aka 1D_Inc (10.2013)

$$time AXBase AXBased CNT CNTL CNTP CNTPnum DIV ELC Etag f fl i ie id iFILE iflag ii iifirst LAYBASE lew Lmode lw lwO
MT MTE MTF MTFX MTG MTL MTLAY MTLen MTLenon MTNor MTO MTP MTPL MTPP MTPX MTX OpenF
OUTSTRING PRC PREC PT1 PT2 qq RESL ret RETC retlen retx RF RS secs tim tstamp VRS vs VSL VSP vssq vsx pstamp
layw spt
SXF SYF

)
(vl-load-com)

(princ "\nver = v5.17\n")

(setq vs nil)
(setq VRS (strcase (getvar "PROGRAM")))

;;; ========= set known ActiveX object's names DB =======
(setq AXBase '(
;0=type      1=LINE    2=SPLINE     3=ARC	4=CIRCLE    5=ELLIPSE	  6=POLYLINE	7=3DPLINE	 8=2DPLINE	  9=3DFACE	10=POLYGONMESH	11=POLYFACE	  12=2DSOLID 
("ACAD"     "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBPOLYLINE" "ACDB3DPOLYLINE" "ACDB2DPOLYLINE" "ACDBFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
("BRICSCAD" "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBPOLYLINE" "ACDB3DPOLYLINE" "ACDB2DPOLYLINE" "ACDBFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
("ZWCAD"    "ACDBLINE" "ACDBSPLINE" "ACDBARC" "ACDBCIRCLE" "ACDBELLIPSE" "ACDBLIGHTWEIGHTPOLYLINE" "ACDB3DPOLY" "ACDBPOLYLINE" "ACDB3DFACE" "ACDBPOLYGONMESH" "ACDBPOLYFACEMESH" "ACDBSOLID")
))

(setq AXBaseD (assoc VRS AXBase))

(setq vs (ssadd))	;;empty memslot 14:23 06.10.2012
(setq vsx (ssget '((0 . "POINT,LWPOLYLINE,POLYLINE,LINE,SPLINE,ARC,CIRCLE,ELLIPSE,3DFACE,SOLID")) ))

(if (not vsx)
(setq vsx (ssget "_X" '((0 . "POINT,LWPOLYLINE,POLYLINE,LINE,SPLINE,ARC,CIRCLE,ELLIPSE,3DFACE,SOLID")) ))
)

(setq vsss nil)		;;; negative memslot
(setq vsss (ssadd))	;;; negative memslot

;;; ========= Input data convert conditions ===========

(setq OBN (getstring "\nOBN enter obj name prefix (space for skip):"))   ;;; 18:13 26.05.2015
(if $$1DB  (progn (princ "\n ___________\n") (princ $$1DB) (setq FLS (getstring "\n Load Previous Settings? <space = yes, anykey = no>")) ))


(if (= FLS "")

(mapcar 'set (list 'spt 'lmode 'RF 'RS 'DIV 'PREC 'layw 'SXF 'SYF 'SYZ 'TEXX 'DIMTEX) $$1DB)
(progn

;;points [
(if $$1DPOINTBASE (setq SXF (car $$1DPOINTBASE) SYF (cdr $$1DPOINTBASE)  )  )
(if (not SXF) (setq SXF 0.0))
(if (not SYF) (setq SYF 0.0))
(setq SXX (getreal (strcat "\n X Shift <" (rtos SXF 2 8) ">")))
(setq SYY (getreal (strcat "\n Y Shift <" (rtos SYF 2 8) ">")))

(cond 
((and (not SXX) $$1DPOINTBASE) (setq SXF (car $$1DPOINTBASE) ) )
(SXX (setq SXF SXX))
( t (setq SXF 0.0 ) )
)

(cond 
((and (not SYY) $$1DPOINTBASE) (setq SYF (cdr $$1DPOINTBASE) ) )
(SYY (setq SYF SYY))
( t (setq SYF 0.0 ) )
)

(setq $$1DPOINTBASE (cons SXF SYF))
;;]


(setq RS (getreal	"\nRS (Reduce scale for scene) <1000>:"))
(setq spt (getint "\nSplit x1000 verts value <35>"))
(setq lmode (getint	"\nLM (Object's Distribution mode) [0-2] <0> (0= byLayer, 1= byLayer+byObjectType, 2= byEntity):"))
(setq RF (getint	"\nNT (every Nth dot in pointcloud) <51>:"))
(setq DIV (getint	"\nDIV (Curve segmenting) <8>:"))
;;(setq RESL (getreal	"\nRESL <0.0>"))
(setq PREC (getint "\nPC (Precision: digits after decimal point [0-8])<8>"))
(setq layw (getstring "\nLY (Write layers as materials: space=no, anykey=yes)"))

)
)




;;; ========= Checking Input data convert conditions ===========
(if (not spt) (setq spt 35))
(if (not OBN) (setq OBN ""))
(if (not (member lmode '(1 2 0))) (setq lmode 0))
(if (not (eq (type RS) 'REAL)) (setq RS 1000.0))
(if (not (eq (type RF) 'INT)) (setq RF 51))
(if (or (not (eq (type DIV) 'INT)) (< DIV 1) ) (setq DIV 8))
(if (not (eq (type PREC) 'INT)) (setq PREC 8))
(if (or (> PREC 8) (< PREC 0)) (setq PREC 8))
(if (not (eq (type RESL) 'REAL)) (setq RESL 0.0))
(setq tstamp 401)
(setq pstamp 60001);20001
(setvar "DIMZIN" 8)

(setq $$1DB (list spt lmode RF RS DIV PREC layw SXF SYF SYZ TEXX DIMTEX) )

(setq spt (/ (* 1000 spt) 17))

(setq OUTSTRING (strcat "# Export: LM=" (itoa lmode) " | NT=" (itoa RF) " | RS=1/" (rtos RS 2 1) " | DIV=" (itoa DIV) " | PC=" (itoa PREC) ))

(STIMER) ;;start timer


(setq	i 0 ie 0 ii 0 id 0 CNT 0 iflag nil CNTP 0 CNTL 0 MTX "\n" MTL "\n" MTLen 0 MTLenon 0
	MTPL " " MTPX " "iFILE 0 MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") )  MTFX (strcat (getvar "DWGPREFIX") (getvar "DWGNAME") ))
(setq vssq (ssadd))	;;pointcloud collection
;;; ==== Spent Time Beging ====
;(if (= VRS "ACAD") (setq tim (car (_VL-TIMES))))
(setq VSL (sslength vsx))	

(repeat VSL
	(setq MT (ssname vsx ie))		;;member of selection set
	(setq MTE (cdr (assoc 0 (entget MT))))	;;


;;; ==== Point Filtering ====

(if (and
;(eq (vlax-get lw 'Objectname ) "AcDbPoint")
(= MTE "POINT")
(= (gcD (1+ ie) RF) RF))	;;;every RF-th point is converted
(progn (setq id (1+ id) vssq (ssadd MT vssq))
(if (= (GCD (1+ ie) tstamp) tstamp)(princ (strcat "\nReading PointCloud: " (rtos (* 100 (/ (+ 1.0 ie) VSL)) 2 0) "%")) )
)
)

;;; [[GEOM]
(if (member MTE '("LWPOLYLINE" "LINE" "SPLINE" "POLYLINE" "ARC" "CIRCLE" "ELLIPSE" "3DFACE" "SOLID"))
(progn 
(setq id (1+ id))
	(setq vss (ssadd MT (ssadd))) ;;memslot with last entity
	(setq vsss (ssadd MT vsss)) ;;negativity
	;(setq MTG (ssname vsx (+ 1 ie)))
	(setq lw (vlax-ename->vla-object MT))

(setq PRC (rtos (* 100 (/ (+ 1.0 ie) VSL)) 2 0))

;;; ------ String Length Collecting 1/3 -------
(setq MTLen (+ (/ (strlen MTL) 800.0) MTLen))
(setq secs "_")
;(if (and (= VRS "ACAD")(= (GCD (1+ ie) tstamp) tstamp))
;(setvar "MODEMACRO" (strcat "["(rtos MTLen 2 0) "_e3] Writing Geometry: " PRC "% in " (rtos (/ (- (car (_VL-TIMES)) tim) 1000.) 2 4) ) )
(if (= (GCD (1+ ie) tstamp) tstamp)
(progn (setq secs (GTIMER "process")) (princ (strcat "\n["(rtos MTLen 2 0) "_e3] Writing Geometry: " PRC "% in " secs) ) 
;(princ "  MTlen=") (princ MTLen)
;(princ "  MTlenon=") (princ MTLenon)
))
;)


(setq lwO (strcase (vlax-get lw 'ObjectName)))
(setq MTLAY (substr (vla-get-Layer lw) 1 10))

(if (not (member MTLAY LAYBASE))
	(setq LAYBASE (reverse (cons MTLAY (reverse LAYBASE))))
)

(setq CNT (vl-position MTLAY LAYBASE))
	(if (< CNT 100) (setq CNTPnum (strcat "0" (itoa CNT))))
	(if (< CNT 10) (setq CNTPnum (strcat "00" (itoa CNT))))
(setq MTLAY (strcat "LAY" CNTPnum))


;;; ========= Mass mode ===========
(if (= lmode 0)
	(progn
	(setq Etag "")
	(if (= MTE "3DFACE") (setq Etag "_3F"))
	(IF (= MTE "SOLID") (setq Etag "_2S"))
	(setq MTX (strcat MTX "\no 1D_" MTLAY Etag))
	)
)

;;; =========  ByType/Element mode ===========
(if (member lmode '(1 2))
(progn
	(cond
		((= MTE "ARC") (setq Etag "_A"))
		((= MTE "LINE") (setq Etag "_L"))
		((= MTE "CIRCLE") (setq Etag "_C"))
		((= MTE "SPLINE") (setq Etag "_S"))
		((= MTE "POLYLINE") (setq Etag "_PP"))
		((= MTE "LWPOLYLINE") (setq Etag "_P"))
		((= MTE "ELLIPSE") (setq Etag "_E"))
		((= MTE "3DFACE") (setq Etag "_3F"))
		((= MTE "SOLID") (setq Etag "_2S"))

		(t (setq Etag "_N"))
	)

;;; -------- Object Name ------
	(if (= lmode 1)
	(setq MTX (strcat MTX "\no 1D_" MTLAY Etag))
	(setq MTX (strcat MTX "\no 1D_" MTLAY Etag (itoa CNTP)) CNTP (1+ CNTP))
	)
))


;;;; Obsoleted fragment [[
;|
;;; =========  ByElement mode ===========
(if (= lmode 2)
(progn
(if (member MTE '("LINE" "SPLINE"))	;; setting header
(setq MTX (strcat MTX "\no 1D_" MTLAY "_L_" (itoa CNTL)) CNTL (1+ CNTL))
(setq MTX (strcat MTX "\no 1D_" MTLAY "_P_" (itoa CNTP)) CNTP (1+ CNTP)))
))


(if (member MTE '("LWPOLYLINE"))	(setq lew (vla-get-Elevation lw)) )
(setq retx (apply 'append 		;10 DXF groups
                             (mapcar '(lambda (el)
                                       (if (member (car el) '(42));;10 11 bulge support
                                         (list 
						(cdr el)
					); only coords
                                       ) ;_  if
                                     ) ;_  lambda
                                   (entget MT)			;current obj
                             ) ;_  mapcar
                     ) ;_  apply
)

(if (member MTE '("LWPOLYLINE"))
(setq ret (mapcar '(lambda (x) (append x (list lew))) retx) )  ;;;adding Z to verts
(setq ret retx)
)

|;
;;;; Obsoleted fragment ]]


(setq MTNor nil)
(if (= (cdr (assoc 0 (entget MT))) "LWPOLYLINE")
(setq MTNor (1D:ZWAL (vlax-get (vlax-ename->vla-object MT) 'Normal)))
)

;;; ==== exception ====
(if

nil

;|
;; ==== prevoius 
(or	(and (= (cdr (assoc 0 (entget MT))) "LWPOLYLINE")
(not (member (nth 2 MTNor ) '(1.0 -1.0)))		;;; $$$NOR 12:04 18.02.2014
 MTNor )	;;tellicad
; (= (vlax-get (vlax-ename->vla-object MT) 'ObjectName) "AcDbPolyFaceMesh")
nil
)
|;

(setq vs (ssadd MT vs)) ;;;;;vssqsssss
(progn

;;;;; ========= Getting Verts =======
(setq ret (1Dlib:id-get-verts MT DIV RESL) )

(if ret	;; stability issue: if entity gave no vertexes - skip.
(progn

;;;;; ========= Apply Scale and Shift =======	;;;$$$ 9:45 05.12.2013
; telli

(SETQ RETC '())
(FOREACH N RET (if (= ( type N) 'LIST)
(SETQ RETC (CONS (list 
(/ (+ (nth 0 N) SXF) RS)
(/ (+ (nth 1 N) SYF) RS)
(/ (nth 2 N) RS)
) RETC)
))
)
(SETQ RET RETC)

;;;;; ========= First/Last Point =======
(setq PT1 (car ret))		;;;first point
(setq PT2 (car (reverse ret)))	;;;last point

;;;;; ========= Writing Main =======
(setq MTL "" retlen "")

;telli
(foreach n ret
(setq ii (1+ ii))
(if (= ( type n) 'LIST)
(PROGN

(setq retlen (strcat retlen "\n v "
			(rtos (nth 0 n) 2 PREC) " "
			(rtos (nth 2 n) 2 PREC) " "
			(rtos (- 0.0 (nth 1 n)) 2 PREC)
)
)

(if iflag (setq MTL (strcat MTL "\n f " (itoa ii) " " (itoa (1+ ii)) ))
	(setq iifirst ii) ) ;;markup first vertex number
(setq iflag "count verts +1 ")
)
))


(setq MTX (strcat MTX retlen))

;;; ------ String Length Collecting 2/3 -------
(setq MTLen (+ MTLen (/ (strlen retlen) 800.0)))



;;;;; ========= Writing Closing Segment if exists =======
;;; ------ for Ellipses ------
(if (and (= MTE "ELLIPSE") ELC)
(progn
(setq MTL (strcat MTL "\n f " (itoa (1+ ii)) " " (itoa (1+ iifirst)) ) )
(setq ELC nil)	;;; Ellipses closing
))
;;; ------ for Other objects ------
(if (or (member MTE '("LWPOLYLINE" "SPLINE"))
;	(eq lwO (nth 10 AXBaseD))
;	(eq lwO (nth 11 AXBaseD))
	(eq lwO (nth 8 AXBaseD))
	)


(if (or (< (distance PT1 PT2) (/ 1 RF) )
	(= (vla-get-closed lw) -1) (= (vla-get-closed lw) :vlax-true));;tellicad
(setq MTL (strcat MTL "\n f " (itoa (1+ ii)) " " (itoa (1+ iifirst)) ) )
)
)

(if (= MTE "CIRCLE")
(setq MTL (strcat MTL "\n f " (itoa (1+ ii)) " " (itoa (1+ iifirst)) ) )
)

;;; --- 3DFACE ---
(if (= lwO (nth 9 AXBaseD))
(setq MTL (strcat "\ns off\n f " (itoa  (+ ii 1)) " " (itoa (- ii 2)) " " (itoa (- ii 1)) " " (itoa  (- ii 0)) ) )
)

;;; --- 2DSOLID ---
(if (= lwO (nth 12 AXBaseD))
(setq MTL (strcat "\ns off\n f " (itoa  (+ ii 1)) " " (itoa (- ii 1)) " " (itoa (- ii 2)) " " (itoa  (- ii 0)) ) )
)



;;;;; ========= Applying Material =======
(if (= layw "")
;;(setq MTLAY "1D_MAT")
(setq iflag nil MTX (strcat MTX "" MTL ))
(setq iflag nil MTX (strcat MTX "\nusemtl " MTLAY MTL ))
)
;;;(princ "\n MTLAY: ")(princ MTLAY) (princ "  MTL:") (princ MTL)




)
	(setq vs (ssadd MT vs)) ;;; if no verices - skipped
)
)) ;;;LWCHECK
))
;;; GEOM]


	(setq ie (1+ ie))

(setq OpenF 0)
;(if (and (= (GCD ie 5002) 5002) 		;;; by objects/bytes count ))
;(if (= (GCD (1+ (fix MTLen)) 3002) 3002) 	;;; by bytes count)
(if (> (- MTlen MTlenon) spt) 			;;; by bytes split count spt

(progn 
(setq MTLenon MTlen
	iFILE (1+ iFILE))
(1D:Write_OBJ MTX 0 iFILE )
(setq ii 0 id 0 CNT 0 iflag nil MTX "\n" MTL "\n" )
)
)
)	;;;repeat



;;; ====== Collecting Points to one object ======
(setq ie 0 VSP (sslength vssq))
(if (> VSP 2)
	(progn
	(repeat VSP
	(setq MTPP (ssname vssq ie))
	(setq lw (vlax-ename->vla-object MTPP))


		(if (= (cdr (assoc 0 (entget MTPP))) "POINT")
		(progn
		(setq ii (1+ ii))  ;; vertex count
		;(setq MTPP (cdr (assoc 10 (entget MTPP))))
		
		
		(setq MTPP (1D:ZWAL (vlax-get lw 'Coordinates)))
		(if (nth 2 MTPP)
		(setq MTP (strcat 
				(rtos (/ (nth 0 MTPP) RS) 2 PREC) " "
				(rtos (/ (nth 2 MTPP) RS) 2 PREC) " "
				(rtos (/ (- 0.0 (nth 1 MTPP)) RS) 2 PREC)
				))
		(progn
				;(alert "\n1111")
				(setq MTP (strcat 
				(rtos (/ (nth 0 MTPP) RS) 2 PREC) " 0.0 "
				(rtos (/ (- 0.0 (nth 1 MTPP)) RS) 2 PREC)
				))
		)
		
		)
		(SETQ MTP (vl-string-translate "( )" "   " MTP) )
		(setq MTPX (strcat MTPX "\nv " MTP ) )

(if (= (GCD (1+ ie) tstamp) tstamp)
(progn (setq secs (GTIMER "process")) (princ (strcat "\nWriting PointCloud: " (rtos (* 100 (/ (+ 1.0 ie) VSP)) 2 0) "% in " secs) )))

		(setq MTPL (strcat MTPL "\n f 1 " (itoa (1+ ii)) ))
		) )

(if (= (GCD (1+ ie) pstamp) pstamp)	;;;split pointcloud by pstamp-th point

(progn 
(setq MTX (strcat MTX "\no 1D_Pcloud" MTPX MTPL))
(setq MTLen (+ MTLen (/ (strlen (strcat "\no 1D_Pcloud" MTPX MTPL))  800.0) ))


(setq iFILE (1+ iFILE))
(1D:Write_OBJ MTX 0 iFILE)
(setq ii 0 id 0 CNT 0 iflag nil MTX "\n" MTL "\n" MTPL " " MTPX " ")

)
)

		(setq ie (1+ ie) id (1+ id))
	)

(setq MTX (strcat MTX "\no 1D_Pcloud" MTPX MTPL))

;;; ------ String Length Collecting 3/3 -------
(setq MTLen (+ MTLen (/ (strlen (strcat "\no 1D_Pcloud" MTPX MTPL))  800.0) ))
)
)


;;; ===== Writing file =====
(setq iFILE (1+ iFILE))
(1D:Write_OBJ MTX 1 iFILE )
(setq ii 0 id 0 CNT 0 iflag nil CNTP 0 CNTL 0 MTX "\n" MTL "\n" MTLen 0 MTPL " " MTPX " " MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") ) MTFX (strcat (getvar "DWGPREFIX") (getvar "DWGNAME") ))



(princ)
);;defunC























;;;; ========= Blender OBJ to CAD import =========
;;;; =============================================
;;;; =============================================




;;;; ============ REQUIRED AND USEFUL FUNCTIONS ----

;;; ==== REPLACES i-th item in lst with itm ===== >1DZ0
(defun replacenth (lst i itm)
  (setq i (1+ i))
  (mapcar
    '(lambda (x)
      (if (zerop (setq i (1- i))) itm x)
    )
    lst
  )
)



;;; ==== reads ActiveX property of ename/vla-object ==== >1DZ0
(defun 1d_axprop (lw prop /) ;;; reads ax value
(if (= (type lw ) 'ENAME) (setq lw (vlax-ename->vla-object lw)))
(if
(and
(vlax-property-available-p lw prop)	;;    
(not (vl-catch-all-error-p (vl-catch-all-apply '(lambda () (vlax-get-property lw prop) ) )) ) ;;    
)
(progn
(setq lw (vlax-get-property lw prop) )
(if (eq (type lw) 'variant)  (setq lw (vlax-safearray->list (vlax-variant-value lw ) ) ))
)
)
lw
)


;;; ==== Custom Bitcodes convertion ===== >1D2OBJ
;;; returns bitcdes, bitsum and position list (for bits, where positions 1,2,3,4,5,6,7... are bits 1,2,4,8,16,32,64...)
;;; Examples:
;;; (1D_BS  '(1 16 32 64) 0) ---> ((1 5 6 7) . 113)        = ((positions) . bitsum)
;;; (1D_BS '(1 5 6 7)     1) ---> ((1 16 32 64) . 113)     = ((bits) . bitsum)
;;; (1D_BS 113 1)            ---> ((1 16 32 64) (1 5 6 7)) = ((bits) (positions))
;;; xin=0 for bitlist input, xin=1 for poslist input

(defun 1D_BS (x xin / q l ll)
(if (= (TYPE x) 'LIST) 
	(if (= xin 0)
		(progn	;;; bitlist input
			(foreach n x (setq ll (cons (1+ (fix (/ (log n) (log 2)))) ll) ) )
			(setq x (cons (reverse ll) (apply 'logior x)))
		)
		(progn	;;; poslist input
			(if (< (apply 'max x) 32)
				(progn
					(setq x (apply 'append (mapcar '(lambda (n) (list (expt 2 (- n 1) ))) x)))
					(setq x (cons x (apply 'logior x)))
				)
				(princ "\n Limit of 32th position found")
			)
		)
	)
)

(if (= (TYPE x) 'INT)	;;; integer input
(progn 
(setq q 1)
(while (<= q x) (if (> (logand q x) 0) (setq l (cons (logand q x) l))) (setq q (* q 2)) )
(setq ll nil)
(foreach n l (setq ll (cons (1+ (fix (/ (log n) (log 2)))) ll) ) )
(setq x (cons (reverse l) (list ll)))
))
x
)




;;; =========== Creates/tweakes layers ========================= >1D2OBJ
(defun 1D:LAYDEF (LAY LAYSET LAYPLOT LAYCURR  / LC LMAN LMAN1 LMAN2 LT LW)

;;; Example:
;;; (1D:LAYDEF "_" '(15 "Con" 0.30) 0 1)
;;;("NAME" '(color "ltype" lweigth) 1=plotted 1=setcurrent)
;;; ("_" "" 1 0)


;;; ==== UNFREEZE, UNLOCK, TURNING ON  ====
(if (tblsearch "LAYER" LAY)
(progn
(setq LMAN (entget (tblobjname "LAYER" LAY)))

(setq LMAN1 (cdr (assoc 70 LMAN)))		;; frozen // locked
(setq LMAN2 (cdr (assoc 62 LMAN)))		;; color // off 


;; == THAWING if frozen ==
(if (member LMAN1 '(1 4 5) )
	(progn 
		(command "_-LAYER" "_T" LAY "")
		(princ (strcat "\n   *** Layer [" LAY "] THAWED ***" ))
	)
)

;; == UNLOCK if locked ==
(if (member LMAN1 '(1 4 5) )
	(progn 
		(command "_-LAYER" "_U" LAY "")
		(princ (strcat "\n   *** LAYER [" LAY "] UNLOCKED ***" ))
	)
)

;; == SHOWING ON ==
(if (< LMAN2 0)
	(progn 
		(command "_-LAYER" "_ON" LAY "")
		(princ (strcat "\n   *** LAYER [" LAY "] TURNED ON ***" ))
	)
)


(if (= (vl-princ-to-string (TYPE LAYSET)) "LIST")
	(progn
		(setq LC (nth 0 LAYSET))
		(setq LW (nth 2 LAYSET))
		(setq LT (strcase (nth 1 LAYSET)) )
		(if (member LT '("" " " "CON") ) (setq LT "Continuous"))
			(if (= (vl-princ-to-string LAYSET) (vl-princ-to-string
				(list
					(abs (cdr (assoc 62 LMAN))) ;;	laycolor
					(cdr (assoc 6 LMAN)) ;; layltype
					(/ (cdr (assoc 370 LMAN)) 100.0) ;; laywidth
				)))

					(princ (strcat "\n   *** PROPS OF LAYER [" LAY "] __ OK ***" ))
					(progn
						(vl-cmdf "_layer" "_C" LC LAY "")
						(vl-cmdf "_layer" "_L" LT LAY "")
						(vl-cmdf "_layer" "_LW" LW LAY "")
						(princ (strcat "\n   *** PROPS OF LAYER [" LAY "] REDEFINED ***" ))
					)

			)
		
	)
)




(if (and (= LAYPLOT 0) (= 1 (cdr (assoc 290 (entget (tblobjname "LAYER" LAY))))) )
 (PROGN (command "_-LAYER" "_P" "_N" LAY "") (princ (strcat "\n   *** LAYER [" LAY "] turned to UNPLOTTABLE  ***" )))
)
(if (and (= LAYPLOT 1) (= 0 (cdr (assoc 290 (entget (tblobjname "LAYER" LAY))))) ) 
 (PROGN (command "_-LAYER" "_P" "_P" LAY "") (princ (strcat "\n   *** LAYER [" LAY "] turned to PLOTTABLE  ***" )))
)

(if (and (= LAYCURR 1) t)
(progn (setvar "CLAYER" LAY) (princ (strcat "\n   *** LAYER [" LAY "] is CURRENT  ***" ))))
);;progn if found




;;; ======= CREATING =======
(progn 
	(if (= (vl-princ-to-string (TYPE LAYSET)) "LIST")
		(progn
				;if (= (vl-list-length LAYSET) )

			(vl-cmdf "_layer" "_make" LAY "") (princ (strcat "\n   *** LAYER [" LAY "] not found and CREATED  ***" ))
			(setq LC (nth 0 LAYSET))				(vl-cmdf "_layer" "_C" LC LAY "")
			(setq LW (nth 2 LAYSET))				(vl-cmdf "_layer" "_LW" LW LAY "")
			(setq LT (strcase (nth 1 LAYSET)) )
			(if (member LT '("" " " "CON") ) (setq LT "Continuous"))
			(vl-cmdf "_layer" "_L" LT LAY "")
(if (= LAYPLOT 0)
(PROGN (command "_-LAYER" "_P" "_N" LAY "") (princ (strcat "\n   *** LAYER [" LAY "] turned UNPLOTTABLE  ***" )))
(PROGN (command "_-LAYER" "_P" "_P" LAY "") (princ (strcat "\n   *** LAYER [" LAY "] turned PLOTTABLE  ***" )))
)
(if (= LAYCURR 1) (PROGN (setvar "CLAYER" LAY) (princ (strcat "\n   *** LAYER [" LAY "] turned CURRENT  ***" ))))

		)
		(princ (strcat "\n** LAYER [" LAY "] DID NOT FOUND, please, set properties list to create it **"))
	)
);progn2
);IF

(princ)
)


;;; === Code by Lee Mac http://www.lee-mac.com/programs.html ===

;;; List Flatten 
(defun LM:Flatten ( l )
  (if l
    (if (atom l) (list l)
      (append (LM:Flatten (car l)) (LM:Flatten (cdr l)))
    )
  )
)


(defun LM:Unique ( l )
    (if l (cons (car l) (LM:Unique (vl-remove (car l) (cdr l)))))
)


(defun 1DLM:AddMesh ( space m n lst )
  ;;  Lee Mac  ~  16.06.10
  (if (and (<= 2 m 256) (<= 2 n 256))
    (vla-Add3DMesh space m n (LM:3DPointVariant lst))
  )
)





(defun LM:3DPointVariant ( ptLst )
  ;;  Lee Mac  ~  16.06.10
  (vlax-make-variant
    (vlax-safearray-fill
      (vlax-make-safearray vlax-VBDouble
        (cons 0 (1- (* 3 (length ptLst))))
      )
      (apply 'append ptLst)
    )
  )
)




;;; http://www.theswamp.org/index.php?topic=43444.0
(defun ent-mesh (vertices faces edges / header ent-list)
;;Entity header:
(setq header '((0 . "MESH")
(100 . "AcDbEntity")
(100 . "AcDbSubDMesh")
(71 . 2)
(72 . 0)
(91 . 0)))
;;Vertices data:
(setq ent-list (cons (cons 92 (length vertices)) ent-list))
(foreach vertex vertices
(setq ent-list (cons (cons 10 vertex) ent-list)))
;;Faces data:
(setq
ent-list (cons (cons 93
(+ (length faces)
(length (apply 'append faces))))
ent-list))
(foreach face faces
(setq ent-list (cons (cons 90 (length face)) ent-list))
(foreach vertexindex face
(setq ent-list (cons (cons 90 vertexindex) ent-list))))
;;Edges data:
(setq ent-list (cons (cons 94 (/ (length edges) 2)) ent-list))
(foreach vertexindex edges
(setq ent-list (cons (cons 90 vertexindex) ent-list)))
;;Crease data:
(setq ent-list (cons (cons 95 (/ (length edges) 2)) ent-list))
(setq ent-list (append header (reverse ent-list)))
(entmake ent-list))








;;; ==== FLATTENS selected lines ====
(defun C:1DZ0 (/ ix MT stz enz lst i lw 
;;;  Paul Kotelevets aka 1D_Inc aka 1D Inciner (10.2013)
;;; replacenth
;;; 1d_axprop
)
(setq ix (getint "\nAXIS to Flatten: 0=x 1=y 2=z <2>: "))
(if (not ix) (setq ix 2))
(setq lst (vl-remove ix '(0 1 2)) )

(setq vss (ssget '((0 . "LINE"))) i 0 )
(if (not vs) (setq vs (ssadd)))
(repeat (sslength vss)
(setq MT (ssname vss i))
(setq lw (vlax-ename->vla-object MT))
(setq stz (1d_axprop lw 'startpoint))
(setq enz (1d_axprop lw 'endpoint))

(if
(and
(equal (nth (nth 0 lst) stz) (nth (nth 0 lst) enz) 0.0001)
(equal (nth (nth 1 lst) stz) (nth (nth 1 lst) enz) 0.0001)
)
;;(setq vs (ssadd MT vs))
(ENTDEL mt)

(progn
(vla-put-startpoint lw (vlax-3d-point (trans (replaceNth stz ix 0.0) 1 0) ))
(vla-put-endpoint lw (vlax-3d-point (trans (replaceNth enz ix 0.0) 1 0) ))
)

)
(setq i (1+ i))
)
(C:33)
)






;;; ================== MAIN FUNCTION ======== 25.10.2013
;;; --------------------- by 1D_Inc --------------------
;;; ====================================================
;;; ====================================================
	;;; pline$$


(defun C:1D2OBJ (/ ;;; bypass BLENDER OBJ import
;;;  Paul Kotelevets aka 1D_Inc aka 1D Inciner (10.2013 - 01.2020)
DBV ;vertices list 
FLIST ;biuld list
;DBFLIST ; instructions list
DBF DBX DBR DBLX f l x timx timy kk ixl ic FUZ VRS 1d_MkLINE UVX ir
A DBF DBLX DBV DBVV DBVX F FILENAME  FLX FUZ FX I IL IR IX KK L LAY LF LL MSPACE MT S TIMX UVX VRS X

;;;==== Requres ====
;;; 1D:LAYDEF
;;; 1D_BS
)

(vl-load-com)

(princ "\nver = v5.17\n")

;;; ==== MAKE LINES from point datalist local function ====


(defun 1d_MkLINE (DBX /)
(if (and DBX DBV)
  (foreach n DBX
    (if	(and n (nth (1- (car n)) DBV) (nth (1- (cadr n)) DBV) )
      (progn
	(entmake
	  (list
	    '(0 . "LINE")
	    (cons 10 (nth (1- (car n)) DBV))
	    (cons 11 (nth (1- (cadr n)) DBV))
	    '(210 0.0 0.0 1.0)
	  )
	)

      )
    )
  )
)
  ;; foreach
)


(defun 1d_MkPLINE (DBX elevs /) ;;5.09
(if (and DBX DBV)
(entmake-pline-elev (mapcar '(lambda (q) (nth (1- q) DBV) ) DBX) elevs)
)
  ;; foreach
)


;;;   create build polyline
(defun entmake-pline-elev (lst elevs)
  ;; lst -       
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.)))
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.) (100. 0.)))

(if (= elevs 1)
  (entmakex (append (list '(0 . "LWPOLYLINE")
                          '(100 . "AcDbEntity")
                          '(100 . "AcDbPolyline")
                          (cons 90 (length lst))
			  (cons 38 (last (last lst)))
                          ) ;_ end of list
                    (mapcar '(lambda (x) (cons 10 x)) lst)
                    ) ;_ end of append
            ) ;_ end of entmakex

(entmakex (append (list '(0 . "LWPOLYLINE")
                          '(100 . "AcDbEntity")
                          '(100 . "AcDbPolyline")
                          (cons 90 (length lst))
                          ) ;_ end of list
                    (mapcar '(lambda (x) (cons 10 x)) lst)
                    ) ;_ end of append
            ) ;_ end of entmakex

); if
  ) ;_ end of defun

;|  ;;pline$$
(defun PELES (/ db FX)
(setq DBL (vl-sort DBL (function (lambda (a b) (<  (car a) (car b) ) ) )) )
(setq db (mapcar 'cadr DBL) fx nil)
(foreach n DBL)


)
|;

;|
(defun 1d_MkLINE (DBX mspace /)

  (foreach n DBX
    (if	n
      (progn
	(vla-addline mspace (vlax-3d-point (nth (1- (car n)) DBV) )  (vlax-3d-point (nth (1- (cadr n)) DBV) )    )
      )
    )
  )
  ;; foreach
) 
|;

(setvar "OSMODE" 0)
(setq mspace (vla-get-modelspace (vla-get-ActiveDocument (vlax-get-acad-object))) )

;;; ==== GET PROGRAM NAME ====
(setq VRS (strcase (getvar "PROGRAM")) DBR nil idbr 0 RD nil)


(if $$1DOBJ  (progn (princ "\n ___________\n") (princ $$1DOBJ) (setq RD (getstring "\n Load Previous Settings? <space = yes, anykey = no>")) ))


(if (= RD "")

(mapcar 'set (list 'FUZ 'kk 'FLX 'UVX) $$1DOBJ)

(progn
;;; ===== GET CONDITIONS ====
(setq FUZ (getint "\n PRECISION? <8>"))
(if (not FUZ ) (setq FUZ 8))

(setq kk (getreal "\nScale up? <1000> :"))
(if (not kk) (setq kk 1e3))
(setq FLX (getstring "\nGeomerty mode: 1= mesh, 2=lines, 3= points, 4= 3dfaces 5 = plines flat 6 = plines elev <2>: "))
(IF (= FLX "") (SETQ FLX "2"))
(setq UVX (getstring "UVX? (space = no, anykey = yes) <no>"))
(if (= UVX "") (setq UVX 'V) (setq UVX 'VT))

(setq $$1DOBJ (list  FUZ kk FLX UVX) )
))

;;;; ==== convert "123456" to '(1 2 3 4 5 6) ====
(setq FLX (read (strcat "("
(vl-list->string (apply 'append (mapcar '(lambda (a)  (list a 32))  (vl-string->list FLX) ) )  )
")" )) )

(setq FLX (cdr (1d_bs FLX 1) )) ;;; getting local summ

;;; ==== FILEREAD ====
(setq filename (getfiled " " "" "obj" 0))
(if filename
(setq f (open filename "r"))
(princ "\n  "))

(if (= VRS "ACAD")
(progn
(if (> (setq s (vl-file-size  filename)) 400000.0)
(progn
(princ (/ S 20000.0) ) (princ " seconds")
(getstring "\n Sure?")
)
)
(setq timx (car (_VL-TIMES)))
)
)


(princ "\nReading FILE....") (print filename)
(setq DBV nil DBF nil DBFLIST nil DBVV nil DBVX nil DBLX nil DBL nil i 0 ix 0 il 1)





(while
(setq s (read-line f))
;;(setq l (read (strcat "( " s " )")))

;;; ==== READ DOTS in material names ==== 
(if (wcmatch s "*usem*" )
(setq ll (read  (strcat "(" (vl-string-translate "'" " " (vl-string-translate "." "_" s ) ) ")")))
(setq ll (read  (strcat "(" (vl-string-translate "'" " " s ) ")")))
)
(setq l ll)


;;(princ "\nl = ") (princ l)

(if (= (car l) 'O )

(progn
(setq DBLX (cons i DBLX) fx l)
;|
	(if (= 16 (logand 16 FLX))	;; plines
		()
	)
|;
)
)




(if (= (car l) UVX )
(progn
(setq l (cdr l))		;;; $$$last
(if (> (length l) 2) (setq l (list (nth 0 l) (- (nth 2 l)) (nth 1 l) )))		;;; YZ -ZY transformations
(setq DBV (append DBV (list (mapcar '(lambda (a) (* (read (rtos a 2 FUZ)) kk)) l ))  ) i (1+ i)) 
(if (= 4 (logand 4 FLX)) (setq DBVX (append DBVX (list (mapcar '(lambda (a) (* (read (rtos a 2 FUZ)) kk)) l ))  )) )  ;;; LOCAL VERTS for verts
)
)  ;;; GLOBAL VERTS for lines/faces



;;;; ================ LAYERS ========================

(if (= (car l) 'USEMTL )
(progn (setq LAY (vl-princ-to-string (cdr l)) )
;;(princ "\n") (princ L)
(if (tblsearch "LAYER" LAY)
(setvar "CLAYER" LAY)
(progn
(if (> il 10) (setq il (+ 3 il)) (setq il (+ 1 il)) )
(setq il (/ (apply '+ (vl-string->list LAY)) 250.0))
(setq il (fix (* 250 (- il (fix il)))) )
(if (> il 253) (setq il (- il 242 )) )
(1D:LAYDEF LAY (list il "Con" 0.00) 1 1)
)

;| color cycle test
(setq il 1)
(repeat 500
(if (> il 10) (setq il (+ 3 il)) (setq il (+ 1 il)) )
(if (> il 253) (setq il (- il 242 )) )
(princ "\n")
(princ il)
)
|;

))
)


(if  (or (= (car l) 'F ) (= (car l) 'L ) 
(= (car l) 'V ) ;;$$
)

(progn 
(if
(or
(= 2 (logand 2 FLX))	;; lines
(= 1 (logand 1 FLX))	;; faces mesh
(= 8 (logand 8 FLX))	;; faces face
(= 4 (logand 4 FLX))	;; points?
(= 16 (logand 16 FLX))	;; plines
(= 32 (logand 32 FLX))	;; plines elev
)

(progn
(if (vl-symbolp (cadr l))
	(progn 
	(setq FLIST
	(mapcar (function (lambda (x)	;;; SYMBOL-STRING-VALUE conversion
(if (= UVX 'V)
	(read (substr (vl-symbol-name x) 1  (vl-string-search "/" (vl-symbol-name x) )) )					;;; first part = FACE (UV)
	(read (substr (vl-symbol-name x) (+ 2 (vl-string-search "/" (vl-symbol-name x) )) (strlen (vl-symbol-name x)) ))	;;; last part = UV
)
	)) (cdr l))
	) )
	(setq FLIST (cdr l))
)
)
)

(setq DBFLIST (cons FLIST DBFLIST))


;;; ========= PLINES ==============[[
(if (= 16 (logand 16 FLX))	;;; pline$$ 
;;(setq DBL (cons (cdr l) DBL))
(1d_MkPLINE FLIST 0)

)


(if (= 32 (logand 32 FLX))	;;; pline$$ 
;;(setq DBL (cons (cdr l) DBL))
(1d_MkPLINE FLIST 1)

)
;;; ========= PLINES ==============]]



;;; ========= LINES ==============[[
(if (= 2 (logand 2 FLX))	;;; lines
(progn
	(setq l FLIST)
	(if (> (length l) 2)
	(progn (setq MT (mapcar 'list l (append (cdr l) (list (car l))) ) ) )	;; pairwising ; thanks to Lee Mac
	(if (= (length l) 2) (progn (setq MT (list l)) ) )
	)
	
;;	(princ "\nMT = ") (princ MT)

(1d_MkLINE MT)
) )
;;; ========= LINES ==============]]




;;; ========= FACES/MESH ==============[[
(if (or (= 1 (logand 1 FLX)) (= 8 (logand 8 FLX))) 	;;; mesh/faces
(progn
(setq l FLIST)


;;; === MESH SUBD ENTMAKE ===
;;; =========================
(if 
(and  (= 1 (logand 1 FLX)) DBV)
(progn
(if 
(and (> (length l) 2)
	(EQUAL									;;; ONLY EQUAL operator 
	(mapcar '(lambda (x) (nth (1- x) DBV) ) l)				;;; if gathered list	$$$
	(vl-remove-if 'null (mapcar '(lambda (x) (nth (1- x) DBV) ) l) )	;;; equal to cleaned (not containing nil values)
	)
)

(progn
(setq ir -1)
(ent-mesh
(mapcar '(lambda (x) (nth (1- x) DBV) ) l)					;;; verticles list from DBV
(setq lf (list (mapcar '(lambda (x) (setq ir (1+ ir)) ) l)))			;;; faces '((0 1 2 3))
(LM:Flatten (mapcar 'list lf (append (cdr lf) (list (car lf))) ))		;;; edges '(0 1 1 2 2 3 3 0) $$$
)

)
)
))


;;; === 3DFACE ENTMAKE ===
;;; ======================
(if 
(and  (= 8 (logand 8 FLX)) DBV) ;;; UV protection

(progn
(if 
(and 	(= (length l) 4)
	(nth (1- (nth 0 l)) DBV)
        (nth (1- (nth 1 l)) DBV)
	(nth (1- (nth 2 l)) DBV)
        (nth (1- (nth 3 l)) DBV)
)
(progn

(setq DBR (cons (list LAY (nth (1- (nth 0 l)) DBV) (nth (1- (nth 1 l)) DBV) (nth (1- (nth 2 l)) DBV) (nth (1- (nth 3 l)) DBV)) DBR)  idbr (1+ idbr))
;|
	(entmake (list '(0 . "3DFACE") ;***
        '(6 . "BYLAYER")
        ; '(8 . "0")
        (cons 10 (nth (1- (nth 0 l)) DBV) ) ;***
        (cons 11 (nth (1- (nth 1 l)) DBV) ) ;***
        (cons 12 (nth (1- (nth 2 l)) DBV) ) ;***
        (cons 13 (nth (1- (nth 3 l)) DBV) ) ;***
        '(39 . 0.0)
        '(62 . 256)
        '(70 . 0))
	)

|;
)
)


(if 
(and 	(= (length l) 3)
	(nth (1- (nth 0 l)) DBV)
        (nth (1- (nth 1 l)) DBV)
	(nth (1- (nth 2 l)) DBV)
)
(progn

(setq DBR (cons (list LAY (nth (1- (nth 0 l)) DBV) (nth (1- (nth 1 l)) DBV) (nth (1- (nth 2 l)) DBV) (nth (1- (nth 2 l)) DBV)) DBR) idbr (1+ idbr) )

;|
	(entmake (list '(0 . "3DFACE") ;***
        '(6 . "BYLAYER")
        ; '(8 . "0")
        (cons 10 (nth (1- (nth 0 l)) DBV) ) ;***
        (cons 11 (nth (1- (nth 1 l)) DBV) ) ;***
        (cons 12 (nth (1- (nth 2 l)) DBV) ) ;***
        (cons 13 (nth (1- (nth 2 l)) DBV) ) ;***
        '(39 . 0.0)
        '(62 . 256)
        '(70 . 0))
	)
|;

)
)
))


;|[
;;; === POLYFACE VLA-ADD ===
(if (= (length l) 4)
(progn

(setq xx      (list (nth (1- (nth 0 l)) DBV)  ;***
        	(nth (1- (nth 1 l)) DBV)  ;***
	        (nth (1- (nth 3 l)) DBV)  ;***
        	(nth (1- (nth 2 l)) DBV)  ;*** 
		)
)
(LM:AddMesh mspace 2 2 xx)
)
)
]|;


(if (= (length l) 2)  (1d_MkLINE (list l)) )	;;; addictive lines

)) ;; if faces
;;; ========= FACES/MESH ==============]]


;;; ========= POINTS ==============[[
(if (= 4 (logand 4 FLX))
(progn
;;(princ "\nmaking POINTS...")
;;;;; ===== Entmake points ==========
(setq l FLIST)

(foreach n l		;;$$$
(entmake
   (list 
        '(0 . "POINT")
        (cons 10 (nth (1- n) DBV))
        '(210 0.0 0.0 1.0)
    )
)
)



;|
(foreach n DBVx		;;$$$
(entmake
   (list 
        '(0 . "POINT")
        (cons 10 n )
        '(210 0.0 0.0 1.0)
    )
)
)
|;
(setq DBVx nil)
)
)
;;; ========= POINTS ==============]]



))	;; (if (= (car l) 'F ) ...)

(if (> idbr 333)
(progn
(princ "\n Biulding 3DFACES   ")

(if (= VRS "ACAD")
(princ (strcat "at "
                (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4)
                " secs."
        ) ;_  strcat
))

(mapcar '(lambda (q)
(entmake (list '(0 . "3DFACE") ;***
        '(6 . "BYLAYER")
        (cons 8  (car q))
        (cons 10 (nth 1 q) ) ;***
        (cons 11 (nth 2 q) ) ;***
        (cons 12 (nth 3 q) ) ;***
        (cons 13 (nth 4 q) ) ;***
        '(39 . 0.0)
        '(62 . 256)
        '(70 . 0))
)
) 
DBR)
(setq idbr 0 DBR nil)
))

);; while


;;; ============= BUILDING FACES

;|
(setq id 0)
(repeat 500 (setq DBF (cons (nth (setq id (1+ id) ) DBR) DBF) ) )

|;


(if DBR
(progn
(princ "\n Biulding 3DFACES   ")

(if (= VRS "ACAD")
(princ (strcat "at "
                (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4)
                " secs."
        ) ;_  strcat
))

(mapcar '(lambda (q)
(entmake (list '(0 . "3DFACE") ;***
        '(6 . "BYLAYER")
        (cons 8  (car q))
        (cons 10 (nth 1 q) ) ;***
        (cons 11 (nth 2 q) ) ;***
        (cons 12 (nth 3 q) ) ;***
        (cons 13 (nth 4 q) ) ;***
        '(39 . 0.0)
        '(62 . 256)
        '(70 . 0))
)
) 
DBR)
(setq DBR nil)
))


;;;

(princ "\n Vertices = ")
(princ (length DBV))
(princ "\nFile = ")
(princ filename)
(princ "\n Done...")
(if (= VRS "ACAD")
(princ (strcat "at "
                (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4)
                " secs."
        ) ;_  strcat
))

(princ)
) ;;;; defun C:1D2OBJ















;;;; ========== Previous version (without layers) =========

(defun C:OBJJ (/	;;; old featured :full scan
;;;  Paul Kotelevets aka 1D_Inc aka 1D Inciner (10.2013)
DBV DBF DBX DBLX 
FILENAME FLX I IX KK MT VRS
f l x timx timy kk ixl ic
)

;;; ==== GET PROGRAM NAME ====
(setq VRS (strcase (getvar "PROGRAM")))

(setvar "OSMODE" 0)
(setq kk 1e3)
(setq FLX (getstring "\nGeomerty mode: 1= faces, 2=lines, 3= points <2>: "))
(IF (= FLX "") (SETQ FLX "2"))


;;;; convert "1234" to '(1 2 3 4)
(setq FLX (read (strcat "("
(vl-list->string (apply 'append (mapcar '(lambda (a)  (list a 32))  (vl-string->list FLX) ) )  )
")" )) )

(setq FLX (cdr (1d_bs FLX 1) )) ;;; getting local summ

;FILEREAD
(setq filename (getfiled " " "" "obj" 0))
(if filename
(setq f (open filename "r"))
(princ "\n  "))

;;; ==== GET PROGRAM NAME ====
(if (= VRS "ACAD")
(progn
(if (> (setq s (vl-file-size  filename)) 400000.0)
(progn
(princ (/ S 20000.0) ) (princ " seconds")
(getstring "\n Sure?")
))
(setq timx (car (_VL-TIMES)) timy timx)

)
)


(setq DBV nil DBF nil DBLX nil i 0 ix 0)
(princ "\nreading FILE....")

(while
(setq s (read-line f))
;;(setq l (read (strcat "( " s " )")))
(setq l (read  (strcat "(" (vl-string-translate "'()" "   " s ) ")")))

(if (= (car l) 'O ) (setq DBLX (cons i DBLX)))
(if (= (car l) 'V ) (setq DBV (cons (mapcar '(lambda (a) (* a kk)) (cdr l) )  DBV) i (1+ i)))

(if (= (car l) 'F )
	(if (vl-symbolp (cadr l))
		(setq DBF (cons 
(mapcar (function (lambda (x)	;;; SYMBOL-STRING-VALUE conversion
;;(read (substr (vl-symbol-name x) (+ 2 (vl-string-search "/" (vl-symbol-name x) )) (strlen (vl-symbol-name x)) ))	;;; last part = UV
(read (substr (vl-symbol-name x) 1  (vl-string-search "/" (vl-symbol-name x) )) )					;;; first part = FACE
)) (cdr l))
			 DBF))
		(setq DBF (cons (cdr l) DBF))
	)
)
)

(setq DBV (reverse DBV) DBF (reverse DBF) DBLX (vl-sort DBLX '<) )	;;
(setq DBX nil)					;; edges collection
(setq vs (ssadd))

(if (= VRS "ACAD")
(progn
(princ (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4))
(princ " = ")
(princ (rtos (/ (- (car (_VL-TIMES)) timy) 1000.) 2 4))
(setq timy (car (_VL-TIMES)))
))




(if (= 4 (logand 4 FLX))
(progn
(princ "\nmaking POINTS...")
;;;;; ===== Entmake points ==========
(foreach n DBV
(entmake
   (list 
        '(0 . "POINT") 
        (cons 10 n ) 
        '(210 0.0 0.0 1.0) 
    )
)
(setq vs (ssadd (entlast) vs ))
)

(if (= VRS "ACAD") (progn
(princ (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4))
(princ " = ")
(princ (rtos (/ (- (car (_VL-TIMES)) timy) 1000.) 2 4))
(setq timy (car (_VL-TIMES)))
)
)

))


(if (= 1 (logand 1 FLX))
(progn
(princ "\nmaking FACES....")
;;;;; ===== Entmake faces ==========
(foreach n DBF

(if n
(progn
(if (= (length n) 4) (progn
(setq mt n)
(entmake (list '(0 . "3DFACE") ;***
               '(6 . "BYLAYER")
              ; '(8 . "0")
               (cons 10 (nth (1- (nth 0 n)) DBV) ) ;***
               (cons 11 (nth (1- (nth 1 n)) DBV) ) ;***
               (cons 12 (nth (1- (nth 2 n)) DBV) ) ;***
               (cons 13 (nth (1- (nth 3 n)) DBV) ) ;***
               '(39 . 0.0)
               '(62 . 256)
               '(70 . 0))
)
 (setq vs (ssadd (entlast) vs )))


)
(if (= (length n) 3) (progn
(setq mt n)
(entmake (list '(0 . "3DFACE") ;***
               '(6 . "BYLAYER")
              ; '(8 . "0")
               (cons 10 (nth (1- (nth 0 n)) DBV) ) ;***
               (cons 11 (nth (1- (nth 1 n)) DBV) ) ;***
               (cons 12 (nth (1- (nth 2 n)) DBV) ) ;***
               (cons 13 (nth (1- (nth 2 n)) DBV) ) ;***
               '(39 . 0.0)
               '(62 . 256)
               '(70 . 0))
)
 (setq vs (ssadd (entlast) vs )))


)
)
)



)

(if (= VRS "ACAD") (progn
(princ (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4))
(princ " = ")
(princ (rtos (/ (- (car (_VL-TIMES)) timy) 1000.) 2 4))
(setq timy (car (_VL-TIMES)))
))

)
)



(if (= 2 (logand 2 FLX))
	(progn

(defun 1d_MkLINE (DBX / )
(foreach n DBX
(if n (progn
(entmake
   (list
        '(0 . "LINE")
        (cons 10 (nth (1- (car n)) DBV) )
        (cons 11 (nth (1- (cadr n)) DBV) )
        '(210 0.0 0.0 1.0)
    )
)
 (setq vs (ssadd (entlast) vs ))
))
);; foreach
)

	(setq ix 1 ixl 0 ic 0)
	(princ "\nreading EDGES...")
	;;; ======== Flattening DBF to DBX ==========
	(foreach l DBF
		(if (> (length l) 2)
			(progn
			(setq MT (mapcar 'list l (append (cdr l) (list (car l))) ) )	;; pairwising ; thank you Lee
			(setq MT (apply 'append  (mapcar (function (lambda (x)(if (or (member x DBX) (member (reverse x) DBX)) (list) (list x)))) MT)))
			(setq DBX (append MT DBX) ixl (1+ ixl))
			)
		(if (= (length l) 2)
			(progn
			(setq MT (list l))
			(setq MT (apply 'append  (mapcar (function (lambda (x)(if (or (member x DBX) (member (reverse x) DBX)) (list) (list x)))) MT)))
			(setq DBX (append MT DBX) ixl (1+ ixl))
			)
			
		)
		)

			;;;;; ===== Entmake lines ==========
			(if (= (gcd ix 503) 503)	;;; buffer limiting
			(progn

			(setq MT (caar DBX))
			(setq ic (car (vl-sort-i DBLX (function (lambda (e1 e2) (< (abs (- MT e1)) (abs (- MT e2)) ) ) ))))
			(if (> ic 253) 
			(setq ic (fix (* (- (/ ic 253.0) (fix (/ ic 253.0)) ) 253) ))		;; IC COLOR ROUNDOFF
			(setq ic (1+ ic)))


			(princ "\nmaking LINES...")
		(if (= VRS "ACAD") (progn
			(princ (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4))
			(princ " = ")
			(princ (rtos (/ (- (car (_VL-TIMES)) timy) 1000.) 2 4))
			(setq timy (car (_VL-TIMES)))
		))
			
			(1d_MkLINE DBX)
			(command "_CHPROP" vs "" "_C" ic "")
			(setq DBX nil vs (ssadd))
			))

(setq ix (1+ ix))
	)

;;;;; ===== Entmake lines ==========
(if DBX 
(progn
(princ "\nmaking LINES...")
(if (= VRS "ACAD") (progn
(princ (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4))
(princ " = ")
(princ (rtos (/ (- (car (_VL-TIMES)) timy) 1000.) 2 4))
(setq timy (car (_VL-TIMES)))
))

(1d_MkLINE DBX)
;;(setq DBX nil)
))

)) ;;; ifprogn logand



(princ "\nVerts:  ")
(princ (length DBV))
(princ "\nFaces:  ")
(princ (length DBF))
(princ "\nEdges:  ")
(princ ixl)

(princ "\n....DONE ")

(if (= VRS "ACAD") (progn
(princ (strcat "at "
                (rtos (/ (- (car (_VL-TIMES)) timx) 1000.) 2 4)
                " secs."
        ) ;_  strcat
)
))

(princ)
) ;;; defun C:OBJJ

(defun vpc ( val ) (setq val (vl-princ-to-string  val)) val )
(defun VCE ( obj /) (if (= (type obj) 'ENAME) (setq obj (vlax-ename->vla-object obj)) (setq obj (vlax-vla-object->ename obj)) ))
(DEFUN remove-doubles (lst) (IF lst (CONS (CAR lst) (remove-doubles (VL-REMOVE (CAR lst) lst))) ))
(defun LM:LWVertices ( e ) (if (setq e (member (assoc 10 e) e)) (cons (CDR (assoc 10 e)) (LM:LWVertices (cdr e)) ) ) )
(defun 1DLM:LWVertices (e gr) (if (setq e (member (assoc gr e) e)) (cons (CDR (assoc gr e)) (1DLM:LWVertices (cdr e) gr) ) ) )

;;; ================ 1DM MESH section
(defun 1DLM:MSHEdges (e gr flg) (if (and (setq e (member (assoc gr e) e) ) (member (assoc flg e) e) )
(cons (CDR (assoc gr e)) (1DLM:MSHEdges (cdr e) gr flg) )
))

(defun LM:REGroupByNum ( l / r ) ;; no count control
    (if (setq n (car l)) 
	(progn (setq l (cdr l))
        (cons
            (reverse (repeat n (if (car l) (setq r (cons (1+ (car l) ) r) l (cdr l))) r))
            (LM:REGroupByNum l)
        )
	)
    )
)
(defun LM:REGroupByNum-c ( l nn / r )	;; count control
(if (and (setq n (car l))  (> nn 0) ) (progn (setq l (cdr l)) (cons (reverse (repeat n (setq r (cons (1+ (car l)) r) l (cdr l) nn (- nn 1)) r)) (LM:REGroupByNum-c l nn) ))))







(defun C:1DM ( / MT MTX MTE MTV F FL MTED MTF MTT N Q VS Z RS) ;;; read mesh 16:14 13.01.2015






;;; ========= Input data convert conditions ===========
(setq RS (getreal	"\nRS (Reduce scale for scene) <1000>:"))

;;; ========= Checking Input data convert conditions ===========
(if (not (eq (type RS) 'REAL)) (setq RS 1000.0))



(setq vs (ssget '((0 . "MESH"))) ix 0 id -1 MTT "")
(stimer)
(repeat (sslength vs)
(setq MT (ssname vs (setq id (1+ id)) ))
;; ============== GATHER DATA ===============
(setq MTX (entget MT))
(setq MTV (1DLM:LWVertices MTX 10) )		;; ((226.097 11.6825 22.361) (246.687 29.2151 26.2868))
(setq MTVR nil)
(foreach nn MTV (setq MTVR (cons (mapcar '(lambda (q) (/ q RS)) nn) MTVR) ) )
(setq MTV (reverse MTVR))


;;(setq MTED (1DLM:MSHEdges MTX 90 94) )		;; ([3] 0 1 2 [3] 3 4 5 [3] 6 7 8)
(setq MTED nil ir -1 z nil)
(while (not (eq (car z) 94)) (setq ir (1+ ir) z (nth ir MTX)) (if (= (car z) 90) (setq MTED (cons (cdr z) MTED) )) )
(setq MTED (reverse MTED))

(setq MTE (LM:REGroupByNum MTED))		;; ((1 2 3) (4 5 6) (7 8 9)) +1 incremented
(setq MTE (mapcar '(lambda (q) (mapcar '(lambda (z) (+ ix z)) q) ) MTE))	;; global index increment apply
(setq ix (+ ix (length MTV)))
;(princ "\n == ") (princ (gtimer 1))

;; ============== FORM OVERALL TEXT =========
(setq MTT (strcat  MTT "\n\n" "o 1DMESH_" (itoa id)
(apply 'strcat (mapcar '(lambda (q) (strcat "\nv " (rtos (nth 0 q) 2 8 ) " "  (rtos (nth 2 q) 2 8 ) " "  (rtos (- (nth 1 q)) 2 8 ))) MTV ) )
"\nusemtl None"
"\ns off"
(apply 'strcat (mapcar '(lambda (q)  (strcat "\nf " (apply 'strcat (mapcar '(lambda (z) (strcat (itoa z) " " )) q)) )) MTE))
)
)
)	;; repeat

(princ "\n ===== ") (princ (gtimer 1))
(setq MTF (strcat (getvar "DWGPREFIX") (getvar "DWGNAME")))
(setq fl (vl-filename-mktemp MTF (strcat (getvar "dwgprefix")) ".obj"))
   (setq f (open fl "w"))
   (write-line MTT f)
   (close f)
)




(princ "\n======================================
\nFunctions:
\n=== Misc:
\n ][ - parse DXF of selected object
\n ][P - parse ActiveX of selected object
\n 1DZ0 - flatten lines 
\n 23 33 - set and get memslot 3
\n 24 44 - set and get memslot 4
\n=== Geometry generators:
\n 1DWG - grow walls from polylines sketch
\n 1DWF - set height database from polylines sketch
\n 1DBQ - draw balconies
\n 1DRQ - generate railings
\n=== Export-import functions:
\n 1D2B - export OBJ
\n 1DBB - export blocks, 1DFEX - export 3DFACES, 
\n 1D2OBJ - import OBJ
\n 1DM - export meshes
\n======================================
\nCad <-> Blender converter v5.17 is loading.
\n======================================
")
(princ)






(setq 1DTRANSLITBASE 
'(
("" . "A") ("" . "B") ("" . "V")
("" . "G") ("" . "D") ("" . "E")
("" . "J") ("" . "Z") ("" . "I")
("" . "I") ("" . "K") ("" . "L")
("" . "M") ("" . "N") ("" . "O")
("" . "P") ("" . "R")("" . "S")
("" . "T") ("" . "Y") ("" . "F")
("" . "H") ("" . "C") ("" . "CS")
("" . "SH") ("" . "SH") ("" . "'")
("" . "Y") ("" . "'") ("" . "E")
("" . "U") ("" . "A")

("" . "a") ("" . "b") ("" . "v")
("" . "g") ("" . "d") ("" . "e")
("" . "j") ("" . "z") ("" . "i")
("" . "i") ("" . "k") ("" . "l")
("" . "m") ("" . "n") ("" . "o")
("" . "p") ("" . "r") ("" . "s")
("" . "t") ("" . "y") ("" . "f")
("" . "h") ("" . "c") ("" . "cs")
("" . "sh") ("" . "sh") ("" . "'")
("" . "y") ("" . "'") ("" . "e")
("" . "u") ("" . "a")
)
)



(defun 1DLatin (str trs / ) ;; translation ability
(if (= trs 1) 
(setq str (vl-list->string 
(mapcar '(lambda (q)
(if (assoc (chr q) 1DTRANSLITBASE)
(ascii (cdr (assoc (chr q) 1DTRANSLITBASE)))
q
))
(vl-string->list str)
)))
)

(vl-list->string (mapcar '(lambda (q) (if (not (<= 47  q 122) ) 
(cond
((= q 32) 95)
((<= 38  q 122) q) ;;;$$$$45
((<= 192  q 223) (fix (1D_INP  q '((192.0 . 223.0) (65.0 . 90.0) ))) )
( (<= 224  q 255) (fix (1D_INP  q '((224.0 . 255.0) (97.0 . 122.0) ))) )
(t (fix (1D_INP  q '((0.0 . 191.0) (97.0 . 122.0) ))) )
)
  q)) (vl-string->list str) )) ;; interpolatin to Latin*


) ;;; defun


(defun 1D:Write_FILE (MTX OpenF frmt tmp / )
;;; ===== Writing file header =====
(if (= OpenF 1) (setq secs (GTIMER "end")) (setq secs (GTIMER "process")))
(princ secs )

(setq MTX (strcat 
"# File: '"
MTFX

"'
# DWG2OBJ4Blender_Converter v5.17 by Paul Kotelevets (1D_Inciner) 09.10.2012-2022
# http://blenderartists.org/forum/showthread.php?267990-Direct-Autocad-Intellicad-to-Blender-exporter-LISP-to-Download
\n# [ File num: " 
(itoa ifile)
"\n# Time: " secs 
"\n# LM-layer mode, NT-PointCloud Nth, RS-Reduce Scale, DIV-Curve segmenting, PC-Precision # "
MTX
))


;;; ===== Writing OBJ file =====
(princ "\nWriting file")

(if (= tmp 1)
(setq FL (vl-filename-mktemp MTF (strcat (getvar "dwgprefix") ) frmt)) ;;; naming obj
(setq FL  (strcat (vl-filename-directory FL) "\\" (vl-filename-base FL) ".txt"))
)
   (setq f (open FL "w")) (write-line MTX f) (close f)
(if (= OpenF 1)
(progn
;;; ==== Opening created OBJ file in notepad on export end =====
(princ "\nOpening file  ")
   (vlax-invoke-method (setq f (vlax-get-or-create-object "wscript.shell"))
    "run" (strcat "notepad.exe " "\"" FL "\"")
    1
    :vlax-false) ;_ end of vlax-invoke-method
  (vlax-release-object f); do not search
))

(princ "\nFile output:\n") (princ FL)  
(princ)
)









(defun printout (DB subd)
;no support for ind ((-18.1077 2.64 33.0354) (-17.8448 2.64 31.8646) (-17.7862 2.64 31.8777) (-18.0491 2.64 33.0485))
	(foreach n DB
	(setq FF (reverse (mapcar '(lambda (q / f)
	(setq f (vl-princ-to-string q) f (substr f 2 (- (strlen f) 2)) )
	(setq SF (strcat SF "\nv " f )) 
	(setq iv (1+ iv))
			);;lambda
	n)
	))
	
	(setq FF (vl-princ-to-string FF) FF (substr FF 2 (- (strlen FF) 2)) )		;;"(5 6 7 8)" - vertices nums
	(setq SV (strcat SV "\nf " FF))
	);foreach
	(setq DBT (strcat DBT "\n\no " subd SF "\nusemtl " MTLX SV ))
;;	(princ "\n === ix -") (princ ix) (princ " === ixx -") (princ ixx)
	(setq ix (+ ix ixx) DBX nil ixx 0)
)

(defun CAT (x d /) (cdr (assoc x d)))	;;$$##


(defun LM:GroupByNum ( l n / r )
    (if l
        (cons
            (reverse (repeat n (setq r (cons (car l) r) l (cdr l)) r))
            (LM:GroupByNum l n)
        )
    )
)











(defun C:ABC1d (/)
(setq ii 0 st "")
(repeat 255 (setq ii (1+ ii))
(setq st (strcat st  (itoa ii) "." (chr ii) "    " ) )
)
(vla-AddMText (vla-get-modelspace (vla-get-activedocument (vlax-get-acad-object))) (vlax-3d-point (trans (getpoint "\nInsertion Point: ") 1 0)) 100.0 st)
)




(defun C:1DAB () ;;; getting text dbase reading char
(setq pt 0 DB nil NM (- (getint "\n Start pos") 1))
(while pt
(setq vs (ssget '((0 . "LWPOLYLINE"))) id -1 DD nil)
(setq pt (getpoint "\n POINT:"))
(setq NM (1+ NM))
(repeat (sslength vs)
(setq MT (ssname vs (setq id (1+ id))))
(vla-put-color (vce MT) 3 )
(setq VX (LM:LWVertices (entget MT) ) )
(if (= (1d_axprop MT 'closed) :vlax-true ) 
(setq vx (append vx (list (car vx))))
)

(setq VX (mapcar '(lambda (q / f)  
(setq f (mapcar  '(lambda (z) (fix z) ) (mapcar '- q pt)))
(list (- (car f) 23) (cadr f))
) VX) )
(setq DD (cons VX  DD))
)
(setq DB (append  DB (list (cons NM DD) )  ))
(princ "\n") (princ (cons NM DD) )
)
)












(setq 1DTXB  ;; FONT
'(
;;(32 ((7 10) (17 10)))

(33 ((4 0) (7 16)) ((9 33) (18 100)))
(34 ((40 100) (24 66)) ((14 66) (29 100)))
(35 ((36 0) (51 100)) ((30 100) (15 0)) ((57 66) (14 66)) ((9 33) (52 33)))
(36 ((40 100) (26 0)) ((7 16) (39 16) (52 33) (44 50) (22 50) (14 66) (27 83) (59 83)))
(37 ((36 0) (39 16) (50 16) (47 0) (47 0) (36 0)) ((61 100) (5 0)) ((16 83) (27 83) (30 100) (19 100) (19 100) (16 83)))
(38 ((52 33) (26 0) (15 0) (7 16) (9 33) (36 66) (38 83) (30 100) (17 83) (14 66) (47 0)))
(39 ((14 66) (30 100)))
(40 ((40 100) (14 66) (9 33) (26 0)))
(41 ((18 100) (35 66) (30 33) (5 0)))
(42 ((16 83) (50 16)) ((59 83) (7 16)) ((55 50) (12 50)) ((28 16) (38 83)))
(43 ((55 50) (12 50)) ((28 16) (38 83)))
(44 ((18 16) (15 0) (3 -16)))
(45 ((12 50) (54 50)))
(46 ((5 0) (7 16)))

(47 ((5 0) (61 100)))
(48 ((7 16) (17 83) (30 100) (40 100) (49 83) (39 16) (26 0) (15 0) (15 0) (7 16)))
(49 ((5 0) (26 0)) ((16 83) (29 100) (15 0)))
(50 ((16 82) (29 99) (51 99) (59 82) (56 66) (43 50) (22 50) (9 33) (5 0) (47 0)))
(51 ((33 50) (44 50) (52 33) (50 16) (36 0) (15 0) (7 16)) ((16 82) (30 99) (51 99) (59 82) (57 66) (44 50)))
(52 ((52 33) (9 33) (51 99) (36 0)))
(53 ((7 16) (15 0) (36 0) (50 16) (55 50) (46 66) (14 66) (19 99) (61 99)))
(54 ((12 50) (44 50) (52 33) (50 16) (36 0) (15 0) (7 16) (14 66) (40 99) (51 99)))
(55 ((19 99) (61 99) (15 0)))
(56 ((44 50) (22 50) (14 66) (17 82) (30 99) (51 99) (59 82) (57 66) (44 50) (52 33) (50 16) (37 0) (15 0) (7 16) (9 33) (22 50)))
(57 ((15 0) (26 0) (52 33) (59 82) (51 99) (30 99) (17 82) (14 66) (23 50) (55 50)))
(58 ((9 33) (7 16)) ((14 66) (12 50)))
(59 ((20 33) (15 0) (2 -16)) ((24 66) (22 50)))
(60 ((51 100) (12 50) (36 0)))
(61 ((52 33) (9 33)) ((14 66) (57 66)))
(62 ((18 100) (43 50) (4 0)))
(63 ((16 83) (29 100) (40 100) (48 83) (46 66) (33 50) (30 33)) ((28 16) (26 0)))
(64 ((44 50) (30 33) (20 33) (22 50) (35 66) (46 66) (41 33) (54 50) (59 83) (51 100) (30 100) (16 83) (7 16) (15 0) (47 0)))
(65 ((5 0) (9 33)) ((52 33) (40 100) (9 33) (52 33) (47 0)))
(66 ((22 50) (44 50) (57 66) (59 83) (51 100) (19 100)) ((5 0) (36 0) (50 16) (52 33) (44 50)) ((30 100) (15 0)))
(67 ((50 16) (36 0) (15 0) (7 16) (16 83) (30 100) (51 100) (59 83)))
(68 ((5 0) (36 0) (50 16) (59 83) (51 100) (19 100)) ((30 100) (15 0)))
(69 ((61 100) (19 100) (5 0) (47 0)) ((12 50) (33 50)))
(70 ((12 50) (33 50)) ((5 0) (18 100) (61 100)))
(71 ((44 50) (54 50) (47 0) (15 0) (7 16) (16 83) (30 100) (61 100)))
(72 ((61 100) (47 0)) ((12 50) (55 50)) ((5 0) (19 100)))
(73 ((19 100) (40 100)) ((5 0) (26 0)) ((30 100) (15 0)))
(74 ((7 16) (15 0) (36 0) (49 16) (61 100)))
(75 ((12 50) (22 50) (47 0)) ((61 100) (22 50)) ((5 0) (19 100)))



(76 ((19 100) (5 0) (47 0)))
(77 ((5 0) (19 100) (30 33) (61 100) (47 0)))
(78 ((5 0) (19 100) (47 0) (61 100)))
(79 ((19 100) (61 100) (47 0) (5 0) (5 0) (19 100)))
(80 ((5 0) (18 100) (51 100) (59 83) (56 66) (43 50) (12 50)))
(81 ((31 33) (39 16) (26 0) (15 0) (7 16) (16 83) (30 100) (51 100) (59 83) (52 33) (39 16) (47 0)))
(82 ((22 50) (47 0)) ((5 0) (19 100) (51 100) (59 83) (57 66) (44 50) (12 50)))
(83 ((7 16) (15 0) (36 0) (50 16) (16 83) (30 100) (51 100) (59 83)))
(84 ((19 100) (61 100)) ((40 100) (26 0)))
(85 ((19 100) (7 16) (15 0) (37 0) (50 16) (61 100)))
(86 ((19 100) (37 0) (83 100)))
(87 ((19 100) (26 0) (43 50) (47 0) (83 100)))
(88 ((19 100) (47 0)) ((5 0) (61 100)))
(89 ((18 100) (33 50)) ((26 0) (33 50) (61 100)))
(90 ((18 100) (61 100) (5 0) (47 0)))
(91 ((26 0) (5 0) (19 100) (40 100)))
(92 ((18 100) (47 0)))
(93 ((18 100) (40 100) (26 0) (5 0)))
(94 ((14 66) (40 100) (57 66)))
(95 ((3 -16) (45 -16)))
(96 ((19 100) (25 66)))
(97 ((39 16) (44 50) (36 66) (25 66) (12 50) (7 16) (15 0) (26 0) (26 0) (39 16) (47 0)))
(98 ((36 66) (46 66) (55 50) (50 16) (36 0) (26 0) (9 33) (9 33) (36 66)) ((5 0) (19 100)))
(99 ((57 66) (25 66) (12 50) (7 16) (15 0) (47 0)))
(100 ((61 100) (47 0)) ((26 0) (15 0) (7 16) (12 50) (24 66) (35 66) (52 33) (52 33) (26 0)))
(101 ((9 33) (41 33) (54 50) (46 66) (24 66) (12 50) (7 16) (15 0) (36 0)))
(102 ((59 83) (51 100) (40 100) (27 83) (15 0)) ((12 50) (43 50)))
(103 ((3 -16) (11 -32) (32 -32) (45 -16) (54 50) (46 66) (25 66) (12 50) (7 16) (15 0) (47 0)))
(104 ((5 0) (19 100)) ((9 33) (36 66) (46 66) (55 50) (47 0)))
(105 ((16 83) (18 100)) ((5 0) (14 66)))
(106 ((48 83) (51 100)) ((2 -16) (11 -32) (21 -32) (34 -16) (46 66)))
(107 ((9 33) (30 33)) ((56 66) (30 33) (47 0)) ((4 0) (18 100)))
(108 ((18 100) (7 16) (15 0)))
(109 ((30 33) (33 50) (46 66) (54 50) (47 0)) ((12 50) (24 66) (33 50)) ((5 0) (14 66)))
(110 ((9 33) (35 66) (46 66) (54 49) (47 0)) ((5 0) (14 66)))
(111 ((15 0) (7 16) (12 49) (25 66) (46 66) (54 49) (50 16) (36 0) (36 0) (15 0)))
(112 ((0 -33) (14 66)) ((12 49) (24 66) (46 66) (54 49) (49 16) (36 0) (5 0)))
(113 ((54 49) (46 66) (25 66) (12 49) (7 16) (15 0) (47 0)) ((43 -33) (57 66)))
(114 ((5 0) (14 66)) ((9 33) (35 66) (46 66) (54 49)))
(115 ((5 0) (36 0) (50 16) (41 33) (20 33) (12 49) (25 66) (57 66)))
(116 ((14 66) (56 66)) ((40 99) (28 16) (36 0) (49 16)))
(117 ((57 66) (47 0)) ((14 66) (7 16) (15 0) (26 0) (52 33)))
(118 ((14 66) (26 0) (57 66))) (119 ((14 66) (15 0) (36 66) (37 0) (57 66)))


(119 ((14 66) (15 0) (36 66) (37 0) (57 66)))
(120 ((14 66) (47 0)) ((5 0) (56 66)))
(121 ((57 66) (11 -33) (0 -33)) ((14 66) (26 0)))
(122 ((14 66) (57 66) (5 0) (47 0)))

(124 ((5 0) (18 100)))
(126 ((12 50) (25 66) (44 50) (57 66)))
(136 ((13 58) (45 58)) ((11 41) (37 41)) ((50 16) (49 8) (42 0) (21 0) (12 16) (17 50) (22 83) (35 99) (56 99) (60 91) (59 83)))


(168 ((61 100) (19 100) (5 0) (47 0)) ((12 50) (33 50)) )
(184 ((9 33) (41 33) (54 50) (46 66) (25 66) (12 50) (7 16) (15 0) (36 0) ))

(192 ((52 33) (40 100) (9 33) (52 33) (47 0)) ((5 0) (9 33)))
(193 ((48 83) (51 100) (18 100) (12 50) (43 50) (52 33) (49 16) (36 0) (5 0) (18 100) (51 100)))
(194 ((22 50) (43 50) (57 66) (59 83) (51 100) (18 100)) ((5 0) (36 0) (49 16) (52 33) (43 50)) ((29 100) (15 0)))
(195 ((5 0) (18 100) (61 100) (59 83)))
(196 ((2 -16) (5 0) (58 0) (72 100) (51 100) (20 33) (15 0) (68 0) (66 -16)))
(197 ((61 100) (19 100) (5 0) (47 0)) ((12 50) (33 50)))
(198 ((19 100) (69 0)) ((51 100) (36 0)) ((5 0) (83 100)))
(199 ((33 50) (43 50) (57 66) (59 83) (51 100) (29 100) (16 83)) ((7 16) (15 0) (36 0) (49 16) (52 33) (43 50)))
(200 ((19 100) (5 0) (57 66) (47 0) (61 100)))
(201 ((50 100) (29 100)) ((18 100) (5 0) (56 66) (47 0) (61 100)))
(202 ((12 50) (22 50) (47 0)) ((61 100) (22 50)) ((5 0) (19 100)))
(203 ((5 0) (15 0) (61 100) (72 100) (58 0)))

(204 ((5 0) (18 100) (30 33) (61 100) (47 0)))
(205 ((61 100) (47 0)) ((12 50) (55 50)) ((5 0) (19 100)))
(206 ((16 83) (29 100) (51 100) (59 83) (49 16) (36 0) (15 0) (7 16) (7 16) (16 83)))
(207 ((5 0) (19 100) (61 100) (47 0)))
(208 ((5 0) (19 100) (51 100) (59 83) (57 66) (44 50) (12 50)))
(209 ((49 16) (36 0) (15 0) (7 16) (16 83) (29 100) (51 100) (59 83)))

(210 ((18 99) (61 99)) ((40 99) (26 0)))
(211 ((7 16) (15 0) (36 0) (49 16) (61 99) (52 32) (20 32) (12 49) (19 99)))
(212 ((29 99) (16 82) (12 49) (20 32) (41 32) (54 49) (59 82) (51 99) (40 99) (29 99)) ((26 0) (40 99)))
(213 ((18 99) (47 0)) ((5 0) (61 99)))
(214 ((61 99) (47 0) (58 0) (55 -16)) ((18 99) (5 0) (47 0)))
(215 ((19 99) (12 49) (20 32) (52 32) (61 99) (47 0)))
(216 ((46 66) (36 0) (69 0) (83 99)) ((19 99) (5 0) (36 0)))
(217 ((83 99) (68 0) (79 0) (76 -16)) ((46 66) (36 0) (68 0)) ((18 99) (5 0) (36 0)))
(218 ((16 82) (19 99) (29 99) (15 0) (47 0) (60 16) (63 32) (54 49) (22 49)))
(219 ((72 99) (58 0)) ((5 0) (36 0) (49 16) (52 32) (44 49) (12 49) (5 0) (19 99)))
(220 ((5 0) (36 0) (49 16) (52 33) (44 50) (12 50) (5 0) (19 100)))
(221 ((33 50) (54 50) (59 83) (51 100) (29 100) (16 83)) ((7 16) (15 0) (36 0) (49 16) (54 50)))
(222 ((27 83) (40 100) (50 100) (59 83) (49 16) (36 0) (26 0) (18 16) (22 50) (27 83)) ((4 0) (18 100) (12 50) (22 50)))
(223 ((5 0) (31 33) (52 33) (61 100) (30 100) (17 83) (12 50) (20 33) (52 33) (47 0)))
(224 ((12 50) (24 66) (46 66) (54 50) (57 66) (47 0) (49 16) (36 0) (15 0) (7 16) (7 16) (12 50)))
(225 ((46 66) (14 66) (5 0) (36 0) (49 16) (41 33) (9 33)))
(226 ((36 0) (5 0) (14 66) (36 66)) ((35 66) (38 66)  (43 60)  (44 52)  (42 43)  (37 36)  (31 33) (9 33) (41 33) (44 32) (47 30) (49 27) (49 14) (46 5)  (40 0) (37 0)))
(227 ((5 0) (14 66) (46 66)))
(228 ((2 -16) (4 0) (15 0) (22 50) (35 66) (46 66) (36 0) (15 0) (47 0) (45 -16)))
(229 ((9 33) (41 33) (54 50) (46 66) (25 66) (12 50) (7 16) (15 0) (36 0)))
(230 ((26 0) (35 66)) ((14 66) (47 0)) ((5 0) (57 66)))
(231 ((30 33) (41 33) (49 16) (36 0) (15 0) (7 16)) ((12 50) (24 66) (46 66) (54 50) (41 33)))
(232 ((14 66) (5 0) (57 66) (47 0)))
(233 ((46 66) (25 66)) ((14 66) (5 0) (57 66) (47 0)))
(234 ((56 66) (30 33) (47 0)) ((5 0) (14 66) (9 33) (30 33)))
(235 ((5 0) (35 66) (57 66) (47 0)))
(236 ((5 0) (14 66) (30 33) (56 66) (47 0)))
(237 ((5 0) (14 66) (9 33) (52 33) (57 66) (47 0)))
(238 ((12 50) (24 66) (46 66) (54 50) (49 16) (36 0) (15 0) (7 16) (7 16) (12 50)))
(239 ((4 0) (14 66) (56 66) (47 0)))
(240 ((5 0) (14 66) (46 66) (54 49) (52 33) (39 16) (7 16)))
(241 ((49 16) (36 0) (15 0) (7 16) (12 49) (24 66) (46 66) (54 49)))
(242 ((26 0) (36 66) (14 66) (57 66)))
(243 ((7 16) (15 0) (36 0) (49 16) (56 66) (52 33) (30 33) (14 66)))
(244 ((26 0) (35 66)) ((24 66) (12 49) (9 33) (18 16) (39 16) (52 33) (54 49) (46 66) (35 66) (24 66)))
(245 ((14 66) (47 0)) ((5 0) (57 66)))
(246 ((46 66) (36 0) (47 0) (45 -16)) ((14 66) (5 0) (36 0)))
(247 ((14 66) (7 16) (39 16) (46 66) (36 0)))
(248 ((30 33) (26 0) (47 0) (57 66)) ((14 66) (5 0) (26 0)))

(249 ((57 66) (47 0) (58 0) (56 -16)) ((31 33) (26 0) (47 0)) ((14 66) (5 0) (26 0)))
(250 ((14 66) (25 66) (15 0) (36 0) (49 16) (41 33) (20 33)))
(251 ((47 0) (57 66)) ((14 66) (9 33) (30 33) (39 16) (26 0) (5 0) (5 0) (14 66)))
(252 ((14 66) (9 33) (31 33) (39 16) (26 0) (5 0) (5 0) (14 66)))
(253 ((31 33) (52 33) (54 50) (46 66) (25 66) (12 50)) ((7 16) (15 0) (36 0) (50 16) (52 33)))
(254 ((22 50) (35 66) (46 66) (54 50) (49 16) (36 0) (26 0) (18 16) (20 33) (22 50)) ((5 0) (14 66) (9 33) (20 33)))
(255 ((52 33) (57 66) (25 66) (12 50) (20 33) (31 33) (52 33) (47 0)) ((5 0) (31 33)))
)
)








(defun 1DTEXTVERTS (MT / TS MB dpos TXSC ptx)		;;; textworks not used anymore

(setq TS (1d_axprop MT 'Textstring) ptx (1d_axprop MT 'Insertionpoint) TXSC (/ (1d_axprop MT 'Height) 100.0) dpos 0)
(setq MB (list  '(0.0 0.0 0.0) (list TXSC TXSC TXSC) (1d_axprop MT 'Rotation) ))

(setq TX
(mapcar '(lambda (q)
(setq dpos (+ dpos (* 60.0 1))) ;TXSC
(if (eq q 32) (setq dpos (+ dpos (* 60.0 1))) )
(if (eq q 9) (setq dpos (+ dpos (* 60.0 4))) )
(if (> q 130) (setq q (- q 848)))

(if (setq q (cat q 1DTXB))
(mapcar '(lambda (z)
(mapcar '(lambda (x)
(mapcar '+ ptx 
(1dtrans (mapcar '+ (list dpos 0.0 0.0)  (list (nth 0 x) (nth 1 x) 0.0 ) ) MB)
)) z)
) q )
)



)

(vl-string->list TS)
)
)

(setq TX (apply 'append TX) )
;;(foreach n TX (entmake-pline n))
);; defun






(defun 1D_LM:StringWrap ( str pat / pos )
    (if (vl-string-search pat str )
        (cons
            (substr str 1
		(setq pos (vl-string-search pat str ))
            )
            (1D_LM:StringWrap (substr str (+ 1 pos (strlen pat))) pat)
        )
        (list str)
    )
)





(defun C:DTT (/)
(setq MTE (car (entsel)))
(setq MT (entget MTE))



(setq MT (mapcar '(lambda (q) (cons q (cat q MT) )) '(10  14  13 15 12 16 11  )))
(setq mt (vl-remove-if '(lambda (q) (equal (apply '+ (cdr q)) 0 1 ) ) MT))
(princ "\n== ") (princ MT)(princ "===\n ")
(entmake-pline (mapcar 'cdr MT))


(mapcar '(lambda (q)
(vla-addtext (vla-get-modelspace (vla-get-activedocument (vlax-get-acad-object))) 
(itoa (car q)) (vlax-3d-point (trans (cdr q) 1 0)) 200.5))
MT)



(princ)


)


(defun 1dunf (a )
(if (and a (vl-string-search "\\f" a))
(strcat (substr a 1 (vl-string-search "\\f" a)  )   (1dunf (substr a (+ 2 (vl-string-search ";" a)) ) ) )
""
)
)


(defun 1DV_EXTRACT ( MTE DIV TEXX DIMTEX / ivb IND ivv MX MBX px fls MTX MXX dpos ypos ) ;;; INDexworks
;;(setq MTE (car (entsel)))    (setq MX (1DV_EXTRACT (setq MTE (car (entsel))) 5 TEXX DIMTEX))
(setq MX nil MT (entget MTE) lw (vlax-ename->vla-object MTE) )
(setq IND (cat (1d_axprop lw 'ObjectName)
 '(("AcDbBlockReference" . 0) ("AcDbFace" . 1) ("AcDbPolyline" . 2) ("AcDb3dPolyline" . 3)
("AcDbLine" . 4) ("AcDbArc" . 5) ("AcDbCircle" . 6) ("AcDbMText" . 7) ("AcDbText" . 7) ("AcDb2dPolyline" . 3) ("AcDbSpline" . 8) 
("AcDbRotatedDimension" . 9) ("AcDbAlignedDimension" . 9) 
;("AcDb2LineAngularDimension" . 9)
("AcDbSubDMesh" . 11) ("AcDbEllipse" . 20) ("AcDbPoint" . 24) ;;; ELLIPSE works ELLS
)
))  ;;; INDexworks

(cond ;;; INLAY WORKOUT VERTICES EXTRACTION 1 = readind INDexes



( (eq IND 11)	;;; SUBDMESH works
(setq fls 1) ;;; flag to unlist output
(setq MTVR nil ixm 0  MTT "" MTV (1DLM:LWVertices MT 10) )
;;(foreach nn MTV (setq MTVR (cons (mapcar '(lambda (q) (/ q RS)) nn) MTVR) ) )  ;; output scaling
;;(setq MTV (reverse MTVR))
(setq MTED nil ir -1 z nil)				;; ([3] 0 1 2 [3] 3 4 5 [3] 6 7 8)
(while (not (eq (car z) 94)) (setq ir (1+ ir) z (nth ir MT)) (if (= (car z) 90) (setq MTED (cons (cdr z) MTED) )) )
(setq MTED (reverse MTED))

(setq MT (LM:REGroupByNum MTED) )			;; ((1 2 3) (4 5 6) (7 8 9)) +1 incremented
(setq MT (mapcar '(lambda (q) (mapcar '(lambda (z) (+ ixm z)) q) ) MT))	;; global index increment apply
(setq ixm (+ ixm (length MTV)))

(setq MX (cons MTV MT) )
)


( (and TEXX (eq IND 9))	;;; DIM works
(setq fls 1) ;;; flag to unlist output
;;; =======  processing dim geometry

(setq MTX (1d_axprop lw 'Measurement) ptx (1d_axprop lw 'Textposition) TXSC (/ (1d_axprop lw 'TextHeight) DIMTEX) dpos 0 ypos 0) ;;; dimscale
(setq MBX (list  '(0.0 0.0 0.0) (list TXSC TXSC TXSC) (1d_axprop lw 'Rotation) ))

(if (eq "AcDb2LineAngularDimension" (1d_axprop lw 'ObjectName) )
(setq MTX (RTD MTX) ivv (list 14 13 15 13 11)) ;; to degrees
(setq
ivb 
(car 
(vl-sort
(list (1dperZ (cat 10 MT) MTX (+ (* pi 1.5) (angle (cat 10 MT) (cat 14 MT) ))) (1dperZ (cat 10 MT) MTX (+ (/ pi 2) (angle (cat 10 MT) (cat 14 MT) ))) )
 '(lambda (a b) (<  (distance a (cat 13 MT) ) (distance b (cat 13 MT) ) ) )
)
);car MB

MT (cons (cons -11 ivb) MT )
ivv (list 15 14  10 -11 13 -11 11))
)

;;; =======  processing dim text
(setq MTX (rtos (+ MTX 0.0) 2 2))
(setq MXX
(mapcar '(lambda (q)
(setq dpos (+ dpos (* 60.0 1))) ;TXSC next char
(if (> q 130) (setq q (- q 848)))

(if (setq q (cat q 1DTXB))
(mapcar '(lambda (z)
(mapcar '(lambda (x)
(mapcar '+ ptx 
(1dtrans (mapcar '+ (list dpos ypos 0.0)  (list (nth 0 x) (nth 1 x) 0.0 ) ) MBX)   ;;;local 1DTRANS
)) z)
) q )
) )
(vl-string->list MTX)
) )
(setq MXX (apply 'append MXX) )

(setq MT (mapcar '(lambda (q) (cons q (cat q MT) )) ivv ) )
(setq MT (vl-remove-if '(lambda (q) (equal (apply '+ (cdr q)) 0 1 ) ) MT))
(setq MX (append (list (mapcar 'cdr MT)) MXX))
)


( (eq IND 8)  ;;; SPLINE works
(setq px (vcget "85" lw nil) ivv  (/ px  (* DIV (1d_axprop lw 'NumberOfcontrolPoints))) ) ;;; dividing distance to controlpoints_x_division
(setq MX (list (vcget "73" lw nil) ) ivb 0 px (- px ivv)) ;;; first point
(while (<= ivb px) (setq MX (cons (vcget "4" lw (setq ivb (+ ivb ivv)) ) MX) ) ) ;;; filling pointlist
(setq MX (cons (vcget "83" lw nil) MX) ) ;;; last point
)


( (and TEXX (eq IND 7)) ;;TEXT  textworks
(setq fls 1) ;;; flag to unlist output

(setq MTX (1d_axprop lw 'Textstring) ptx (1d_axprop lw 'Insertionpoint) TXSC (/ (1d_axprop lw 'Height) 100.0) dpos 0 ypos 0 MX nil) ;;;1dunf
(setq MBX (list  '(0.0 0.0 0.0) (list TXSC TXSC TXSC) (1d_axprop lw 'Rotation) ))
(setq MTX (1D_LM:StringWrap MTX "\\P") )
(foreach n MTX
(setq MXX
(mapcar '(lambda (q)
(setq dpos (+ dpos (* 60.0 1))) ;TXSC next char
(if (eq q 32) (setq dpos (+ dpos (* 60.0 1))) )
(if (eq q 9) (setq dpos (+ dpos (* 60.0 4))) )
(if (> q 130) (setq q (- q 848)))

(if (setq q (cat q 1DTXB))
(mapcar '(lambda (z)
(mapcar '(lambda (x)
(mapcar '+ ptx 
(1dtrans (mapcar '+ (list dpos ypos 0.0)  (list (nth 0 x) (nth 1 x) 0.0 ) ) MBX)   ;;;local 1DTRANS
)) z)
) q )
) )
(vl-string->list n)
) )
(setq MXX (apply 'append MXX) )
(setq ypos (- ypos (* 150.0 1)) dpos 0) ;TXSC next string
(setq MX (append MXX MX))
)
)

( (eq IND 6) ;;3D CIRCLE
(setq ivb (/ (* 1 pi) (* 2 DIV)) ivv -1 MX nil)
(repeat (+ (* 4 DIV) 0) (setq MX (cons (vlax-curve-getPointAtParam lw  (* ivb (setq ivv (1+ ivv))) )  MX)) )
)

( (eq IND 5) ;;3D ARC
(setq MX (list (1d_axprop MTE 'StartPoint) (1d_axprop MTE 'EndPoint)) ) ;; Elevation
(setq FITZERO (+ 0 (fix (* (1d_axprop lw 'Totalangle) DIV))))
(if (eq FITZERO 0) (setq FITZERO 1) )
(setq ivv 0 px nil ivb (/ (1d_axprop lw 'ArcLength) FITZERO ) ) ;;; segments division ARCFIX 4.08  0 = 2
(setq MX (mapcar '(lambda (q) (vcget "4" lw (* ivb (setq ivv (1+ ivv)))) ) (repeat (- (fix (* (1d_axprop lw 'Totalangle) DIV)) 1) (setq px (cons nil px)) ))) ;; arcpoints dist
(setq MX (append (list (1d_axprop MTE 'StartPoint)) MX  (list (1d_axprop MTE 'EndPoint)) ) )
)

( (eq IND 4) ;;3D LINE
(setq MX (list  (1d_axprop MTE 'StartPoint)  (1d_axprop MTE 'EndPoint) ) )
)

( (eq IND 24) ;;3D POINT section
(setq MX (1d_axprop MTE 'Coordinates))

(if $$1DPOINTBASE
(setq MX (list MX (list (-(car $$1DPOINTBASE)) (-(cdr $$1DPOINTBASE)) 0.0 ) ) )
(setq MX (list MX (list 0.0 0.0 0.0 ) ) )	;; xy = (nth 0 MX) (nth 1 MX) z= (- (nth 2 MX) 0.1)
))

( (eq IND 3) ;;2D/3D POLY
(setq MX (LM:GroupByNum (1d_axprop MTE 'Coordinates) 3) )
)

( (eq IND 2) ;;LWPOLYLINE 3D scan
;;(setq MX (LM:LWVertices MT) lvl (- (cat 38 MT) 160)) ;; Elevation
;;(setq MX (mapcar '(lambda (q) (1dtrans  (list (car q) (cadr q) lvl) MB)) MX))

(progn
(setq lw (vce MTE))
(setq MX (1DLM:LWVertices MT 42) )
(setq ivv -1 ) ;;; segments division
(setq MX
(vl-remove-if 'null (apply 'append (mapcar '(lambda (q / px)
(setq ivv (1+ ivv) px (vcget "3" lw ivv)  )
(if (= q 0.0)
	(list px)	;;; normal point
	(progn		;;; split bulgency
		(if (< q 0.2) (setq q 0.9))	;; 20:24 19.03.2018 resolve flat arc
		(if (> q 1.0) (setq q (* 2 q)) )
		(setq px nil ivb (- ivv (/ 1.0 (* (abs q) DIV ))) )
		(repeat (1+ (fix (* (abs q) DIV))) (setq px (cons nil px)) )
	 	(mapcar '(lambda (o) (vcget "3" lw (setq ivb (+ ivb (/ 1.0 (* (abs q) DIV)))) ) )  px)
	)
)
) ;;; lambda
MX ;;; list of bulgencies to detect some
) ;;;  mapcar
))
)
;;(if (= (1d_axprop lw 'closed) :vlax-true)  (setq MX (append MX (list (car MX)) )) MX ) ;;; that is I call recycling (c)
)
)

((eq IND 1) ;;3DFACE
(setq MX (list (cat 10 MT) (cat 11 MT) (cat 12 MT) (cat 13 MT) ) )
)


( (eq IND 20)  ;;; ELLIPSE works
(setq px (vcget "85" lw nil) ivv  (/ px  (* DIV 1)) ) ;;; dividing distance to controlpoints_x_division
(setq MX (list (vcget "73" lw nil) ) ivb 0 px (- px ivv)) ;;; first point
(while (<= ivb px) (setq MX (cons (vcget "4" lw (setq ivb (+ ivb ivv)) ) MX) ) ) ;;; filling pointlist
(setq MX (cons (vcget "83" lw nil) MX) ) ;;; last point
)


( t nil)
);; cond

(if fls MX (list MX) )

);; defun 1DV_EXTRACT











(defun C:1DFEX (/ ;;; FAST FACES EXPORT 16:46 10.07.2017 
DBT
AA DBM DBT DBX DIVS F FF FL ID IFILE IV
IX	;global counter
IXX	;local counter
;MTE	; proceed object
ML MT  MTF MTFX MTL MTLX OBN Q RS SXF SYF SECS SF SV X GM
DIV DIVS F FLS IND LAYW LMODE LW MB MX PREC Q RF RS SPT VS VSS X
DVX F MXE Q RS X
)


(princ "\n1DFEX___  v5.17 14.07.2022\n")
(if (not (eq (getvar "INSUNITS" ) 4) )   (alert "INSUNITS != 4, changed"))
(setvar "INSUNITS" 4)



;[
(if (= 0 (getvar "GRIDMODE")) (setq GM 0) (setq GM nil))
(princ "\n[GRID compartible function (F7)]")
(if GM (princ "\n>>>") (princ "\n   "))
(princ "[OFF]  = PLINES AS WIRES")
(if GM (princ "\n   ") (princ "\n>>>"))
(princ "[ON]   = PLINES AS NGONS")
;]




(defun 1D_INP (VAL LST /) ;; (2000 '((1000.0 . 5000.0) (10.0 . 50.0)) )
(+ (* (/ (- (cdadr LST) (caadr LST) ) (- (cdar LST) (caar LST) ) ) (- VAL (caar LST) ) ) (caadr LST) )
)


;;; ============================
;;; ============================ MAIN FUNC
;;; ============================


(setq vs (ssget '((0 . "3DFACE,LWPOLYLINE,LINE,CIRCLE,POLYLINE,ARC,TEXT,MTEXT,SPLINE,DIMENSION,MESH,ELLIPSE,POINT"))) ) ;;; INDexworks
(setq id -1 AA nil RS 1000.0 SXF 0.0 SYF 0.0  DBT "" OBN "-" iv 0 ifile 0 ix 0)


(setq OBN (getstring "\nOBN enter obj name prefix (space for skip):"))
(if $$1DB  (progn (princ "\n ___________\n") (princ $$1DB) (setq FLS (getstring "\n Load Previous Settings? <space = yes, anykey = no>")) ))

(if (= FLS "")

(mapcar 'set (list 'spt 'lmode 'RF 'RS 'DIV 'PREC 'layw 'SXF 'SYF 'SYZ 'TEXX) $$1DB)
(progn

;;(setq DIV (getint	"\nDIV (Curve segmenting) <8>:"))
(setq TEXX (getint "\nTEXX (export texts <space = no, anykey = yes>) <no>:"))
(setq DIMTEX (getreal "\nDIMTEX reduce text value <10>:"))
(if (not DIMTEX) (setq DIMTEX 10))
(setq SYZ (getint "\nCoordinates rotation (Space = Flat [YZ], 1 =  Default [-ZY])")) ;;YZ axis

;;points [
(if $$1DPOINTBASE (setq SXF (car $$1DPOINTBASE) SYF (cdr $$1DPOINTBASE)  )  )
(if (not SXF) (setq SXF 0.0))
(if (not SYF) (setq SYF 0.0))

(setq SXX (getreal (strcat "\n X Shift <" (rtos SXF 2 8) ">")))
(setq SYY (getreal (strcat "\n Y Shift <" (rtos SYF 2 8) ">")))

(cond 
((and (not SXX) $$1DPOINTBASE) (setq SXF (car $$1DPOINTBASE) ) )
(SXX (setq SXF SXX))
( t (setq SXF 0.0 ) )
)

(cond 
((and (not SYY) $$1DPOINTBASE) (setq SYF (cdr $$1DPOINTBASE) ) )
(SYY (setq SYF SYY))
( t (setq SYF 0.0 ) )
)

(setq $$1DPOINTBASE (cons SXF SYF))
;;]

(setq RS (getreal	"\nRS (Reduce scale for scene) <1000>:"))
(setq DIV (getint	"\nDIV (Curve segmenting) <8>:"))

)
)


;;; ========= Checking Input data convert conditions ===========
(if (not OBN) (setq OBN ""))
(if (not (eq (type RS) 'REAL)) (setq RS 1000.0))
(if (or (not (eq (type DIV) 'INT)) (< DIV 1) ) (setq DIV 8))
(if (not (eq (type PREC) 'INT)) (setq PREC 8))

(setvar "DIMZIN" 8)
(setq $$1DB (list spt lmode RF RS DIV PREC layw SXF SYF SYZ TEXX) )
(setq DIVS (list (/ SXF RS) (/ SYF RS) 0.0) )

;(setq OBN (strcat "-" (itoa ifile)))
(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") )  MTFX (strcat (getvar "DWGPREFIX") (getvar "DWGNAME")))
(STIMER)


(getstring "\nStart Export?")

;;; while MAIN section
(while vs

(setq MTL (cat 8 (entget (ssname vs 0))) AA (cons MTL AA) )
(setq MTLX  (1DLatin MTL 1) ) ;; interpolatin to Latin*

;; (setq i 0 cch nil) (repeat 255 (setq cch (cons (cons (setq i (1+ i))  (chr i)) cch))) ;; characters alphabet
;; (foreach n cch (princ "\n") (princ n))

(setq vss (ssadd))
(setq id -1 DBX nil ixx 0)

(repeat (sslength vs)
(setq SV "" SF "" DBM nil)
(setq ML (cat 8 (setq MT (entget (setq MTE (ssname vs (setq id (1+ id)))) )) ))
;;; applying shift and scale
(if (eq MTL ML)

(progn
;;	(setq IND (car MB))
;;	(setq MB (last MB)) ;;; catched transforms
;;	(setq lw (vce MTE)) ;;; vla-object


(setq lw (vlax-ename->vla-object MTE) MX nil)

(setq IND (cat (1d_axprop lw 'ObjectName)	;;; INDexworks 1DFEX
'(("AcDbBlockReference" . 0) ("AcDbFace" . 1) ("AcDbPolyline" . 2) ("AcDb3dPolyline" . 3)
("AcDbLine" . 4) ("AcDbArc" . 5) ("AcDbCircle" . 6) ("AcDbMText" . 7) ("AcDbText" . 7) ("AcDb2dPolyline" . 3) ("AcDbSpline" . 8) 
("AcDbRotatedDimension" . 9) ("AcDbAlignedDimension" . 9) 
;("AcDb2LineAngularDimension" . 9)
("AcDbSubDMesh" . 11) ("AcDbEllipse" . 20) ("AcDbPoint" . 24) ;;; ELLIPSE works
)

 ) ) ;;; INDexworks

;;; ==============================
;;; GETTING VERTICES FROM GEOMETRY
;;; ==============================


(setq MB (list '(0.0 0.0 0.0) '(1.0 1.0 1.0) 0.0 ))

;;;======================= extracting text

(setq MX (1DV_EXTRACT MTE DIV TEXX DIMTEX)) ;;; including texts



;|
	;;(setq MX (mapcar '(lambda (q) (1dtrans q MB)) MX))	;;; applying inblock modification
(if (>= (apply '* (nth 1 MB) ) 0) 
(setq MX (list MX) ) ;;; original normals order
(setq MX (list (reverse MX)) ) ;;; fixed mirrored normals order
)
|;


(if (and (vlax-property-available-p (vce MTE) 'closed) (eq (1d_axprop MTE 'closed) :vlax-true) ) (setq IND 6) ) ;; now that's I calling recycling (c)

(setq DVX nil)
(if (= IND 11) (setq MXE (cdr MX) MX (list (car MX))) ) ;; meshworks FEX - trailing faces INDexes
(foreach n MX

;;; ========================================================
;;; ----------------- Applyng Scale, rotate, flip, translate
;;; ========================================================

	(setq DVX (cons
		(list IND (mapcar  '(lambda (q / f) (setq f (apply 'mapcar (cons '+ (list (mapcar '(lambda (x) (/ x RS) ) q) DIVS ) )) ) ;;; added DIVS movement in block
(if SYZ	 (list (nth 0 f) (nth 2 f) (- (nth 1 f)) )		;;; flip YZ axis
f)

				)
		n)) 	;;; gathered verts
	 DVX)
	ixx (1+ ixx)
	)
)
(if (= IND 11) (setq DVX (list (append (car DVX) (list MXE) ))) ) ;; meshworks FEX - retrailing faces INDces to last element
(setq DBX (append DVX DBX))


)	;; progn (eq MTL ML)



(setq vss (ssadd MTE vss) )

) ;; if (eq MTL ML)
;;;(setq qqq DBX)  ;;; meshworks
(if (> ixx 222) (prinCout DBX MTLX))

(if (> ix 222222) ;;; 8888 222222    4444 = 3-8 Mb
	(progn 
	(prinCout DBX MTLX)
	;;(setq OBN (strcat "-" (itoa (setq ifile (1+ ifile) ) ) ))
	(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") ))
	(1D:Write_FILE DBT 0 ".obj" 1)
	(setq DBX nil DBT "" SF "" SV "" iv 0 ix 0)
	(gc)
	)
)


);; repeat

(prinCout DBX MTLX)

(princ "\n______  ") (princ MTL) (princ " ___ ") (princ MTLX)
(if (> (sslength vss) 0) (setq vs vss) (setq vs nil))

) ;;while

;;(setq OBN (strcat "-" (itoa (setq ifile (1+ ifile) ))))
(princ "\n Total files:  ")
(princ ifile)
(princ "\n Time : " )
(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") ))
(1D:Write_FILE DBT 1 ".obj" 1)

);; defun 1D_FEX



;;(princ "\n 1dfex \n")












;; INLAY WORKOUT VERTICES EXTRACTION 2 
(defun prinCout (DB subd / FF f inx FV)
;support for IND (1 ((-18.1077 2.64 33.0354) (-17.8448 2.64 31.8646) (-17.7862 2.64 31.8777) (-18.0491 2.64 33.0485)))

	(foreach n DB ;;; INDexworks

(setq inx (car n)) ;; switch LWPOLYLINE MODE
(if (not (member inx '(7 9 11)))
(if (not GM) (setq inx 1))
)

(cond
((= inx 1) ;;;; pointlist proceeds as ngons FACES MODE

	(setq FF (reverse (mapcar '(lambda (q / f)
	(setq SF (strcat SF "\nv " (apply 'strcat (mapcar '(lambda (z)  (strcat (rtos z 2 8) " ") ) q)) )) 	;;; - forming verts list
	(setq iv (1+ iv))											;;; - forming output numlist from free numbers!!
			);;lambda
	(cadr n) )
	))
	
	(setq FF (vl-princ-to-string FF) FF (substr FF 2 (- (strlen FF) 2)) )		;;"(5 6 7 8)" - vertices nums
	(setq SV (strcat SV "\nf " FF))

)

((member inx '(2 3 4 5 6 7 8 9 20 24)) ;;;; pointlist proceeds as outlines PLINES MODE INDexes read textworks ;;; ELLIPSE works

	(setq FF (reverse (mapcar '(lambda (q / f)
	(setq SF (strcat SF "\nv " (apply 'strcat (mapcar '(lambda (z)  (strcat (rtos z 2 8) " ") ) q)) )) 	;;; - forming verts list
	(setq iv (1+ iv))											;;; - forming output numlist from free numbers!!
			);;lambda
	(cadr n)) 
	))
	
	(setq FF (mapcar '(lambda (a b) (if b (list a b)) ) FF (cdr FF)) )
;;	(setq FF (vl-princ-to-string FF) FF (substr FF 2 (- (strlen FF) 2)) )		;;"(5 6) (7 8)" - vertices nums

	(foreach n FF (setq SV (strcat SV "\nf " (itoa (car n)) " " (itoa (cadr n)) )))

(if (member inx '(6)) (setq SV (strcat SV "\nf " (itoa (caar FF)) " " (itoa (cadr (last FF))) )) )		;; circle recycling

)


((member inx '(11) )  ;; meshworks prinCout = complex vertices patterns  (11 ((1.1 2.2 3.3) (1.1 2.2 3.3)) ((1 2 3) (4 3) (2 5 4 6)) ) = (11 (verts) (faces))
;; ============== FORM OVERALL TEXT =========
(setq FF (length (mapcar '(lambda (q / f) (setq SF (strcat SF "\nv " (apply 'strcat (mapcar '(lambda (z)  (strcat (rtos z 2 8) " ") ) q) ) ) ) ) (nth 1 n) )) )		;; verts
(setq SV (strcat SV (apply 'strcat (mapcar '(lambda (q)  (strcat "\nf " (apply 'strcat (mapcar '(lambda (z) (strcat (itoa (+ z iv)) " " )) q)) )) (nth 2 n) ))))	;; faces
(setq iv (+ FF iv))	;; global int
) ;;11


(t nil)
)
	);foreach


	(setq DBT (strcat DBT "\n\no " subd SF "\nusemtl " MTLX SV ))
;;	(princ "\n === ix -") (princ ix) (princ " === ixx -") (princ ixx)
	(setq ix (+ ix ixx) DBX nil ixx 0)
) ;;defun prinCout






(defun 1D_INP (VAL LST /) ;; (2000 '((1000.0 . 5000.0) (10.0 . 50.0)) )
(+ (* (/ (- (cdadr LST) (caadr LST) ) (- (cdar LST) (caar LST) ) ) (- VAL (caar LST) ) ) (caadr LST) )
)


(defun vcget (id lw px)	;; (vcget "02" lwt pt1)		;;; curve-point parameters converter
(if (= (type id) 'STR) (setq id (mapcar '(lambda (q) (read (chr q))) (vl-string->list id)) ) )
(if (car id) (progn (cond
((= 0 (car id)) (setq px (vlax-curve-getClosestPointto lw px)))
((= 1 (car id)) (setq px (vlax-curve-getparamatpoint lw px)))
((= 2 (car id)) (setq px (vlax-curve-getdistatpoint lw px)))
((= 3 (car id)) (setq px (vlax-curve-getpointatparam lw px)))
((= 4 (car id)) (setq px (vlax-curve-getpointatdist lw px)))

((= 5 (car id)) (setq px (vlax-curve-getdistatparam lw px)))
((= 6 (car id)) (setq px (vlax-curve-getparamatdist lw px)))

((= 7 (car id)) (setq px (vlax-curve-getStartParam lw)))
((= 8 (car id)) (setq px (vlax-curve-getEndParam lw)))

(t nil)
) ))	;ifprogn
(if (cdr id) (vcget (cdr id) lw  px) px)
)



(defun 1dtrans (pt lstx / ang dis pts) ;; (list '(500.0 600.0 20.0)  '((1000.0 2000.0 0.0) (1.0 1.0 1.0) 0.5) ) ;; (loc scale rot)
(setq pt0 '(0.0 0.0 0.0))
(setq pts (apply 'mapcar (cons '* (list pt  (nth 1 lstx)) ) ))					;;; applying scale
(setq ang (+  (angle pt0 pts) (last lstx)) dis (distance (list (nth 0 pts) (nth 1 pts) ) pt0 ))	;;; summed angle to procees
(setq ang (list (* (cos ang ) dis) (* (sin ang) dis) (nth 2 pts)))				;;; applying rotation by blocks zero
(setq ang (apply 'mapcar (cons '+ (list ang  (car lstx))) )) 					;;; applying tanslation
)






(defun C:1DL () ;; check for last proceed for 1DFEX 1DBB

(setq vs (ssadd) )
(if MBL (setq vs (ssadd MBL vs ) ))
(if MTE (setq vs (ssadd MTE vs ) ))
(C:33)
)





(defun C:1DBB (/ ;; 17.07.2015

 BLCOLL DB DBAC DBCC DBDD DBF DBL DBLN DBMM DBMT DB2P DBP DBSS DBT DBX DIV DIVS DVX
ENB ENS ENT ESL F FLS 
GBC_A GBC_B GBC_E
 ID IND INLAY IV IX IXX LAYW LMODE MATT MB
MBR ML MT 
;MTE MBL
MTF MTFX MTL MTLX MX MXB MXE OBN PREC Q RF RS SF SPT SV VS X

)
;;; CONVERT BLOCK 3DFACES to OBJ 18:11 29.06.2015
;;; Convert LWPOLYLINES to NGONS with lowering down to 0.16m 10:41 01.07.2015

(princ "\n1DBB___  v5.17 14.07.2022\n")

(if (not (eq (getvar "INSUNITS" ) 4) )   (alert "INSUNITS != 4, changed"))
(setvar "INSUNITS" 4)

;; (BSS (car (entsel)))
(setq vs (ssget '((0 . "INSERT"))) idb -1 MTB nil MTPS nil)

;[
(if (= 0 (getvar "GRIDMODE")) (setq GM 0) (setq GM nil))
(princ "\n[GRID compartible function (F7)]")
(if GM (princ "\n>>>") (princ "\n   "))
(princ "[OFF]  = PLINES AS WIRES")
(if GM (princ "\n   ") (princ "\n>>>"))
(princ "[ON]   = PLINES AS NGONS")
;]


(setq id -1 AA nil RS 1000.0 SXF 0.0 SYF 0.0 DIV 15  DBT "" OBN "" iv 0 ifile 0 ix 0)



;;; =================== SETTINGS[
(setq OBN (getstring "\nOBN enter obj name prefix (space for skip):"))
(if $$1DB  (progn (princ "\n ___________\n") (princ $$1DB) (setq FLS (getstring "\n Load Previous Settings? <space = yes, anykey = no>")) ))


(if (= FLS "")

(mapcar 'set (list 'spt 'lmode 'RF 'RS 'DIV 'PREC 'layw 'SXF 'SYF 'SYZ 'TEXX) $$1DB)
(progn

(setq TEXX (getint "\nTEXX (export texts <space = no, anykey = yes>) <no>:"))
(setq DIMTEX (getreal "\nDIMTEX reduce text value <10>:"))
(if (not DIMTEX) (setq DIMTEX 10))

;;points [
(if $$1DPOINTBASE (setq SXF (car $$1DPOINTBASE) SYF (cdr $$1DPOINTBASE)  )  )
(if (not SXF) (setq SXF 0.0))
(if (not SYF) (setq SYF 0.0))
(setq SXX (getreal (strcat "\n X Shift <" (rtos SXF 2 8) ">")))
(setq SYY (getreal (strcat "\n Y Shift <" (rtos SYF 2 8) ">")))

(cond 
((and (not SXX) $$1DPOINTBASE) (setq SXF (car $$1DPOINTBASE) ) )
(SXX (setq SXF SXX))
( t (setq SXF 0.0 ) )
)

(cond 
((and (not SYY) $$1DPOINTBASE) (setq SYF (cdr $$1DPOINTBASE) ) )
(SYY (setq SYF SYY))
( t (setq SYF 0.0 ) )
)

(setq $$1DPOINTBASE (cons SXF SYF))
;;]

(setq RS (getreal	"\nRS (Reduce scale for scene) <1000>:"))
(setq DIV (getint	"\nDIV (Curve segmenting) <8>:"))

)
)




;;; ========= Checking Input data convert conditions ===========
(if (not OBN) (setq OBN ""))

(if (not (eq (type RS) 'REAL)) (setq RS 1000.0))
(if (or (not (eq (type DIV) 'INT)) (< DIV 1) ) (setq DIV 8))
(if (not (eq (type PREC) 'INT)) (setq PREC 8))

(setvar "DIMZIN" 8)

(setq $$1DB (list spt lmode RF RS DIV PREC layw SXF SYF SYZ TEXX) )

(setq DIVS (list (/ SXF RS) (/ SYF RS) 0.0) )




;;; =================== SETTINGS]


(defun 1D_BFACES ( BLK /

BLCOLL BLKS BNAME ENT GBX INLAY

GBC_B GBC_B3 GBC_A GBC_E

;BNAME	;; NAME OF current Block
;GBC_E	;; flat dbase
;GBC_B	;; block's dbase
;GBC_B	;; block's dbase with internal
;INLAY	;; block's count
;ENS	;; flat types conslist
;ENB	;; blocks names conslist
;ESL	;; byblock layers
;ENL	;; block layers conslist
;ENSC	;; flat count
;ENBC	;; block count
;MODE	;; silence mode 1 = loud
;DO_IT2	;; Subroutine
;TTT2	;; Subroutine

)


;|
(defun entmake-pline (lst) (entmakex (append (list '(0 . "LWPOLYLINE") '(100 . "AcDbEntity") '(100 . "AcDbPolyline") (cons 90 (length lst))) (mapcar '(lambda (x) (cons 10 x)) lst))) )
(defun LM:LWVertices ( e ) (if (setq e (member (assoc 10 e) e)) (cons (CDR (assoc 10 e)) (LM:LWVertices (cdr e)) ) ) )
(setq MB '((500.0 500.0 0.0) 1.0) MT (LM:LWVertices (entget (car (entsel)))))
(entmake-pline (mapcar '(lambda (q) (1dtrans (append q '(500.0)) MB) ) MT))
|;


;;; ======= defun 1 :flat ==========
(defun DO_IT4 ()  ;; returns GBC_A and GBC_E structures, require ENT and MATT
(if (= (vla-get-objectname ENT) "AcDbBlockReference")			;; if block transformation matrix = ((3911.88 10399.8 0.0) (-0.29446 0.29446 0.29446) 5.12716)
	(progn	(TTT4 ENT  (list 
(apply 'mapcar (cons '+ (list (car MATT) (1d_axprop ENT 'Insertionpoint) ) ))

(apply 'mapcar (cons '* (list (nth 1 MATT)
(list 
(1d_axprop ENT 'XEffectiveScaleFactor )
(1d_axprop ENT 'YEffectiveScaleFactor )
(1d_axprop ENT 'ZEffectiveScaleFactor )
)

) ))

(+ (vla-get-rotation ENT) (last MATT))) ) )						;; proceeding nested of block 1dtrans
	(progn
(cond
((= (1d_axprop ENT 'ObjectName) "AcDbBlockReference") (setq INLAY 0))			;;;; setting INDexes INDexworks block BB
((= (1d_axprop ENT 'ObjectName) "AcDbFace") (setq INLAY 1))				;;; INLAY is converted to IND to proceed separately
((= (1d_axprop ENT 'ObjectName) "AcDbPolyline") (setq INLAY 2))
((= (1d_axprop ENT 'ObjectName) "AcDb3dPolyline") (setq INLAY 3))
((= (1d_axprop ENT 'ObjectName) "AcDbLine") (setq INLAY 4))
((= (1d_axprop ENT 'ObjectName) "AcDbArc") (setq INLAY 5))
((= (1d_axprop ENT 'ObjectName) "AcDbEllipse") (setq INLAY 20)) ;; ELLS
((= (1d_axprop ENT 'ObjectName) "AcDbCircle") (setq INLAY 6)) ;;; 1dbb
((= (1d_axprop ENT 'ObjectName) "AcDbText") (setq INLAY 7)) ;;; 1dbb
((= (1d_axprop ENT 'ObjectName) "AcDbMText") (setq INLAY 7)) ;;; 1dbb
((= (1d_axprop ENT 'ObjectName) "AcDbSpline") (setq INLAY 8)) ;;; 1dbb
((= (1d_axprop ENT 'ObjectName) "AcDbAlignedDimension") (setq INLAY 9)) ;;; 1dbb
((= (1d_axprop ENT 'ObjectName) "AcDbRotatedDimension") (setq INLAY 9)) ;;; 1dbb
;((= (1d_axprop ENT 'ObjectName) "AcDb2LineAngularDimension") (setq INLAY 9)) ;;; 1dbb MB
((= (1d_axprop ENT 'ObjectName) "AcDbSubDMesh") (setq INLAY 11)) ;;; 1dbb MB meshworks BB
((= (1d_axprop ENT 'ObjectName) "AcDbPoint") (setq INLAY 24)) ;;; 1dbb MB meshworks BB
((= (1d_axprop ENT 'ObjectName) "AcDb2dPolyline") (setq INLAY 3));1dbb DB2P
(t nil)
)

(setq GBC_E (cons (list  INLAY (vlax-vla-object->ename ENT) MATT) GBC_E))
)		;; collecting pure nested entity
)
(setq GBC_A (cons (cons INLAY (vlax-vla-object->ename ENT))  GBC_A))	;; collecting any nested ename
)

;;; ======= defun 2 :deep ==========
(defun TTT4 (BLK  MATT /)							;; Block_Vla-Object, Nested_Level
(setq GBC_E (cons (list 0 (vlax-vla-object->ename BLK)  MATT ) GBC_E))
(vlax-for ENT (vla-item BLCOLL (vla-get-name BLK) )
(DO_IT4)
)
)

(defun GREP_ENT (NAME GBC_E /) (vl-remove-if 'null (mapcar '(lambda (q) (if (= (cdr (assoc 0 (entget (nth 1 q)))) NAME)  ;;; grep items
q
)) GBC_E)
) 
)


;;; =========================================
;;; ============================= MAIN FUNC
;;; =========================================


;;(getstring "\nStart Export?")




(STIMER)

(if (= (type BLK) 'ENAME) (setq BLK (vlax-ename->vla-object BLK)))

(setq BLKS (acet-table-name-list (list "block" 1 4 16)) )
(setq
GBC_E nil
GBC_B nil
GBC_A nil
INLAY 0
ENS nil
ESL nil
ENB nil
MATT (list '(0.0 0.0 0.0) '(1.0 1.0 1.0) 0.0 ) ;; transformation matrix (loc scale rot) for 1dtrans ok
)

;;(setq IUSCALE (vla-get-InsUnitsFactor BLK))     ;;; IUSCALE
(setq BNAME (vla-get-Effectivename  BLK))	;;; INDexworks BB
(setq BNAME  (1DLatin BNAME 1) ) ;; interpolatin to Latin*
(vlax-for ENT (vla-item (setq BLCOLL (vla-get-blocks (vla-get-activedocument (vlax-get-acad-object) ) ) )   (vla-get-name BLK) )
(DO_IT4)
)


(setq DB (GREP_ENT "3DFACE" GBC_E))		;;; getting 3DFACES from selection
(setq DBL (GREP_ENT "LWPOLYLINE" GBC_E))	;;; getting LWPOLYLINES from selection
(setq DBP (GREP_ENT "POLYLINE" GBC_E))		;;; getting 3DPOLYLINES from selection
(setq DBAC (GREP_ENT "ARC" GBC_E))		;;; getting ARCS from selection
(setq DBLN (GREP_ENT "LINE" GBC_E))		;;; getting LINES from selection
(setq DBCC (GREP_ENT "CIRCLE" GBC_E))		;;; getting CIRCLE from selection
(setq DBSS (GREP_ENT "SPLINE" GBC_E))		;;; getting SPLINE from selection
(setq DBMM (GREP_ENT "MESH" GBC_E))		;;; getting MESH from selection meshworks BB
(setq DBDD (GREP_ENT "DIMENSION" GBC_E))	;;; getting DIM from selection
(setq DBPT (GREP_ENT "POINT" GBC_E))		;;; getting POINT from selection
(setq DBEL (GREP_ENT "ELLIPSE" GBC_E))		;;; getting ELLIPSE from selection ELLS

(setq DBMT (vl-remove-if 'null (mapcar '(lambda (q) (if (member (cdr (assoc 0 (entget (nth 1 q)))) '("TEXT" "MTEXT"))  ;;; getting TEXT from selection textworks BB
q
)) GBC_E)
))


(setq DB (append DB DBL))	;;; INDexworks
(setq DB (append DB DBP))
(setq DB (append DB DBLN))
(setq DB (append DB DBAC))
(setq DB (append DB DBCC))
(setq DB (append DB DBSS))
(setq DB (append DB DBDD))
(setq DB (append DB DBEL))	;; ELLS
(setq DB (append DB DBMT))	;; textworks
(setq DB (append DB DBMM))	;; meshworks BB
(setq DB (append DB DBPT))	;; points BB


;; DB === ((3 <Entity name: b49d300e50> ((0.0 0.0 0.0) (1.0 1.0 1.0) 0.0)) (3 <Entity name: b49d300df0> ((0.0 0.0 0.0) (1.0 1.0 1.0) 0.0)))


;;(setq OBN (strcat "+" (itoa ifile)))
(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") )  MTFX (strcat (getvar "DWGPREFIX") (getvar "DWGNAME")))

(while DB
(setq MTL (cat 8 (entget (nth 1 (nth 0 DB)))) AA (cons MTL AA) )   ;;; define matt 
;;(setq MTLX  (vl-list->string (mapcar '(lambda (q) (if (not (<= 47  q 122) ) (fix (1D_INP  q '((123.0 . 255.0) (65.0 . 90.0))))  q)) (vl-string->list MTL) ))) ;; interpolatin to latin
(setq MTLX  (1DLatin MTL 1) ) ;; interpolatin to Latin*
(setq DBF nil)
(setq id -1 DBX nil ixx 0)

(repeat (length DB)
(setq SV "" SF "" )
(setq ML (cat 8 (setq MT (entget (setq MTE (nth 1 (setq MB (nth (setq id (1+ id)) DB)) )) )) ))

;;; applying shift and scale
(if (eq MTL ML)

(progn
	(setq IND (car MB))
	(setq MB (last MB)) ;;; catched transforms
;;	(setq lw (vce MTE)) ;;; vla-object


;;; ==============================
;;; GETTING VERTICES FROM GEOMETRY
;;; ==============================


(setq qqZER mte)
(setq MX (1DV_EXTRACT MTE DIV TEXX DIMTEX) )

(if (= IND 11) (setq MXE (cdr MX) MX (list (car MX))) ) ;; meshworks BB - trailing faces INDces

(setq DVX nil)

(foreach nbb MX

;; (setq nbb (nth 0  (1DV_EXTRACT MTE DIV ) ))
;;;(setq qqq MB)
(setq MXB (mapcar '(lambda (q) (1dtrans q MB)) nbb))	;;; applying inblock modification to vertices

(if (and (vlax-property-available-p (vce MTE) 'closed) (eq (1d_axprop MTE 'closed) :vlax-true) ) (setq IND 6) ) ;; now that's I calling recycling (c)

(if (>= (apply '* (nth 1 MB)) 0) 
(setq MBR MXB ) ;;; original normals order
(if (= IND 11) ;; meshworks BB
(progn (setq MBR MXB ) (setq MXE (mapcar '(lambda (q) (reverse q)) MXE)))  ;;; flipping mirror mesh
(setq MBR (reverse MXB) )) ;;; fixed mirrored flipped normals order 
)


;;; ========================================================
;;; ----------------- Applyng Scale, rotate, flip, translate
;;; ========================================================




	(setq DVX (cons
		(list IND (mapcar  '(lambda (q / f) (setq f (apply 'mapcar (cons '+ (list (mapcar '(lambda (x) (/ x RS) ) q) '(0.0 0.0 0.0) ) )) ) ;;; no DIVS movement inside block
;;				(list (nth 0 f) (nth 2 f) (- (nth 1 f)) )		;;; flip YZ axis
f
				)
		MBR)) 	;;; gathered verts
	 DVX)
	ixx (1+ ixx)
	)
) ;;; foreach

(if (= IND 11) (setq DVX (list (append (car DVX) (list MXE) ))) ) ;; meshworks BB - retrailing faces INDces to last element
(setq DBX (append DVX DBX))
)



(setq DBF (cons MB DBF) )
) ;;(eq MTL ML) 




(if (> ixx 222) (prinCout DBX BNAME))

(if (> ix 222222) ;;; 8888 222222
	(progn 
	(prinCout DBX BNAME)
	;;(setq OBN (strcat "+" (itoa (setq ifile (1+ ifile) ) ) ))	;;;$$$
	(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") ))
	(1D:Write_FILE DBT 0 ".obj" 1)
	(setq DBX nil DBT "" SF "" SV "" iv 0 ix 0)
	(gc)
	)
)

);; repeat

(prinCout DBX BNAME)
(princ "\n______  ") (princ MTL) (princ " ___ ") (princ MTLX)
(if (> (length DBF) 0) (setq DB DBF) (setq DB nil))

) ;;while
) ;;defun

;;;;


;;; ==============================
;;; ==============================
;;; ==============================
;;; FEB MAIN CODE




(repeat (sslength vs)
(setq MBL (ssname vs (setq idb (1+ idb))))
(if (and (eq "Model" (cdr (assoc 410 (entget MBL))))  (not (vlax-Property-Available-p (vlax-ename->vla-object  MBL) 'Path) ) )
(progn

(setq MBNAME (1d_axprop MBL 'effectivename))
(princ "\n ==========\n===") (princ MBNAME)

(if (not (member MBNAME MTB) )
(progn
(setq MTB (cons MBNAME MTB))
(setq QQBB MBL)
(1D_BFACES MBL)
)
)

(setq IUSCALE (1d_axprop MBL 'InsUnitsFactor))

(setq MTPS (cons (list (1DLatin MBNAME 1)  (apply 'mapcar (cons '+ (list (mapcar '(lambda (q) (/ q RS)) (1d_axprop MBL 'InsertionPoint)) DIVS ) ))  ;;; no RS division at objects
(list 
(* IUSCALE (1d_axprop MBL  'XEffectiveScaleFactor ))
(* IUSCALE (1d_axprop MBL  'YEffectiveScaleFactor ))
(* IUSCALE (1d_axprop MBL  'ZEffectiveScaleFactor ))

 )  (1d_axprop MBL 'rotation) ) ;;; collecting INSERTIONLIST 
MTPS))

);; progn
)

);; repeat

;;; GLOBAL OUTPUT

;; (setq OBN (strcat "+" (itoa (setq ifile (1+ ifile) ))))
(princ "\n Total files:  ")
(princ ifile)
(princ "\n Time : " )
(setq MTF (strcat (getvar "DWGPREFIX") OBN (getvar "DWGNAME") ))
(1D:Write_FILE DBT 0 ".obj" 1)

(if MTPS
(progn
(setq FX "") (foreach n MTPS  (setq FX (strcat "\n" (vl-princ-to-string n) FX)))
(setq FX (strcat "\n# Blocks = "(vpc MTB) FX))
(setq FX (strcat "\n# Blocknames = "(vpc (mapcar '(lambda (q) (1DLatin q 1)) MTB )) FX))
(1D:Write_FILE FX 1 ".txt" 0) ;; same filename
)
)

)
;; defun 1DBB























;;; ======================================== 
;;; ======================================== 
;;; ======================================== 
;;; ======================================== 
;;; ======================================== 
;;; ===================== WALLGROW SECTION

(defun 1D_GTCycle ( f ff strx / cfc lwt MT)
(setq cfc nil)
(while (not cfc)
(princ strx)
(setq lwt (eval ff))
	(if lwt
		(progn
		;;; --------- >> -----------
(if f 
		(if  (member (cdr (assoc 0 (entget (car lwt)))) f )
				(progn
					(setq MT (car lwt))
					(setq CFC 0)					;;; final
					(setq lwt (vlax-ename->vla-object (car lwt)))	;;; 
				)
			(princ "\n Wrong object type")
		)

		(progn
			(setq MT (car lwt))
			(setq CFC 0)					;;; final
			(setq lwt (vlax-ename->vla-object (car lwt)))	;;; 
		)
)
		)
	(princ "\n Object wasn't selected ")
	)
)
MT
)

(defun 1D_TWOX (a b) (mapcar '+ (mapcar '- b a) b ) )
(defun 1D_MID (ll) (mapcar '(lambda (x) (/ x (length ll))) (apply 'mapcar (cons '+ ll ) ) ))
(defun CAT (x d /) (cdr (assoc x d)))	;;$$##
(defun CAF (a x d /) (cons a (cdr (assoc x d)) ) )
(defun 1dper  (pt dis ang  / )  (list (+ (nth 0 pt) (* (cos ang ) dis)) (+ (nth 1 pt) (* (sin ang) dis))  ) ) ;; perpendicular
(defun 1dperZ (pt dis ang  / )  (list (+ (nth 0 pt) (* (cos ang ) dis)) (+ (nth 1 pt) (* (sin ang) dis)) (nth 2 pt) ) )
(defun replacenth (lst i itm) (setq i (1+ i)) (mapcar '(lambda (x) (if (zerop (setq i (1- i))) itm x) )  lst) )
(defun EntNext_to_End (ent / a)     (if (setq ent (entnext ent))       (cons ent (EntNext_to_End ent))))

(defun 1D_range (l db / )
(setq l (vl-remove-if 'null (mapcar '(lambda (q) (if (<= (car l) q (cadr l)) q  )) db)))	;; range cl
(mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l))						;; recycle list
)


(DEFUN VS777 (/)
(if (and (/= VS  nil) (> (sslength VS) 0))
(progn
(princ "\n")
(command "_SELECT" "_U" "")
;(command "_ZOOM" "_O" VS "")
;(command "_ZOOM" "_S"  "0.9x")
(princ (sslength vs))
(SSSETFIRST VS VS)

);progn
);if
(princ "...done")
(princ)
);defun


(defun 1D_FRAME (p1 p2 hx dx /
A B DH DX L L1 L2 L3 P3 P4 PA PB PC PD Q RANG
)
;; (1D_FRAME (getpoint) (getpoint) '(3000.0 9000.0) 750.0)
(setq p1 (list (car p1) (cadr p1) (car hx)) p2 (list (car p2) (cadr p2) (car hx)) )
(setq rang (angle p1 p2) dh (- (cadr hx) (car hx)))
(setq	pa  (mapcar '+ (append (1dper p1 dx rang) (list (car hx)) ) (list 0 0 dx ))
	pb  (mapcar '+ (append (1dper p2 (- dx) rang) (list (car hx)) ) (list 0 0 dx ))
	pc  (mapcar '+ pb (list 0 0 (- dh (* 2 dx))) )
	pd  (mapcar '+ pa (list 0 0 (- dh (* 2 dx))) )
	p3  (mapcar '+ p2 (list 0 0 dh) )
	p4  (mapcar '+ p1 (list 0 0 dh) )
l1 (list p1 p2 p3 p4)
l1 (mapcar 'list l1 (append (cdr l1) (list (car l1))) )
l2 (list pa pb pc pd)
l3 (mapcar '(lambda (q) (1dperZ q dx (+ rang (/ pi 2)))) l2)
l2 (mapcar 'list l2 (append (cdr l2) (list (car l2))) )
l3 (mapcar 'list l3 (append (cdr l3) (list (car l3))) )
l (mapcar '(lambda (a b) (list (car a) (car b) (cadr a) (cadr b)))  l1 l2)
l2 (mapcar '(lambda (a b) (list (car a) (car b) (cadr a) (cadr b)))  l2 l3)
)
(foreach q l (3DFaceD (nth 1 q) (nth 3 q) (nth 2 q) (nth 0 q) (getvar "CLAYER")) )
(foreach q l2 (3DFaceD (nth 1 q) (nth 3 q) (nth 2 q) (nth 0 q) (getvar "CLAYER")) )
)


;;;; ==============================================
;;;; --------------------------------------- HOR
;;;; ==============================================
(defun 1D_HIMP (pt1 pt2 rd hd mod sh /  DB DIS L PT1 PT2 Q RANG RD ) ;; horiz
;;   _               _       _      _      _
;; .|_| 00 .|_| 11 .| | 13 .|  22 .|_  12 .  32 .|_ 21
;;
;; (1D_HIMP pt1 pt2 200.0 '(0 300) 1 2) 

(setq	pt1 (list (nth 0 pt1) (nth 1 pt1) )
	pt2 (list (nth 0 pt2) (nth 1 pt2) )
	rang (angle pt1 pt2)
	dis (distance pt1 pt2)
	hd (vl-sort hd '<))
;;;; _______________________________________ first point contour
(setq l (mapcar '(lambda (q) (append (1dper pt1 rd (+ rang (* (/ pi 2) (car q))))  (cdr q) ) )
(mapcar 'list '(-1  -1  1  1) (list (nth 0 hd ) (nth 1 hd ) (nth 1 hd ) (nth 0 hd ) ) )
))
(setq db (mapcar 'list l (append (cdr l) (list (car l))) ))
;;;; _______________________________________ second point contour
(setq l (mapcar '(lambda (q) (append (1dper pt2 rd (+ rang (* (/ pi 2) (car q))))  (cdr q) ) )
(mapcar 'list '(-1  -1  1  1) (list (nth 0 hd ) (nth 1 hd ) (nth 1 hd ) (nth 0 hd ) ) ) ))
(setq l (mapcar 'list l (append (cdr l) (list (car l))) ))
;;;; _______________________________________ proceed values
(setq l (mapcar 'list db l))
(setq l (mapcar '(lambda (q) (apply 'append q) ) l)) ;; compress
(repeat sh (setq l (append (cdr l) (list (car l)))) )
(repeat mod (setq l (cdr l)) )
(mapcar '(lambda (q) (3DFaceD (nth 1 q) (nth 3 q) (nth 2 q) (nth 0 q) (getvar "CLAYER"))) l)
)


;;;; ==============================================
;;;; --------------------------------------- VERT
;;;; ==============================================

(defun 1D_VIMP (pt1 rd hd rang cut / l HD PT1 Q RANG RD) ;; I (1D_VIMP pt 40.0 '(0.0  500.0) rang '(0 . 0))
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) hd (mapcar 'list hd) )
(setq l (mapcar '(lambda (q) (1dper (1dper pt1 rd (+ rang (* pi (car q)))) rd (+ rang (* pi (cdr q)))) )
'((-1 . 0.5) (-1 . -0.5) (0 . -0.5) (0 . 0.5) ) ) )	;;; plan view
(setq l (mapcar 'list l (append (cdr l) (list (car l))) ))

(repeat (car cut) (setq l (append (cdr l) (list (car l))) ) )
(repeat (cdr cut) (setq l (cdr l)))

(mapcar '(lambda (q)
(3DFaceL (mapcar '(lambda (a b) (append (nth a q) (nth b hd)) ) '(0 0 1 1 ) '(0 1 1 0) ) nil)
;(3DFaceD (append (nth 0 q) (car hd) ) (append (nth 0 q) (cadr hd)) (append (nth 1 q) (cadr hd)) (append (nth 1 q) (car hd))  (getvar "CLAYER") )
) l)

;;(1D_VIMP (getpoint) 40.0 '(0.0 500.0) (angle (getpoint) (getpoint) )  '(0 . 1))
)



(defun 3DFaceD (p1 p2 p3 p4 lay)
    (entmakex (list (cons 0 "3DFACE")
                    (cons 8 lay)
                    (cons 10 p1)
                    (cons 11 p2)
                    (cons 12 p3)
                    (cons 13 p4)
              ) ;_ end of list
    ) ;_ end of entmakex
)

;;; list datadriven 3Dface
(defun 3DFaceL (lstx lay) (if (not lay) (setq lay (getvar "CLAYER")))
(entmakex (append (list (cons 0 "3DFACE") (cons 8 lay)) (vl-remove-if 'null (mapcar '(lambda (a b) (if b (cons a b)) ) '(10 11 12 13) lstx ))))
)


(defun entmake-Cpline (lst)
  ;; lst -       
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.)))
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.) (100. 0.)))
  (entmakex (append (list '(0 . "LWPOLYLINE")
                          '(100 . "AcDbEntity")
                          '(100 . "AcDbPolyline")
                          (cons 90 (length lst))
			   '(70 . 1)
                          ) ;_ end of list
                    (mapcar '(lambda (x) (cons 10 x)) lst)
                    ) ;_ end of append
            ) ;_ end of entmakex
  )

;;;   create build polyline
(defun entmake-pline (lst)
  ;; lst -       
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.)))
  ;; (entmake-pline '((0. 0.) (10. 20.) (30. 50.) (100. 0.)))
  (entmakex (append (list '(0 . "LWPOLYLINE")
                          '(100 . "AcDbEntity")
                          '(100 . "AcDbPolyline")
                          (cons 90 (length lst))
                          ) ;_ end of list
                    (mapcar '(lambda (x) (cons 10 x)) lst)
                    ) ;_ end of append
            ) ;_ end of entmakex
  ) ;_ end of defun



;; ================================== COMMANDS
;; ===========================================
;; ===========================================






(defun C:1DWF (/ ;; store windows height
DBZ DBX DBS MT id
)


(princ "\n
\n Color Table: 10-49 FWIN  50-89 DOOR  90-129 WDOOR  130-159 WBALC
")

(setq vs (ssget '((0 . "LWPOLYLINE"))))
(setq id -1 DBS nil)
(repeat (sslength vs)
(setq MT (entget (ssname vs (setq id (1+ id)))))
(setq DBS (cons (cons 
(if (cdr (assoc 62 MT)) (cdr (assoc 62 MT)) 256)		;; color
(list (vl-sort (mapcar 'cadr (LM:LWVertices MT)) '>))			;; Y verts
) DBS)
)
)


(setq DBZ (apply 'append (mapcar 'cadr DBS)))
(setq DBX (apply 'min DBZ))					;; min value


(setq DBS (mapcar '(lambda (q) (list (car q) (mapcar '(lambda (z) (- z DBX)) (cadr q)) )) DBS))		;;(4 (500.0 2000.0))
(setq DBZ (remove-doubles (vl-sort  (apply 'append (mapcar 'cadr DBS)) '<)))
(setq l DBZ)
(setq DBX (cons (apply 'min l) (apply 'max l)) )

(if (> (apply ' max l) 2640.0) (setq AF (apply ' max l) ) (setq AF 2640.0 ) )

(setq DBF (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l)))
(setq $$DBS DBS)


;;;  ============= REMOVAL REST ==============
;|
(setq $$DBS			
(mapcar '(lambda (q) (append (list (car q))
(setq 	l (vl-remove-if 'null (mapcar '(lambda (z) (if (<= (nth 0 (cadr q) ) z (nth 1 (cadr q))) z) ) DBZ))	;; forms bypass list
	l (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l))							;; forms true (list)
l (vl-remove-if '(lambda (q) (member (vpc q) (mapcar 'vpc l))) DBF)						;; rest removal
)
)) DBS )
)
|;

(setq $$DBS			
(mapcar '(lambda (q) (list (car q)
(list (car DBX) (nth 1 (cadr q)) ) (list (nth 0 (cadr q)) (cdr DBX))
)
)  DBS )
)

(setq	l DBZ
	l (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l))
)

(setq $$DBS (vl-sort $$DBS (function (lambda (e1 e2)(> (car e1) (car e2))) )))
(setq $$DBS (cons (append '(256) l) (cdr $$DBS) ) )


;| no not
;;(mapcar '(lambda (q) (list (car q)
;;(vl-remove-if 'null (mapcar '(lambda (z) (if (<= (nth 0 (cadr q) ) z (nth 1 (cadr q))) z) ) DBZ))
;; )) DBS)
|;


(setq $$DBS (append $$DBS 
(list

(list 1	'(0.0 720.0) (list 2360.0 AF))				;; win
(list 2	'(0.0 0.0) (list 2360.0 2640.0) (list 2640.0 AF) )					;; door 1x
(list 3	'(0.0 0.0) (list 2360.0 2640.0) (list 2640.0 AF) )					;; door 2x
(list 4	'(0.0 0.0) (list 2360.0 2640.0) (list 2640.0 AF) )					;; door balcony
)
))

(princ $$DBS)
(princ)
) ;; 1DWF






(defun 1D_LM:GetDynProps (obj)
  ;; Lee Mac  ~  07.04.10
(vl-remove-if (function (lambda (x) (= (car x) "Origin")))   
(mapcar
    (function
      (lambda (x / v)
        (cons (vla-get-PropertyName x)
              (if (= 8192 (logand 8192 (vlax-variant-type (setq v (vla-get-value x))) ))
                (vlax-safearray->list
		(vlax-variant-value v)
		)
                (vlax-variant-value v) ))))
      
    (vlax-invoke obj 'GetDynamicBlockProperties)))
)







(defun C:1DWG (/
A B  DB DBF DBX  DST ELST FLG ID L LW LWT MT MTN N OSM P1 P2 PM PT1 PT2 PT3 Q QQ QX RANG SS clr sq
 DL CL DS
)  ;;; build 3d walls recursive
;; 1D_MID 1dper 1dperZ 1D_FRAME 1D_VIMP 1D_HIMP 1D_FRAME vcget 3DFACED CAT CAF replacenth EntNext_to_End
;; 1D_range
;; 17:31 20.03.2015 // custom height and balcony // multicolor

(command "_UCS" "_W")

;;(1D:LAYDEF "0" '(7 "Continuous" 0.1) 1 0)
(1D:LAYDEF "1D_WIN3D" '(4 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_DOR3D" '(60 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_DOR2-3D" '(120 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_WALL3D" '(20 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_VOID3D" '(202 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_VOID3DBW" '(142 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_VOID3DD" '(92 "Continuous" 0.15) 1 1)



;[
(if (= 0 (getvar "GRIDMODE")) (setq GM 0) (setq GM nil))
(princ "\n[GRID compartible function (F7)]")
(if GM (princ "\n>>>") (princ "\n   "))
(princ "[OFF]  = 800 AUTODIST CLOSEST BLOCK FROM PLINES")
(if GM (princ "\n   ") (princ "\n>>>"))
(princ "[ON]   = MANUAL BLOCK SELECTION")
;]



(if $$DBS (setq DS $$DBS) )



(setq DS (getint "\n 1=common 2=low_first 3=high_first 4=common_first <1>"))
(if (not DS) (setq DS 1))

(if $$DBS (setq DS $$DBS) )

(cond

( (= DS 1)
	(setq DS
	'((256	(0.0 720.0) (720.0 2260.0) (2260.0 2640.0) )		;; wall
	(1	(0.0 720.0) (2260.0 2640.0))				;; win
	(2	(2260.0 2640.0))					;; door 1x
	(3	(2260.0 2640.0))					;; door 2x
	(4	(0.0 720.0) (2260.0 2640.0))					;; door balcony 18:00 10.01.2022
	))
)


( (= DS 2)
	(setq DS
	'(
	(256 (0.0 640.0) (640.0 840.0) (840.0 900.0) (900.0 3780.0))
	(1	(0.0 1530.0) (3470.0 3780.0))
	(2	(0.0 150.0 )(2220.0 3780.0))
	(3	(0.0 900.0) (3470 3780.0) )
	(4	(0.0 900.0)(3470.0 3780.0))					;; door balcony
	)
	)
)

( (= DS 3)
	(setq DS
	'(
	(256 (0.0 900.0) (900.0 940.0) (940.0 1140.0) (1140.0 3780.0))
	(1	(0.0 1860.0) (3370.0 3780.0))
	(2	(0.0 150.0 )(2220.0 3780.0) )
	(3	(0.0 150.0) (2220.0 3780.0) )
	(4	(2260.0 2640.0))					;; door balcony
	)
	)
)

( (= DS 4)
	(setq DS
	'(
	(256 (0.0 900.0) (900.0 1560.0) (1560.0 3780.0))  ;; (256 (0.0 640.0) (640.0 840.0) (840.0 900.0) (900.0 1560.0) (1560.0 3170.0) (3170.0 3780.0))
	(1	(0.0 1560.0) (3070.0 3780.0))
	(2	(0.0 150.0)(2220.0 3780.0) )
	(3	(0.0 150.0) (2220.0 3780.0) )
	(4	(2260.0 2640.0))					;; door balcony
	)
	)
)

);; cond







;|
(setq ds
'(
(256 (0.0 500.0) (500.0 2000.0) (2000.0 2737.8) (2737.8 3090.71) (3090.71 4337.8) (4337.8 4584.65) (4584.65 6108.01) (6108.01 7534.5))
(4 (0.0 2737.8) (4337.8 7534.5))
(3 (0.0 2737.8) (4337.8 7534.5))
(2 (0.0 0.0) (2000.0 7534.5))
(1 (0.0 500.0) (2000.0 7534.5))
)
); setq 
|;




(defun FWIN (pt1 pt2 lwt cl / ixf dsa ixq) ;; window
(setvar "CLAYER" "1D_WIN3D") 
(setq cl (cat cl ds) cl (list (cadar cl) (caadr cl)) )		;; levels (down top)
(if (> (vcget "02" lwt pt2) (vcget "02" lwt pt1) ) (setq pt3 pt1 pt1 pt2 pt2 pt3) )	;;directional truesort
(setq rang (angle pt1 pt2) pt1 (1dper pt1 120.0 (+ rang (/ pi 2)) ) pt2 (1dper pt2 120.0 (+ rang (/ pi 2)) ) )
(1D_FRAME pt1 pt2 cl 80.0)
(setq pt1 (1dper pt1 80.0 (+ rang (/ pi 2)) ) pt2 (1dper pt2 80.0 (+ rang (/ pi 2)) ) pt3 (1D_MID (list pt1 pt2)) )
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) pt2 (list (nth 0 pt2) (nth 1 pt2) ) pt3 (list (nth 0 pt3) (nth 1 pt3) ) )


(if (> (distance pt1 pt2) 1245.0)


;;; mass distribute
(if (< (distance pt1 pt2) 2100.0 )

;;; single distribute
(1D_VIMP pt3 60.0 cl rang  '(0 . 0))

(progn
(setq pt3 nil ixf (distance pt1 pt2) dsa (/ ixf  (fix (/ ixf 700.0) ) ) iq 0)
(repeat (- (fix (/ ixf 700.0) ) 1)
(1D_VIMP 
(1dper pt1  (* (setq iq (1+ iq) ) dsa) rang  )
 60.0 cl rang  '(0 . 0))
 pt3)

)

)

);; if





 ;;(720.0  2360.0)
;|
(mapcar '(lambda (q)
(3DFaceD (append pt1 (list (nth 0 q))) (append pt1 (list (nth 1 q))) (append pt2 (list (nth 1 q))) (append pt2 (list (nth 0 q))) "0" )
	;(setq vs (ssadd (entlast) vs))
	) (list cl)
)
|;
)



;(DOR (nth 0 q) (nth 1 q) lwt 2)
; (setq pt1 (nth 0 q) pt2 (nth 1 q) cl 2)


(defun DOR (pt1 pt2 lwt cl / )
(setq dbx (cons (list pt1 pt2 2) dbx))
(setq cl (cat cl ds))
(if (> (length cl) 1) (setq cl (list (cadar cl) (caadr cl))) (setq cl (list 0.0 (cadar cl))) ) ;; levels (down  top)

(if (> (vcget "02" lwt pt2) (vcget "02" lwt pt1) ) (setq rang (angle pt1 pt2)) (setq rang (angle pt2 pt1 )) )	;;directional truesort
(setq pt1 (1dper pt1 200 (- rang (/ pi 2)) ) pt2 (1dper pt2 200 (- rang (/ pi 2)) ) pt3 (1D_MID (list pt1 pt2)) )
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) pt2 (list (nth 0 pt2) (nth 1 pt2) ) pt3 (list (nth 0 pt3) (nth 1 pt3) ))
(setvar "CLAYER" "1D_DOR3D")
(1D_HIMP pt1 pt2 50.0 cl 1 3)

)



(defun WDOOR ( pt1 pt2 lwt cl /)	;; double door (WDOOR (nth 0 q) (nth 1 q) lwt 3)
(setq cl (cat cl ds))

(if (> (length cl) 1) (setq cl (list (cadar cl) (caadr cl))) (setq cl (list 0.0 (cadar cl))) ) ;; levels (down  top)
(if (> (vcget "02" lwt pt2) (vcget "02" lwt pt1) ) (setq rang (angle pt1 pt2)) (setq rang (angle pt2 pt1 )) )	;;directional truesort
(setq pt1 (1dper pt1 200 (- rang (/ pi 2)) ) pt2 (1dper pt2 200 (- rang (/ pi 2)) ) pt3 (1D_MID (list pt1 pt2)) )
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) pt2 (list (nth 0 pt2) (nth 1 pt2) ) pt3 (list (nth 0 pt3) (nth 1 pt3) ) )
(setvar "CLAYER" "1D_DOR2-3D")
(1D_HIMP pt1 pt2 75.0 (list (car cl) (+ (car cl) 80.0)) 0 0)		;; bottom 2 3
(1D_HIMP pt1 pt2 75.0 (list (cadr cl) (- (cadr cl) 80.0)) 0 0)		;; top 2 3
(1D_HIMP pt1 pt2 60.0 cl 0 0)						;; desc
(1D_VIMP pt1 80.0 cl rang '(0 . 0)) (1D_VIMP pt2 80.0 cl rang '(0 . 0)) (1D_VIMP pt3 80.0 cl rang '(0 . 0))

)




;;; WBALC

(defun WBALC (pt1 pt2 lwt cl / FLG rang clr)
(setvar "CLAYER" "1D_WIN3D")
(SETQ DD CL )
(SETQ CL dd)
(setq cl (cat cl ds))

(if (> (length cl) 1) (setq cl (list (list (cadar cl) (caadr cl))) ) (setq cl (list (list 0.0 (caar cl))) ) ) ;; levels (down  top)



 (setq clr (car (last cl)) )	;; down
 (setq cl  (cadr (car cl)))	;; up
;(setq clr (car cl))
;(setq cl (last cl) )

(if (> (vcget "02" lwt pt2) (vcget "02" lwt pt1) ) (setq rang (angle pt1 pt2) FLG nil) (setq rang (angle pt2 pt1 ) FLG 1) )
(setq pt1 (1dper pt1 200 (- rang (/ pi 2)) ) pt2 (1dper pt2 200 (- rang (/ pi 2)) ) )
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) pt2 (list (nth 0 pt2) (nth 1 pt2) ) )

;; ============================ doorpart
(1D_VIMP pt1 82.0 (list clr  cl ) rang '(0 . 0))  ;; door
(1D_VIMP pt2 82.0 (list clr  cl ) rang  '(0 . 0))	;;door
(1D_HIMP pt2 pt1 75.0 (list (- cl 80.0)  cl ) 1 1)	;; door top
(1D_HIMP pt1 pt2 70.0 (list (+ clr 720.0) (+ clr 800.0)) 1 3)	;; dor bott
(1D_HIMP pt1 pt2 55.0 (list clr (+ clr 800.0)) 1 3)	;;dorpart fill

;|
(if FLG
(3DFaceL (mapcar '(lambda (a b) (append a (list b)) ) (list pt1 pt1 pt2 pt2 ) (list clr cl cl clr) ) "0")
(3DFaceL (mapcar '(lambda (a b) (append a (list b)) ) (reverse (list pt1 pt1 pt2 pt2 )) (list clr cl cl clr) ) "0")
)
|;
)




;;;;;;;;;;;;;;;  ============================  MAINFUNC

(command "_UCS" "_W")
(setq vs (ssadd) DB nil ELST (entlast))
(if (not vss) (setq vss (ssadd)))
(setq MT (1D_GTCycle '("LWPOLYLINE") '(entsel) "\nBuilding Polyline Contour:"))
(setq sq (ssadd mt (ssadd)))

(setq lwt (vlax-ename->vla-object MT) l (LM:LWVertices (entget MT)) id -1 DBX nil DBF nil )

;;; ================ CUT
(setq p1 (getpoint "\n Start CUT:"))
(if p1
(progn
(setq p1  (vcget "0" lwt (trans p1 1 0)))
(setq p2 (vcget "01" lwt (trans (getpoint p1) 1 0) ))
(setq p1 (list (vcget "1" lwt p1) p2 ) )
(setq p1 (vl-sort p1 '<))
(setq l (vl-remove-if 'null (mapcar '(lambda (q) (if (<  (nth 0 p1)  (vcget "1" lwt q) (nth 1 p1) ) q) ) l)))
(setq l (append (cons (vcget "3" lwt (nth 0 p1) ) l) (list (vcget "3" lwt (nth 1 p1) ) ) ))
)
 (setq p1 nil)
)

;;;
(setq l (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l)))
(setq DBX  (mapcar '(lambda (q) (append q (list (1d_axprop MT 'color) ) )) l))
(setq OSM (getvar "OSMODE"))
(setvar "OSMODE" 0)

;; ===================================
;; -------------------------- BLOCKS
;; ===================================

;; -------------------- SELECTING Blocks
;; =====================================

(if 1DSETUP
(progn
(princ "\n == BLOCKS <memslot 4>:")
(setq ss (ssget '((0 . "INSERT"))) id -1 DB nil vsss (ssadd))

(if (and (not ss) vss)
  (progn
    (sssetfirst nil vss)
    (setq ss (getstring "\nUse memsot 4? <1 = yes, space = no>"))
    (if	(= ss "1")
      (progn (command "_SELECT" "_U" "") (princ "\nBlocks = OK") (setq ss vss) )
      (progn (command "_SELECT" "_U" "") (princ "\nBlocks = Empty") (setq ss nil) )
    )
  )
)

(if ss
(repeat (sslength ss)
(setq lw (vlax-ename->vla-object (setq MT (ssname ss (setq id (1+ id)) ))))
	(if (and (= (cdr (assoc 0 (entget MT))) "INSERT" )  (not (vlax-Property-Available-p (vlax-ename->vla-object MT) 'Path) ) )

		(progn
		(setq qx (1D_LM:GetDynProps lw))
		(if (or (vl-some '(lambda ( b ) (assoc b qx)) '("_"  "_" " 90" ""))
			(member  (1d_axprop lw 'effectivename) '("1DF_winX1") )	
			)

			(if GM
				(if (< (distance (1d_axprop lw 'Insertionpoint) (vcget "0" lwt (trans (1d_axprop lw 'Insertionpoint) 1 0)) ) 800.0 ) ;;; closest block to pline
				(setq DB (cons MT DB) vss (ssadd MT vss)) 	
				)														 ;;; closest block to polyine
			(setq DB (cons MT DB) vss (ssadd MT vss))
			)
		)
		;; ______________ Block Centercheck
		(vla-getboundingbox lw 'pt1 'pt2)
		(setq pm (list (vlax-safearray->list pt1) (vlax-safearray->list pt2)) )
;;		(command "_LINE" (nth 0 pm) (nth 1 pm) "") (setq vs (ssadd (entlast) vs) )
		(setq pt1 (mapcar (function (lambda (x) (/ x (length pm)))) (apply 'mapcar (cons '+ pm ) ) ))
		(if (or (> (distance (1d_axprop lw 'Insertionpoint) pt1) 3000.0)  (> (distance (nth 0 pm) (nth 1 pm)) 3000.0) )
(progn (setq vs (ssadd MT vs)) (princ "\n Far Center!!")) )
		)
	)
) ;; repeat
)

(if (> (sslength vs) 0) (progn (C:33) (getstring "\n FAR middlepoins found: Continue?") ) )
(setq vs (ssadd))

(C:44)
(getstring "\n Ready to produce?")



;; ------------------ COLLECTING Blocks
;; ====================================

(foreach n DB
(setq lw (vlax-ename->vla-object n) MTN (1d_axprop lw 'effectivename))
(setq qx (1D_LM:GetDynProps lw))
(vla-getboundingbox lw 'pt1 'pt2)
(setq pm (list (vlax-safearray->list pt1) (vlax-safearray->list pt2)) )
(setq pm (mapcar (function (lambda (x) (/ x (length pm)))) (apply 'mapcar (cons '+ pm ) ) ))
(setq pm (trans (vcget "0" lwt (trans pm 1 0)) 0 1))

;;; ========================================= DEFINE BOUNDS
(setq pt1 (1d_axprop lw 'Insertionpoint) )
(setq pt1 (trans (vcget "0" lwt (trans pt1 1 0)) 0 1))

;;; (if (not (setq dst (CAT "_" qx) )) (setq dst (CAT " 90" qx) ))
(setq dst (vl-some '(lambda (q) (CAT q qx) ) '("_" " 90" "") ))
(if (wcmatch MTN "1DF_winX1") (setq dst 1600.0))

(setq pt2 (vcget "0" lwt (1dper pt1 dst (angle pt1 pm) )) )		;;fdist



(cond
;;; _______________________________________
;;; ================================ WIN_WG

((OR (wcmatch MTN "*__*") (wcmatch MTN "*1DF_winX*") (wcmatch MTN "1DF_winX1") )
(setq
pt1 (vcget "0" lwt (1dper pt1 65 (angle pt1 pt2)))		;;fdist
pt2 (vcget "0" lwt (1dper pt2 65 (angle pt2 pt1)))		;;fdist
dbx (cons (list pt1 pt2 1) dbx)
)
(FWIN pt1 pt2 lwt 1)
)


;;; ___________________________________________
;;;; ============================== BALCONY_WG
((wcmatch MTN "*___*") (setq
pt1 (vcget "0" lwt (1dper pt1 65 (angle pt1 pt2)))		;;fdist
pt3 (vcget "0" lwt (1dper pt2 65 (angle pt2 pt1)))		;;fdist
pt2 (vcget "0" lwt (1dper pt1 765 (angle pt1 pt2)))		;;fdist
dbx (cons (list pt1 pt2 2) dbx) dbx (cons (list pt2 pt3 1) dbx) )

(setvar "CLAYER" "1D_WIN3D")
(if (> (vcget "02" lwt pt2) (vcget "02" lwt pt1) ) (setq rang (angle pt1 pt2) FLG nil) (setq rang (angle pt2 pt1 ) FLG 1) )	;;directional truesort

(WBALC pt1 pt2 lwt 4)

(setq pt1 (1dper pt1 200 (- rang (/ pi 2)) ) pt2 (1dper pt2 200 (- rang (/ pi 2)) ) pt3 (1dper pt3 200 (- rang (/ pi 2)) ) )
(setq pt1 (list (nth 0 pt1) (nth 1 pt1) ) pt2 (list (nth 0 pt2) (nth 1 pt2) ) pt3 (list (nth 0 pt3) (nth 1 pt3) ) )


;; ============================ winpart
(1D_VIMP pt3 80.0 '(720.0  2260.0) rang '(0 . 0))
(1D_HIMP pt3 pt2 70.0 '(720 800) 1 3)	;; bot
(1D_HIMP pt3 pt2 70.0 '(720 800) 1 3)	;;
(1D_HIMP pt3 pt2 75.0 '(2180 2260) 1 1)	;; top

;|
(if FLG
(3DFaceL (mapcar '(lambda (a b) (append a (list b)) ) (reverse (list pt3 pt3 pt2 pt2 )) '(0.0 2800.0 2800.0 0.0) ) "0")
(3DFaceL (mapcar '(lambda (a b) (append a (list b)) ) (list pt3 pt3 pt2 pt2 ) '(0.0 2800.0 2800.0 0.0) ) "0")
)
|;
)

;;; __________________________________________
;;;; ================================ DOORS_WG
( (or (wcmatch MTN "*_ *") (wcmatch MTN "*_ *"))
(setq dbx (cons (list pt1 pt2 2) dbx)) (DOR pt1 pt2 lwt 2) )

( (wcmatch MTN "*_ *") (setq dbx (cons (list pt1 pt2 3) dbx)) (WDOOR pt1 pt2 lwt 3) )
(t nil)
)	;; cond

)


));; if 1DSETUP


;; ===================================
;; ----------------- COLLECTING Plines
;; ===================================

(setq DB nil)
(setvar "CLAYER" "1D_WALL3D")

;;; ======================= First is projective
	;;(setq lw (vlax-ename->vla-object MT))  MT (car DB)
	;;(sssetfirst nil (ssadd PR (ssadd)))

;;; ======================= absorbing others
(princ "\n == COLORED POLYLINES:")
(sssetfirst nil (ssadd) )
(setq vsss (ssget '((0 . "LWPOLYLINE"))) id -1)

(if vsss
(progn
(repeat (sslength vsss)
(if (not (ssmemb (ssname vsss (setq id (1+ id ) ))  sq) )   (setq DB (cons (ssname vsss id) DB) ) )

) 		
(foreach n DB
(setq clr (1d_axprop n 'color))
(setq l (LM:LWVertices (entget n)))
(setq l (mapcar '(lambda (q) (vcget "0" lwt (trans q 1 0) ) ) l) )
(setq l (reverse (cdr (reverse (mapcar 'list l (append (cdr l) (list (car l))) )) )))
(setq DBX (append (mapcar '(lambda (q) 

(cond 
( (= clr 1) (FWIN (nth 0 q) (nth 1 q) lwt 1) )
( (= clr 2) (DOR (nth 0 q) (nth 1 q) lwt 2) )
( (= clr 3) (WDOOR (nth 0 q) (nth 1 q) lwt 3) )
( (= clr 4) (WBALC (nth 0 q) (nth 1 q) lwt 4) )

( (<= 10 clr 49) (FWIN (nth 0 q) (nth 1 q) lwt clr) )
( (<=  50 clr 89) (DOR (nth 0 q) (nth 1 q) lwt clr) )
( (<= 90 clr 129) (WDOOR (nth 0 q) (nth 1 q) lwt clr) )
( (<= 130 clr 159) (WBALC (nth 0 q) (nth 1 q) lwt clr ) )



);; cond
(append q (list clr ) )
)  l) DBX)) ;;; for every segment

)
))

;; ===================================
;; ------------------ FIXING DBX Base
;; ===================================

;;; ======================= reversing local points to lw derection
(setq dbx (mapcar '(lambda (q) (if (< (vcget "01" lwt (nth 1 q)) (vcget "01" lwt (nth 0 q)) ) 
(list (nth 1 q) (nth 0 q) (nth 2 q)) q ) ) dbx) )

;;; ======================= accending sort
(setq dbx (vl-sort dbx '(lambda (a b) (< (vcget "1" lwt (nth 0 a)) (vcget "1" lwt (nth 0 b)) ) ) ) )
;;; ======================= distance get
(setq dbx (mapcar '(lambda (q) (list (list (fix (vcget "2" lwt (nth 0 q)) ) (fix (vcget "2" lwt (nth 1 q))) ) (nth 2 q)) ) dbx) )

(setq qq dbx)		;; TRUEFILTER
(setq qq (mapcar '(lambda (q) (list (caar q) (cadr q))) qq) )	;; TRUEFILTER

(setq l (vl-sort (apply 'append (mapcar 'car dbx) )'<) )
(setq l (reverse (cdr (reverse (mapcar 'list l (append (cdr l) (list (car l))) )) ))  )
(setq db (cdar dbx))
(setq l (mapcar '(lambda (q) (if (assoc q dbx) (assoc q dbx) (append (list q) db) ) ) l))
(setq l (mapcar '(lambda (q) (if (setq mt (assoc (caar q) qq)) (list (car q) (cadr mt) )  q) ) l))	;; TRUEFILTER

;;;; ===================== FINAL CONVERT
(setq dbx (mapcar '(lambda (q)
(list (vcget "4" lwt (nth 0 (car q))) (vcget "4" lwt (nth 1 (car q))) (nth 1 q))) l) )


;; ===================================
;; ---------------- BUILDING DBX WALLS Base
;; ===================================
(setq DB (vl-sort (remove-doubles (LM:Flatten (mapcar 'cdr DS)))  '<) )
(setvar "CLAYER" "1D_WALL3D")
(setq id (getvar "CLAYER"))



(foreach n dbx
(setq p1 (nth 0 n) p2 (nth 1 n) cl (cdr (assoc (nth 2 n) DS) ) )
(setq dl (mapcar '(lambda (a b) (if b (list a b)) ) db (cdr db)) )
(setq cl (apply 'append (mapcar '(lambda (q) (1D_range q db) ) cl)) ) ;;; verticals linear subdiv
(setq dl (vl-remove-if 'null (mapcar '(lambda (q) (if (member q  cl ) nil q) ) dl)))

(mapcar '(lambda (q)
;;(3DFaceD (replacenth p1 2 (nth 0 q)) (replacenth p1 2 (nth 1 q)) (replacenth p2 2 (nth 1 q)) (replacenth p2 2 (nth 0 q)) id )
(3DFaceD (replacenth p2 2 (nth 0 q)) (replacenth p2 2 (nth 1 q)) (replacenth p1 2 (nth 1 q)) (replacenth p1 2 (nth 0 q))    id )
(setq vs (ssadd (entlast) vs))
) cl
)




(cond 
( (eq (last n) 4) (setq 1DPLAY "1D_VOID3DBW"))
( (eq (last n) 2) (setq 1DPLAY "1D_VOID3DD"))
(t (setq 1DPLAY "1D_VOID3D")) 
)

(mapcar '(lambda (q)
(3DFaceD (replacenth p1 2 (nth 0 q)) (replacenth p1 2 (nth 1 q)) (replacenth p2 2 (nth 1 q)) (replacenth p2 2 (nth 0 q)) 1DPLAY )
;(setq vs (ssadd (entlast) vs))
) dl
)




) ;; foreach


;; winwall
;;(foreach n dbx (if (= (last n) 256 ) (mapcar '(lambda (q) (1D_FRAME (nth 1 n) (nth 0 n) q 80.0) ) cl) ) )
;;(foreach n dbx  (command "_POINT" (nth 0 n)) (command "_LINE" (nth 0 n) (nth 1 n) "") (vla-put-color (vlax-ename->vla-object (entlast)) (nth 2 n) ) )
(setvar "OSMODE" OSM)
(mapcar (function (lambda (x) (ssadd x Vs))) (EntNext_to_End eLst))
(VS777)


(princ "\n=====")
(princ "\n=====")
(princ "\n===== DONE")

) ;;;1DWG










(defun C:1DBQ (/ ;; new balcony 17:35 13.05.2015
A ANG B CN DB DST FLG GAP GM ID L LSF LX MSH MX OFX OSM PT1 PT11 PT2 PT22 PTA PTB Q RANG VL
)	;;; RAILING BALCONY IMPOST
;; F= 1dper 3DFaceD 1D_HIMP 1D_VIMP
;;15:49 13.04.2015 -12.0 for balconies

;[
(if (= 0 (getvar "GRIDMODE")) (setq GM 0) (setq GM nil))
(princ "\n[GRID compartible function (F7)]")
(if GM (princ "\n>>>") (princ "\n   "))
(princ "[OFF]  = Solid winglass")
(if GM (princ "\n   ") (princ "\n>>>"))
(princ "[ON]   = Separated winglass")
;]

(setq HRAD 60.0)

(setq OFX 105 vss (ssadd) MX (entlast))



(defun 1D_BRAIL (pt1 pt2 LL HRAD / VM VL FLX)
(setq VM 45.0 VL (/ HRAD 2.0))
(1D:LAYDEF "1D_WALL3D_BRC" '(22 "Continuous" 0.15) 1 1)

(setq ang (angle pt1 pt2) DST (distance pt1 pt2) DB nil MSH 80.0 )




(setq GAP (getstring "\n Offset [1 = double left, 2 = double right, 12 = double both, ` = none]:"))
(if (wcmatch GAP "*1*") (setq pt1 (1dper pt1 OFX ang) ) )
(if (wcmatch GAP "*2*") (setq pt2 (1dper pt2 (- OFX) ang) ) )
(if (wcmatch GAP (strcat (chr 96) ) ) (setq FLX 1 ) (setq FLX nil) )
(SETQ pt1 (LIST (nth 0 pt1 ) (nth 1 pt1 ) 0.0 ) pt2 (LIST (nth 0 pt2 ) (nth 1 pt2 ) 0.0) )
(setq DST (distance pt1 pt2))
(setvar "CLAYER" "1D_BFRAME3D")

;; ============================== COUNT
(if (> DST 1200.0) 
	(progn 
	(setq CN (- (fix (/ DST LL) ) 1) )
	(setq LX (/ (- DST (* CN LL) ) 2.0) )
	(if (< LX 555.0) (setq CN (1- CN) LX (/ (- DST (* CN LL) ) 2.0) ))	;; control 505
	)
	(setq CN 0 LX nil)
)

;; ======================= VERTICALS
;; form flat list
(setq DB nil) (repeat CN (setq DB (cons ll DB))) (setq DB (if LX (cons LX DB)) )
(if (wcmatch GAP "*1*") (1D_VIMP (1dper pt1 (- OFX) ang) VM '(0.0 2800.0) ang '(0 . 0)) )
(if (wcmatch GAP "*2*") (1D_VIMP (1dper pt2 OFX ang) VM '(0.0 2800.0) ang '(0 . 0)) )
(setq id 0 DB (mapcar '(lambda (q) (setq id (+ id q))) DB))
(mapcar '(lambda (q) (1D_VIMP (1dper pt1 q ang) VL '(0.0 2800.0) ang '(0 . 0)) ) DB)	;; BUILD MID
(if (and (not FLG) (not FLX)) (1D_VIMP pt1 VM '(0.0 2800.0) ang '(0 . 0)) )
(if (not FLX) (1D_VIMP pt2 VM '(0.0 2800.0) ang '(0 . 0)))	;; BUILD SIDE
;; ======================= HORIZONTALS
(mapcar '(lambda (q) (1D_HIMP pt1 pt2 (+ (/ HRAD 2) 10) q 1 2) ) '((0.0 50.0) (700.0 750.0)   (1000.0 1100.0) (2590.0 2640.0) ))	;(800.0 850.0) (1150.0 1250.0)
;;(1D_HIMP pt1 pt2 30.0 '(800.0 850.0) 1 2)

(if FLX (setq FLG nil) (setq FLG 1) )

;; ======================= GLASS

(setq rang (angle pt1 pt2) l db)

(setq l (cons (- VM VL) l))
(setq l (reverse (cons (- DST (- VM VL))  (reverse l))))

(setq l (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l)))
(setq lsf l)
 ;; ================================== shrink
(setq l (mapcar '(lambda (q) (list (1dper pt1 (+ (nth 0 q) VL) rang ) (1dper pt1 (- (nth 1 q) VL) rang )  )) l))
 ;; ================================== out
(setq lsf (mapcar '(lambda (q) (list
(1dper  (1dper pt1 (+ (nth 0 q) VL) rang ) 0.0  (- rang (/ pi 2))) ;; 12.0
(1dper  (1dper pt1 (- (nth 1 q) VL) rang ) 0.0 (- rang (/ pi 2)))  ;; 12.0
 )) lsf ))


(if (not GM)
(progn
(foreach q lsf (3DFaceL (mapcar '(lambda (a b) (append (nth a q) (list b) ) ) '(0 0 1 1 ) '(0.0 800.0 800.0 0.0) ) "1D_BDOWN3D") )
(foreach q l (3DFaceL (mapcar '(lambda (a b) (append (nth a q) (list b) ) ) '(0 0 1 1 ) '(800.0 2640.0 2640.0 800.0) ) "1D_BGLASS3D") )
)


(progn

(setq pt11  (append (1dper pt1 0.0 (- rang (/ pi 2))) (list (last pt1)) ) pt22  (append (1dper pt2 0.0 (- rang (/ pi 2))) (list (last pt2)) )); 12.0

	(mapcar '(lambda (q)
	(3DFaceD (replacenth pt11 2 (nth 0 q)) (replacenth pt11 2 (nth 1 q)) (replacenth pt22 2 (nth 1 q)) (replacenth pt22 2 (nth 0 q)) "1D_BDOWN3D" )
	) '((0.0 700.0))
	)

	(mapcar '(lambda (q)
	(3DFaceD (replacenth pt1 2 (nth 0 q)) (replacenth pt1 2 (nth 1 q)) (replacenth pt2 2 (nth 1 q)) (replacenth pt2 2 (nth 0 q)) "1D_BGLASS3D" )
	) '((700.0 2640.0))
	)
)
)




;; ======================= BRICK stripes

(setvar "CLAYER" "1D_WALL3D_BRC")

(setq OSM (getvar "OSMODE") )
(setvar "OSMODE" 0)
(setq pt11 (1dper pt1 63.0 (- rang (/ pi 2))) )
(setq pt22 (1dper pt2 63.0 (- rang (/ pi 2)) ) )
(command "_LINE" pt11 pt22 "")
(setq vss (ssadd (entlast) vss))
(setvar "OSMODE" OSM)

(setvar "CLAYER" "1D_BFRAME3D_b")



;; ======================= MISC metal STRIPES
(setq pta (1dper (1dper pt1 70.0 ang) 100.0 (+ ang (/ pi 2))) )
(setq ptb (1dper (1dper pt2 70.0 (-  ang pi )) 100.0 (+ ang (/ pi 2))) )

(mapcar '(lambda (q) (1D_HIMP pta ptb 15.0 q 2 2) )
'((150 170) (320 340) (490 510) (660 680) (810 830) (960 980) ) )

) ;; defun 

(1D:LAYDEF "0" '(7 "Continuous" 0.1) 1 0)
(1D:LAYDEF "1D_BFRAME3D" '(8 "Continuous" 0.15) 1 0)
(1D:LAYDEF "1D_BFRAME3D_b" '(182 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_BGLASS3D" '(141 "Continuous" 0.15) 1 1)
(1D:LAYDEF "1D_BDOWN3D" '(120 "Continuous" 0.15) 1 1)

(princ "\n Getpoint ")
(if $$pt (princ "[Last]"))
(setq pt1 (getpoint) )
(if (and $$pt (not pt1)) (setq pt1 $$pt) )


(princ "\nSize of mid windows <") (if $$VAL (princ $$VAL) (princ "720.0")) (princ ">:")
(setq LL (getreal))
(if (< LL 400.0)  (if $$VAL (setq LL $$VAL) (setq LL 720.0) ))
(if (not LL)
(if (not $$VAL) (setq LL 720.0) (setq LL (+ $$VAL HRAD)) )
(setq LL (+ LL HRAD)) ;; correct!!!
)

(setq $$VAL (- LL HRAD))
(setq FLG nil)
(while pt1
(if (not pt1) (command "_PEDIT" "_M" vss "" "_J" "_J" "_E" "1200" "") )
(setq pt2 (getpoint pt1) )
(if (not pt2) (command "_PEDIT" "_M" vss "" "_J" "_J" "_E" "1200" "") )
(1D_BRAIL pt1 pt2 LL HRAD)
(setq pt1 pt2)
(if pt2 (setq $$pt pt2)



)


)
(princ)
) ;; 1DBQ






(defun C:1DRQ
(/  ;;railing line support 14:38 27.10.2014
A ANG B CN DB DDH DH DST DX FLG HG HGT ID LVL LX MT N Q RANG RL RM TH
)

(defun LM:GroupByNum ( l n / r ) (if l (cons (reverse (repeat n (setq r (cons (car l) r) l (cdr l)) r)) (LM:GroupByNum l n)) ) )

;(1D:LAYDEF "1D_BFRAME3D" '(8 "Continuous" 0.15) 1 0)
(1D:LAYDEF "1D_BFRAME3D_v" '(150 "Continuous" 0.5) 1 1)

(setq ss (ssget '((0 . "POLYLINE,LWPOLYLINE,LINE"))) id -1 dbx nil FLG nil)
(repeat (sslength ss) 
(setq lw (vlax-ename->vla-object (setq MT (ssname ss (setq id (1+ id)) )) ) )
(cond 
((= (1d_axprop lw 'ObjectName) "AcDb3dPolyline")(setq l (LM:GroupByNum (1d_axprop lw 'Coordinates) 3) ) )
((= (1d_axprop lw 'ObjectName) "AcDbLine") (setq l (list (1d_axprop lw 'StartPoint) (1d_axprop lw 'EndPoint)) ) )
(t (setq l (mapcar '(lambda (q) (append q (list (1d_axprop MT 'Elevation))) ) (LM:LWVertices (entget MT)) ) ))
)

(if (= (1d_axprop lw 'ObjectName) "AcDbLine")
(setq l (list l))
(if
   (=   (vlax-get-property lw    'Closed) :vlax-false)
(setq l (mapcar '(lambda (a b) (if b (list a b)) ) l (cdr l) ) )	;; RECYCLE open
(setq l (mapcar 'list l (append (cdr l) (list (car l))) ))		;; RECYCLE closed
)
)

(setq DBX (cons l DBX))
)

(setq LL (getreal "\n Distance between rails <1200>" ))
(if (not LL) (setq LL 1200.0))

(setq
RM 25.0				;; radius main
RL 17.0 			;; radius misc
TH 80.0				;; Thickness
;;LL 1200.0			;; misc dist
;;LL 200.0			;; misc dist
;;HG '((840.0 . 20.0) (980.0 . 20.0) (1200.0 . 80.0))	;; Height // thick
HG '((900.0 . 30.0) (1100.0 . 30.0) (1500.0 . 80.0))	;; Height // thick
)

(princ "\n")(princ HG)(getstring "  ok?")

(FOREACH L DBX
(foreach n l
;; (setq l (car DBX) n (car l))
;; 
(setq prx (mapcar '(lambda (q) (list (car q) (cadr q)) ) n))	;;; projections
(setq DST (apply 'distance prx ) )
(setq DH (- (nth 2 (car n)) (nth 2 (cadr n)) ))
(setq DX (/ DH DST))
(setq DH (nth 2 (car n)))

;; ============================== COUNT
(if (> DST LL)
	(progn
	(setq CN (- (fix (/ DST LL) ) 1) )
	(setq LX (/ (- DST (* CN LL) ) 2.0) )
	(if (< LX (/ LL 3) ) (setq CN (1- CN) LX (/ (- DST (* CN LL) ) 2.0) ))	;; control 505
	)
	(setq CN 0 LX nil)
)

;; ============================ BUILD
(setq DB nil) 
(setq id 0 HGT (- (car (last HG)) (/ TH 2.0) ) )
(setq ang (angle (car n) (cadr n)))
(repeat CN (setq DB (cons LL DB) )) (if LX (setq DB (cons LX DB)))
(setvar "CLAYER" "1D_BFRAME3D_v")
;;(setvar "CLAYER" "1D_BFRAME3D_v")
;;; __________________________________________ MISC VERTICALS
;;; =========================================================


(mapcar
'(lambda (q) (setq id (+ q id) DDH (- DH (* id DX) ))
(1D_VIMP (1dper (car n) id ang ) RL (list DDH (+ DDH HGT)) ang '(2 . 3))
)
DB)
(setvar "CLAYER" "1D_BFRAME3D_v")

;;; _____________________________________________ HORIZONTALS
;;; =========================================================

(setq DH (* TH (/ (distance (car n) (cadr n) ) DST ))) ;;++ height delta
(setq rang (angle (car prx) (cadr prx)))
(foreach lvl HG
(setq prx (reverse (mapcar '(lambda (q) (mapcar '+ q (list 0.0 0.0 (car lvl)) )  ) n ) ))				;;; reverse for correct normals
(if  (= lvl (last HG))
(progn
;;; ====================  FAT FRAME
(3DFaceL  (setq prx (append (mapcar '(lambda (q) (1dperZ q (/ (cdr lvl) 2.0) (+ rang (/ pi 2))) ) prx)			;; TOP FACE
(mapcar '(lambda (q) (1dperZ q (/ (cdr lvl) 2.0) (- rang (/ pi 2))) ) (reverse prx))) )  nil)
(3DFaceL (setq DST (mapcar '(lambda (q) (mapcar '- q (list 0.0 0.0 (cdr lvl)) )  ) prx )) nil) 				;; BOTTOM FACE
(mapcar '(lambda (q) (3dfacel q nil)) (setq prx (mapcar 'append (LM:GroupByNum prx 2) (mapcar 'reverse (LM:GroupByNum DST 2) ) ))  )	;; SIDE FACES
)
;;; ==================== THIN FRAME
(3DFaceL 
(list (car prx) (mapcar '- (car prx) (list 0.0 0.0 (cdr lvl) ) ) (mapcar '- (cadr prx) (list 0.0 0.0 (cdr lvl) ) ) (cadr prx) )
nil
)) ;;if

);; foreach

;;; __________________________________________ MAIN VERTICALS
;;; =========================================================
(if FLG (setq n (cdr n)))
(mapcar '(lambda (q) (1D_VIMP q RM (list (nth 2 q) (+ (nth 2 q) (car (last HG)) )) ang '(0 . 0)) ) n)
(setq FLG 1)

) ;; foreach
(setq FLG NIL)
;;(1D_VIMP pt1 rd hd rang '(0 . 0))
) ;; FOREACH L dbx
(princ)
) ;;; defun 1DRQ







(defun C:RV ( / int:i e1 ed list:pt *error* *pl-activedoc*)
(setq *error* pltool-err)
  (or *pl-activedoc*
       (setq *pl-activedoc* (vla-get-activedocument (vlax-get-acad-object))))
  (vla-endundomark *pl-activedoc*)
  (vla-startundomark *pl-activedoc*)
    (pl-error-save-sysvar
      (list '("osmode")	'("CLAYER") '("QAFLAGS" 0) '("CMDECHO" 0)))

  (princ "\nChoose Polylines, Splines and Lines for a reverser  ")

  (setq PICK1 nil
	PICK1 (ssget "_:L" '((0 . "*LINE"))))
  (setq int:i 0)
  (while (and PICK1 (setq e1 (ssname PICK1 int:i)))
    (setq list:pt nil)
    (setq ed (entget e1))
    (cond ((= (cdr(assoc 0 ed)) "LINE")
	   (setq e1 (vlax-ename->vla-object e1))
	   (setq list:pt (mapcar '(lambda (x) (vlax-get e1 x))
			   '(StartPoint EndPoint))
		 list:pt  (reverse list:pt))
	   (vla-put-StartPoint e1 (vlax-3d-point (car list:pt)))
	   (vla-put-EndPoint e1 (vlax-3d-point (cadr list:pt)))
	  )
	  ((= (cdr(assoc 0 ed)) "LWPOLYLINE")
	   (lib:plineLW-reverse e1)
          ;(entdel e1)
	   )
	  ((= (cdr(assoc 0 ed)) "SPLINE")(vla-reverse (vlax-ename->vla-object e1))) 
  	  ((= (cdr(assoc 0 ed)) "POLYLINE")
	   (ru-geom-polyline-revers e1))
	  (t nil)
    )	  
    (setq int:i (1+ int:i))
  )

        (princ (strcat "\nThe reverser of " (itoa int:i) " object is executed"))
  (setq PICK1 nil)
(pl-error-restore-sysvar)
(vla-endundomark *pl-activedoc*)
  (princ)
);defun



(setq $$TEXTS 2.0)

(defun C:D2 (/ MT lw pt1 MTS FLG)


(setq MT (entsel) 
;;(entsel "\n BLOCK/XREF_path <DWGpath>: ")
;;(ssget "_+.:S" '((0 . "INSERT")) )
FLG nil
)


(if MT
(progn
(setq lw (vlax-ename->vla-object (car MT)) )

(if (= (cdr (assoc 0 (entget (car MT)))) "INSERT")
(progn
(if (vlax-Property-Available-p lw 'Path)
(progn
(setq FLG 1)
(princ "\n == XREF path:  ") (princ (setq MT (vlax-get-Property lw 'Path)))
(princ "\n == XREF name:  ") (princ (setq MTS (vlax-get-Property lw 'Name)))
)
(progn (princ "\n == BLOCK name:  ") (princ (setq MT (vlax-get-Property lw 'Effectivename))))
)
)
	(if (= (cdr (assoc 0 (entget (car MT)))) "IMAGE")
	(progn 	(progn (princ "\n == IMAGE path:  ") (princ (setq MT (vlax-get-Property lw 'ImageFile) ) )) )
	(setq MT (vlax-get-Property lw 'Layer) )
	)

)

	
)


(progn 
(if (= (setq MTS (getvar "SSFOUND") ) "")
(princ "\n == DWG path <No Sheetset link found>:  ")
(princ "\n == DWG path <Sheetset>:  ")
)
(princ (setq MT (strcat (getvar "DWGPREFIX") (getvar "DWGNAME") )))
)
)

(while (wcmatch MT (STRCAT "*" (CHR 92) (CHR 92) "*")) (setq MT (vl-string-subst (CHR 92) (STRCAT (CHR 92) (CHR 92))   MT)) )
(setq MT	(vl-string-subst  "Y:" "\\ru948.gs.corp.int\\Data" MT) )

(if MTS (progn
(while (wcmatch MTS (STRCAT "*" (CHR 92) (CHR 92) "*")) (setq MTS (vl-string-subst (CHR 92) (STRCAT (CHR 92) (CHR 92)) MTS)) )
(setq MTS	(vl-string-subst  "Y:" "\\ru948.gs.corp.int\\Data" MTS) )
))


(setq pt1   (getpoint) )
(if pt1
(progn
(entmake
    (list
      '(0 . "TEXT")
       (cons 10 (trans pt1 1 0 )) 
       ;(cons 11 (trans pt1 1 0 ))
      (cons 40 $$TEXTSIZER ) 
       (cons 1 MT) 
      '(50 . 0)
      '(41 . 0.8)
      '(51 . 0.26)
       (cons 7 (getvar "TEXTSTYLE")) 
      '(71 . 0) 
      '(72 . 0) 
      '(73 . 0) 
    )
)

(if MTS

(if (= MTS "")
(princ "\n == No Sheetset link found" )


(progn 
(if FLG 
(princ )
(princ (strcat "\n == Sheetset location: " MTS) )
)
;(setq pt1   (getpoint) )
(setq pt1 (list (nth 0 pt1) (- (nth 1 pt1) 500.0 0.0)))
(if (and pt1 (/= MTS "") )
(entmake
    (list
      '(0 . "TEXT")
       (cons 10 (trans pt1 1 0 )) 
       ;(cons 11 (trans pt1 1 0 ))
      '(40 . 100.0) 
       (cons 1 MTS) 
      '(50 . 0)
      '(41 . 0.8)
      '(51 . 0.26)
       (cons 7 (getvar "TEXTSTYLE")) 
      '(71 . 0) 
      '(72 . 0) 
      '(73 . 0) 
    )
)

)))

)

;|
(entmake (mapcar 'cons 
'(0 10 40 1 50 41 51 7 71 72 73)
(mapcar 'eval '( "TEXT" (trans pt1 1 0 ) 100.0  MTS  0 0.8 0.26 (getvar "TEXTSTYLE") 0 0 0)) )
)
|;

)	;;pt1
)	;; global if

(princ)
)



(princ "\n ....Loaded. Press F2 for commands list.")
(princ)

;http://www.theswamp.org/index.php?topic=11561.0
;http://www.theswamp.org/index.php?topic=13526.0
;;(defun ucszdir ()(trans '(0 0 1) 1 0 T))

